
//
// Real Time Linux Example of Analog inputs and outputs
//
// Bill Crum
// New Mexico Tech
// tobor@nmt.edu
// This is only an example. No guarantees are expressed.
//


#include "analogs.h"


// The scope is check the real-time fifo every 50 ms.
// The scope is not in real-time, it displays data that was collected
// from real-time tasks.

AnalogTimer::AnalogTimer( QWidget *parent ) : QTimer( parent ), interval( 1000)
{
   initMetaObject();
   start( interval );
}

// Pause button to look a signal


void AnalogTimer::pause( bool stopIt )
{
    if ( stopIt )
       stop();
  else
      start( interval );
}


//
// Creates the Scope widget
//

Analogs::Analogs( QWidget *parent, const char *name )
    : QFrame( parent, name )
{
    initMetaObject();				// initialize meta object

    timer = new AnalogTimer(this);

        QLabel *msgLabel0 = new QLabel( this, "msgLabel0" );
    msgLabel0->setText( "Analog Inputs:" );
    msgLabel0->setAlignment( AlignRight|AlignVCenter );
    msgLabel0->setGeometry( 10,5, 100, 20 );

        QLabel *msgLabel1 = new QLabel( this, "msgLabel1" );
    msgLabel1->setText( "1:" );
    msgLabel1->setAlignment( AlignRight|AlignVCenter );
    msgLabel1->setGeometry( 5,25, 20, 20 );

        QLabel *msgLabel2 = new QLabel( this, "msgLabel2" );
    msgLabel2->setText( "2:" );
    msgLabel2->setAlignment( AlignRight|AlignVCenter );
    msgLabel2->setGeometry( 5,55, 20, 20 );

        QLabel *msgLabel3 = new QLabel( this, "msgLabel3" );
    msgLabel3->setText( "3:" );
    msgLabel3->setAlignment( AlignRight|AlignVCenter );
    msgLabel3->setGeometry( 5,85, 20, 20 );

        QLabel *msgLabel4 = new QLabel( this, "msgLabel4" );
    msgLabel4->setText( "4:" );
    msgLabel4->setAlignment( AlignRight|AlignVCenter );
    msgLabel4->setGeometry( 5,115, 20, 20 );

    QString s = " 0.00 v";

    analog1 = new QLabel(this, "msglabel");;	
    analog1->setAlignment( AlignRight|AlignVCenter );
    analog1->setFont( QFont("times",18) );
    analog1->setFrameStyle( QFrame::Panel | QFrame::Sunken );

    analog1->setGeometry(30,25,80,25);
    analog1->setText( s );                          // set LCD number/text

    analog2 = new QLabel(this, "msglabel2");;	
    analog2->setAlignment( AlignRight|AlignVCenter );
    analog2->setFont( QFont("times",18) );
    analog2->setFrameStyle( QFrame::Panel | QFrame::Sunken );

    analog2->setGeometry(30,55,80,25);
    s = " 0.00 v";
    analog2->setText( s );                          // set LCD number/text


    analog3 = new QLabel(this, "msglabel3");;	
    analog3->setAlignment( AlignRight|AlignVCenter );
    analog3->setFont( QFont("times",18) );
    analog3->setFrameStyle( QFrame::Panel | QFrame::Sunken );

    analog3->setGeometry(30,85,80,25);
    analog3->setText( s );                          // set LCD number/text

    analog4 = new QLabel(this, "msglabel4");;	
    analog4->setAlignment( AlignRight|AlignVCenter );
    analog4->setFont( QFont("times",18) );
    analog4->setFrameStyle( QFrame::Panel | QFrame::Sunken );

    analog4->setGeometry(30,115,80,25);
    s = " 0.00 v";
    analog4->setText( s );                          // set LCD number/text



    x = 0;
    QLabel *msgLabel7 = new QLabel( this, "msgLabel7" );
    msgLabel7->setText( "Analog Output 2" );
    msgLabel7->setAlignment( AlignRight|AlignVCenter );
    msgLabel7->setGeometry( 270,5, 100, 20 );

    msgLabel8 = new QLabel( this, "msgLabel8" );
    msgLabel8->setAlignment( AlignRight|AlignVCenter );
    msgLabel8->setFont( QFont("times",18) );
    msgLabel8->setText( "  0.00 volts" );
    msgLabel8->setGeometry( 270,50, 100, 20 );


    QRadioButton *rb;
    QButtonGroup *bg = new QButtonGroup( this, "AO1" );
    bg->setTitle( "Analog Output 1" );
    rb = new QRadioButton( bg );
    rb->setText( "Square Wave" );
    rb->setGeometry( 10, 25, 100,30 );
    rb = new QRadioButton( bg );
    rb->setText( "Sine Wave" );
    rb->setGeometry( 10, 55, 100,30 );
    rb = new QRadioButton( bg );
    rb->setText( "Sawtooth Wave" );
    rb->setGeometry( 10, 85, 100,30 );
    bg->setGeometry( 130, 10, 120, 130 );
    connect( bg, SIGNAL(clicked(int)), SLOT(radioButtonClicked(int)) );


    QScrollBar *sb = new QScrollBar( -50, 50, 1, 1, 0, QScrollBar::Horizontal,this, "scrollbar");
    sb->setGeometry ( 270, 100, 110, 16 );

    connect( sb, SIGNAL(valueChanged(int)), SLOT(scrollBarValueChanged(int)) );

    setLineWidth( 2 );				// set frame line width

    resize( 400, 300 );
    if ((ad1 = open("/dev/rtf2", O_RDONLY)) < 0) {
      fprintf(stderr, "Error opening /dev/rtf2\n");
      exit(1);
    }
    if ((ad2 = open("/dev/rtf3", O_RDONLY)) < 0) {
      fprintf(stderr, "Error opening /dev/rtf3\n");
      exit(1);
    }
    if ((ad3 = open("/dev/rtf4", O_RDONLY)) < 0) {
      fprintf(stderr, "Error opening /dev/rtf4\n");
      exit(1);
    }
    if ((ad4 = open("/dev/rtf5", O_RDONLY)) < 0) {
      fprintf(stderr, "Error opening /dev/rtf5\n");
      exit(1);
    }
    if ((da0 = open("/dev/rtf6", O_WRONLY)) < 0) {
      fprintf(stderr, "Error opening /dev/rtf6\n");
      exit(1);
    }
    if ((da1 = open("/dev/rtf7", O_WRONLY)) < 0) {
      fprintf(stderr, "Error opening /dev/rtf7\n");
      exit(1);
    }



    setFrameStyle( QFrame::Box | QFrame::Raised );
    connect( timer, SIGNAL(timeout()), SLOT(showSignal()) );
 
}


void Analogs::radioButtonClicked( int value )
{
  char outbyte;

  if (value == 0)
    value = 3;
  outbyte = (char) value;
    write(da0, &outbyte, 1);
}

void Analogs::showSignal( )
{
  unsigned int y;
  QString str;
  double newvalue;
  float resolution = .000306;

  y = 0;


  if (read (ad1, inbyte, 2) > 0)
    { 
      ;
      y = (unsigned int) inbyte[1];
      y = y << 8;
      y &= 0xff00;
      y += (unsigned int) inbyte[0];
      y &= 0xffff;
      
      newvalue = -10.00;
      newvalue += (resolution * ((float) y)) ;
      
      
      str.sprintf( "%.2f v", newvalue );
      analog1->setText( str );                    

    }

  if (read (ad2, inbyte, 2) > 0)
    {
      y = (unsigned int) inbyte[1];
      y = y << 8;
      y &= 0xffff;
      y += (unsigned int) inbyte[0];
      y &= 0xffe0;
      
      newvalue = -10.00;
      newvalue += (resolution * ((float) y)) ;


      str.sprintf( "%.2f v", newvalue );
      analog2->setText( str );                    
    }

  if (read (ad3, inbyte, 2) > 0)
    {
      y = (unsigned int) inbyte[1];
      y = y << 8;
      y &= 0xffff;
      y += (unsigned int) inbyte[0];
      y &= 0xffe0;
      
      newvalue = -10.00;
      newvalue += (resolution * ((float) y)) ;


      str.sprintf( "%.2f v", newvalue );
      analog3->setText( str );                    
    }
  if (read (ad4, inbyte, 2) > 0)
    {
      y = (unsigned int) inbyte[1];
      y = y << 8;
      y &= 0xfff8;
      y += (unsigned int) inbyte[0];
      y &= 0xffff;
      
      newvalue = -10.00;
      newvalue += (resolution * ((float) y)) ;


      str.sprintf( "%.2f v", newvalue );
      analog4->setText( str );                    
    }

  
}

void Analogs::scrollBarValueChanged( int value )
{
  float newvalue;
    QString str;
    float obyte;
    unsigned char lowbyte[2];
    unsigned int outbyte;

    newvalue = (float) value;

    outbyte = value + 50;
    obyte = ((float) outbyte * 653.56); 
    outbyte = (int)  obyte;

    lowbyte[0] = (unsigned char) (outbyte & 0xff);
    lowbyte[1] = (unsigned char) ((outbyte & 0xff00) >> 8);

    write(da1, lowbyte, 2);

    newvalue = newvalue / 10;
    str.sprintf( " %.2f volts", newvalue );

    msgLabel8->setText( str );


}





