/*Copyright (C) 1999 by Heinz Haeberle, heinz.haeberle@gmx.net

This is a RTLinux-module for C200-based CAN-cards. It supports now
the Phytec CAN-card on Irq 7, IO 0x100+0x120 (2 ports version)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.*/

#include "c200.h"

static unsigned char    pa_init[] = {
//        0x02,           /*  Control-Register    */
//        0x1e,           /*  Control-Register    */
        0x1a,           /*  Control-Register    */
        0x00,           /*  Acc-Code            */
        0xff,           /*  Acc-Mask            */
        0x00,           /*  Bus Timing 0   fuer 500 kBits/s       */
        0x1c,           /*  Bus Timing 1   fuer 500 kBits/s       */
        0xda,           /*  Output-Control      */
};

static bool  C200_CanStart(int BaseAdr)
{
  outb((inb(BaseAdr+C200_CTRL) & ~CR_RR),BaseAdr+C200_CTRL);   /*  Runnig-Mode setzen  */
  return ((inb(BaseAdr+C200_CTRL) & CR_RR) != 0);               /*  Gesetzter Wert zurueckliefern  */
}


static bool  C200_CanReset(int BaseAdr)
{
  outb((inb(BaseAdr+C200_CTRL) | CR_RR),BaseAdr+C200_CTRL);   /*  Reset-Zustand setzen  */
  return(!(inb(BaseAdr+C200_CTRL) & CR_RR));              /*  Gesetzter Wert zurueckliefern  */
}

static bool  C200_CanInit(int BaseAdr, const unsigned char  * pa)
{
  outb((pa[0] | CR_RR), BaseAdr+C200_CTRL);   /*  Flag's und Reset-Request im Ctrl-Reg. setzen  */

  if((inb(BaseAdr+C200_CTRL) & 0x1f) != ((pa[0] & 0x1f) | CR_RR))          /* Laesst sich der Wert nicht schreiben ?  */
    return(1);                          /*  Error: CAN laesst sich nicht initialisieren  */

  outb(pa[1],BaseAdr+C200_ACCC);           /*  Acc-Code setzen  */
  outb(pa[2],BaseAdr+C200_ACCM);           /*  Acc-Mask setzen  */
  outb(pa[3],BaseAdr+C200_BT0);            /*  Bus Timing 0 setzen  */
  outb(pa[4],BaseAdr+C200_BT1);            /*  Bus Timing 1 setzen  */
  outb(pa[5],BaseAdr+C200_OUT_CTRL);       /*  Output-Ctrl setzen  */

  return(0);                            /*  O.K.  */
}

static int CANInit(int BaseAdr, int BaudRate)
{
	int tries=100;
    switch (BaudRate ) {
    case 20 :
          {
            pa_init[3] = 0x18;    /* Bus timing 0  fuer 20 kBits/s */
            pa_init[4] = 0x1c;    /* Bus timing 1  fuer 20 kBits/s */
			break;
		  }
    case 125 :
          {
            pa_init[3] = 0x03;    /* Bus timing 0  fuer 125 kBits/s */
            pa_init[4] = 0x1c;    /* Bus timing 1  fuer 125 kBits/s */
			break;
          }
    case 250 :
		  {
			pa_init[3] = 0x01;    /* Bus timing 0  fuer 250 kBits/s */
			pa_init[4] = 0x1c;    /* Bus timing 1  fuer 250 kBits/s */
            break;
          }
    case 500 :
          {
            pa_init[3] = 0x00;    /* Bus timing 0  fuer 500 kBits/s */
            pa_init[4] = 0x1c;    /* Bus timing 1  fuer 500 kBits/s */
            break;
          }
	case 1000 :
		  {
            pa_init[3] = 0x00;    /* Bus timing 0  fuer 1000 kBits/s */
            pa_init[4] = 0x14;    /* Bus timing 1  fuer 1000 kBits/s */
            break;
          }
	default :
		return 1;
	
  }

  C200_CanReset(BaseAdr);					/* CAN-Controller reset */
  if (C200_CanInit(BaseAdr, pa_init))            /* CAN-Controller initialisieren */
	return 1;
  while((tries--) && (C200_CanStart(BaseAdr)))	            /* CAN-Controller starten */
	rtl_delay(20);  
  if (tries > 0)
  	return 0;
  return 1;	
}

static void C200_CanObjWr(int Adr, unsigned char  *src, int cnt)
{

    int i;
    for(i=0; i< cnt; i++)
        outb(src[i], Adr+i);
}

static unsigned char  C200_Transmit(int Base_Adr, int id, char dlc, const unsigned char  * p_src)
{
	/*  Ist CAN nicht sendebereit ?  */
	if((inb(Base_Adr+C200_STS) & (SR_BS | SR_TBA)) != SR_TBA)
		return(1);                  /*  Error:  CAN nicht sendebereit  */

	/*  Objekt in CAN eintragen  */
	outb((unsigned char )(id >> 3),Base_Adr+C200_TXDB1);    /*  Descriptor zusammenbauen  */
	if ((dlc <= 8) && (dlc >= 0 ))
	{
		outb((unsigned char )(((unsigned char )id << 5) | dlc),Base_Adr+C200_TXDB2);
		C200_CanObjWr(Base_Adr+C200_TXDATA0,(unsigned char  *)p_src,dlc);    /*  Datenunsigned char s kopieren  */
	}
	else /* als remote frame verschicken */
		outb((unsigned char )(((unsigned char )id << 5) | DSCR2_RTR),Base_Adr+C200_TXDB2);
	outb(CMR_TR,Base_Adr+C200_CMND);              /*  Sendeauftrag setzen  */
	return(0);                                /*  Objekt erfolgreich eingetragen  */
}

static void ReadRCVBuf(int BaseAdr, unsigned char * data) // sind immer 10 unsigned char s lang!
{
    short i;
    for (i=0; i<10;i++)
        *data++ = inb(BaseAdr+i + C200_RXDB1);
    outb(CMR_RRB,BaseAdr+C200_CMND);  /*  Receive-Buffer wieder freigeben  */
}

static int ReadFromChip(int BaseAdr, unsigned char  *b)
{
	int ret=0;
	int IRQState = inb(BaseAdr+C200_INTER);
	if (IRQState & IR_RI)	/* es wurde ein unsigned char  empfangen */
	{
		if(inb(BaseAdr+C200_STS) & SR_RBS)
		{
			ReadRCVBuf(BaseAdr, b);
			ret = 1;
		}
	}
	if (IRQState & IR_OI)
	if (inb(BaseAdr+C200_STS) & SR_DO)
	{
		outb(CMR_COS,BaseAdr+C200_CMND);  /*  Overrun Status loeschen! */
	}
	return ret;
}

