/*Copyright (C) 1999 by Heinz Haeberle, heinz.haeberle@gmx.net

This is a RTLinux-module for C200-based CAN-cards. It supports now
the Phytec CAN-card on Irq 7, IO 0x100+0x120 (2 ports version)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.*/

#include <linux/module.h>
#include <linux/kernel.h>
#include <asm/io.h>
#include <rtl.h>
#include <rtl_fifo.h>
#include <time.h>
#include <rtl_sched.h>
#include <rtl_sync.h>
#include <rtl_time.h>
#include <rtl_debug.h>
#include <string.h>
#include "c200.c"
#include "candrv.h"

#define USERSPACESENDFIFO	0
#define USERSPACERECVFIFO	1
#define INTERNALRECVFIFO	9

// the defaults are for the phytec-card with common interrupt
// on irq 7 and adr 100 and 120
static int io_addr[2] 	= {0x100,0x120};
static int irq[2]     	= {7,0};
static int bus_speed[2] = {125,125};

MODULE_PARM(io_addr, "1-2i");
MODULE_PARM(irq, "1-2i");
MODULE_PARM(bus_speed, "1-2i");

static unsigned int crvalue;

//#define RTL_DEBUG

pthread_t threadsend,threadrecv,threadtest;

CANFunction CBArray[2048*2];

int CANAddCallbackFn(int port, int id, CANFunction f)
{
	if ((id < 0) || (id > 2047))
		return -1;
	if ((port < 0) || (port > 1))
		return -1;
	rtl_critical(crvalue);
	CBArray[id + 2048*port]	= f;
	rtl_end_critical(crvalue);
	return 0;
}

int CANRemCallbackFn(int port, int id)
{
	if ((id < 0) || (id > 2047))
		return -1;
	if ((port < 0) || (port > 1))
		return -1;
	rtl_critical(crvalue);
	CBArray[id + 2048*port]	= NULL;
	rtl_end_critical(crvalue);
	return 0;
}

static int ProcessID(sCanMsg *canmsg)
{
	int id = canmsg->id;
	int ret = 0;
	if ((id < 0) || (id > 2047))
		return -1;
	rtl_critical(crvalue);
	if (CBArray[id] != NULL)
		ret = (*CBArray[id])(canmsg);
	rtl_end_critical(crvalue);
	return ret;
}

static void *recvtask(void *dummy)
{
	char b[11];
	sCanMsg canmsg;
#ifdef RTL_DEBUG			
	char Buffer[100];
	int i;
	char *ptr;
#endif	
	pthread_make_periodic_np(pthread_self(),gethrtime(),1000LL*1000LL*100LL);
	while(1)
	{
		while(rtf_get(INTERNALRECVFIFO,b,11)==11)
		{
			canmsg.id = (unsigned int)b[0];
			canmsg.id = (canmsg.id << 3) + (b[1]>>5);
			canmsg.port = (int)b[10];
			canmsg.datalen = (b[1] & ((unsigned char)0x1f));
			memcpy(&canmsg.data[0], &b[2], 8);
			rtf_put(0,&canmsg ,sizeof(canmsg));
#ifdef RTL_DEBUG			
			ptr = &Buffer[0];
			ptr += sprintf(Buffer,"%03x %i",canmsg.id,canmsg.port);
			for (i=0;i<canmsg.datalen;i++)
				ptr += sprintf(ptr," %02x",b[i+2]);
			rtl_printf("recv: %s\n",Buffer);
#endif			
			ProcessID(&canmsg);
		}
		pthread_wait_np();
	}
}

static void *sendtask(void *dummy)
{
	sCanMsg	canmsg;
	int tries,ret; 
	pthread_make_periodic_np(pthread_self(),gethrtime(),1000*10000);
	while(1)
	{
		while(1)
		{
			int i;
			ret = rtf_get(USERSPACERECVFIFO,&canmsg,sizeof(canmsg));
			if (ret != sizeof(canmsg))
				break;
#ifdef RTL_DEBUG		
			rtl_printf("send: %i %03x", canmsg.port,canmsg.id);
			if ((canmsg.datalen < 0) || (canmsg.datalen > 8))
			{
				rtl_printf("REMOTE\n");
			} 
			else
			{
				for(i=0;i<canmsg.datalen;i++)
					rtl_printf(" %02x",canmsg.data[i]);
				rtl_printf("\n");
			}
#endif
			// we try to send it for a second if it isn't working kick 
			// the message off!
			tries = 100;
			if (canmsg.port < 2)
				while ((tries-- > 0) && (C200_Transmit(io_addr[canmsg.port], canmsg.id, canmsg.datalen, &canmsg.data[0])))
					pthread_wait_np(); // maybe the transmitbuffer 
//				rtl_delay(1000);
		}
		pthread_wait_np();           
	}		
	return NULL;
}

//int CANWrite(sCanMsg *msg)
int CANWrite(int port, int id, int len, unsigned char *data)
{
	sCanMsg canmsg;
	int ret;
	canmsg.id = id; canmsg.port = port;canmsg.datalen = len;
	if ((len <=8 ) && (len > 0))
		memcpy(&canmsg.data[0],data,len);
	ret = rtf_put(USERSPACERECVFIFO,&canmsg, sizeof(canmsg));
//rtl_printf("canwrite rtf_put=%i\n",ret);
	pthread_wakeup_np (threadsend);
	return ret;
}

unsigned int intr_handler(unsigned int irq ,struct pt_regs *p) 
{
	unsigned char b[11];
	int packetArrived = 0;
//	rtl_printf("int arrived\n");
	if (io_addr[0])
	{
		while(0 != ReadFromChip(io_addr[0],&b[0]))
		{
			b[10] = 0;
			rtf_put(INTERNALRECVFIFO,b,11);
			packetArrived = 1;
		}
	}
	if (io_addr[1])
	{
		while(0 != ReadFromChip(io_addr[1],&b[0]))
		{
			b[10] = 1;
			rtf_put(INTERNALRECVFIFO,b,11);
			packetArrived = 1;
		}
	}
	rtl_hard_enable_irq(irq);
	if (packetArrived)
		pthread_wakeup_np (threadrecv);
	return 0;
}

int fifoHandler(unsigned int fifo)
{
	if (fifo == USERSPACESENDFIFO)
	{
		pthread_wakeup_np (threadsend);
		return 0;
	}
	else
		return -EINVAL;
}

static void *testtask(void *dummy)
{
    sCanMsg	canmsg;
    unsigned char b[8]={1,0};
    int tries; 
    pthread_make_periodic_np(pthread_self(),gethrtime(),10000*10000);
//    breakpoint();
    while(1)
    {
		CANWrite(0,0,2,b);
		pthread_wait_np();           
    }
}

int init_module(void)
{
	int i;
	pthread_attr_t attr;
	struct sched_param sched_param;
	int thread_status;
	if (io_addr[0] == 0x5000)	// is it a ROTEC?
		inb(0x4000);			// to open the port-area!
	if (CANInit(io_addr[0], bus_speed[0]))
	{
		rtl_printf("can't init CAN\n");
		return -1;
	}
	if (CANInit(io_addr[1], bus_speed[1]))
	{
		rtl_printf("can't init CAN\n");
		return -1;
	}
	rtf_create(USERSPACESENDFIFO,1000);
	rtf_create_handler(USERSPACESENDFIFO,fifoHandler);
	rtf_create(USERSPACERECVFIFO,1000);
	rtf_create(INTERNALRECVFIFO,1000);
	for (i = 0; i< (sizeof(irq) / sizeof(irq[0]));i++)
	{
		if (irq[i] != 0)
		{
			if (rtl_request_irq(irq[i],intr_handler))
			{
				rtl_printf("can't get Interrupt %i\n",irq[i]);
				return -1;
			}
			rtl_hard_enable_irq(irq[i]);
		}
	}
	rtl_printf("candrv on CPU %d\n",rtl_getcpuid());
	pthread_attr_init (&attr);
//	pthread_attr_setcpu_np(&attr, 0 /*!rtl_getcpuid()*/);
	sched_param.sched_priority = 1;
	pthread_attr_setschedparam (&attr, &sched_param);
	rtl_printf("About to thread create\n");
	thread_status = pthread_create (&threadsend,  &attr, sendtask, (void *)1);
	if (thread_status != 0) {
		rtl_printf("failed to create RT-thread; errno=\n", errno);
		return -1;
	} else {
 		rtl_printf("created RT-thread\n"); 
	}
	rtl_printf("About to thread create\n");
	thread_status = pthread_create (&threadrecv,  &attr, recvtask, (void *)1);
	if (thread_status != 0) {
		rtl_printf("failed to create RT-thread; errno=\n", errno);
		return -1;
	} else {
 		rtl_printf("created RT-thread\n"); 
	}
	rtl_printf("About to thread create\n");
#if 0
	thread_status = pthread_create (&threadtest,  &attr, testtask, (void *)1);
	if (thread_status != 0) {
		rtl_printf("failed to create RT-thread; errno=\n", errno);
		return -1;
	} else {
 		rtl_printf("created RT-thread\n"); 
	}
#endif
	memset(CBArray, 0, sizeof(CBArray));
	return 0;
}


void cleanup_module(void)
{
	int i;
	pthread_delete_np (threadrecv);
	for (i = 0; i< (sizeof(irq) / sizeof(irq[0]));i++)
	{
		if (irq[i] != 0)
		{
			rtl_hard_disable_irq(irq[i]);
			rtl_free_irq(irq[i]);
		}
	}
	pthread_delete_np (threadsend);
//	pthread_delete_np (threadtest);
	rtf_destroy(USERSPACESENDFIFO);
	rtf_destroy(USERSPACERECVFIFO);
	rtf_destroy(INTERNALRECVFIFO);
	rtl_printf ("Removing module on CPU %d\n", rtl_getcpuid());
}
