#include <stdio.h>
#include <sys/ioctl.h>
#include <linux/isdn.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <pthread.h>
#include "candrv.h"

#define rtl_critical(a)
#define rtl_end_critical(a)

CANFunction CBArray[2048*2];

int CANAddCallbackFn(int port, int id, CANFunction f)
{
	if ((id < 0) || (id > 2047))
		return -1;
	if ((port < 0) || (port > 1))
		return -1;
	rtl_critical(crvalue);
	CBArray[id + 2048*port]	= f;
	rtl_end_critical(crvalue);
	return 0;
}

int CANRemCallbackFn(int port, int id)
{
	if ((id < 0) || (id > 2047))
		return -1;
	if ((port < 0) || (port > 1))
		return -1;
	rtl_critical(crvalue);
	CBArray[id + 2048*port]	= NULL;
	rtl_end_critical(crvalue);
	return 0;
}

static int ProcessID(sCanMsg *canmsg)
{
	int id = canmsg->id;
	int ret = 0;
	if ((id < 0) || (id > 2047))
		return -1;
	rtl_critical(crvalue);
	if (CBArray[id] != NULL)
		ret = (*CBArray[id])(canmsg);
	rtl_end_critical(crvalue);
	return ret;
}

void *recvtask(void *dummy)
{
	int i;
	int fdrecv=(int)dummy;
	sCanMsg canmsg;
	while(1)
	{
		if (read(fdrecv, &canmsg, sizeof(canmsg)) == sizeof(canmsg))
		{
			ProcessID(&canmsg);
			printf("recv: %i %03x", canmsg.port,canmsg.id);
			if ((canmsg.datalen < 0) || (canmsg.datalen > 8))
			{
				printf("REMOTE\n");
			} 
			else
			{
				for(i=0;i<canmsg.datalen;i++)
					printf(" %02x",canmsg.data[i]);
				printf("\n");
			}
		}
	}
}

int fdsend;

int CANWrite(int port, int id, int len, unsigned char *data)
{
	sCanMsg canmsg;
	canmsg.id = id; canmsg.port = port;canmsg.datalen = len;
	if ((len <=8 ) && (len > 0))
		memcpy(&canmsg.data[0],data,len);
	if (sizeof(canmsg) == write(fdsend, &canmsg, sizeof(canmsg)))
		return 0;	
	return 1;
}

int CANInit(int console)
{
	int i,fdrecv;
	int len,port,id;
	int bi[8];
	char Buffer[50];
	sCanMsg canmsg;
	pthread_t	recvtaskid;
	
	fdrecv = open("/dev/rtf0",O_RDWR | O_CREAT/*|O_NONBLOCK*/);
	fdsend = open("/dev/rtf1",O_RDWR | O_CREAT/*|O_NONBLOCK*/);
	if ((fdsend<0) || (fdrecv<0))
		exit (-1);
	if (pthread_create(&recvtaskid, NULL, recvtask, (void*)fdrecv))
	{
		printf("can't create thread!\n");
		exit (-2);
	}
	if (!console)
		return 1;
	while(1)
	{
		int rc;
		if (fgets(Buffer,sizeof(Buffer)-1,stdin))
		{
			len = sscanf(Buffer, "%i %x %x %x %x %x %x %x %x %x", &port, &id,
						&bi[0],&bi[1],&bi[2],&bi[3],&bi[4],&bi[5],&bi[6],&bi[7]);
			if (len < 2)
//				return -1;			
				continue;
			canmsg.port = (unsigned short)port;
			canmsg.id   = (unsigned short)id;
			if (bi[0] < 0)
				canmsg.datalen = -1;
			else
				canmsg.datalen = len -2;
			for (i=0;i<canmsg.datalen;i++)
				canmsg.data[i] = bi[i];
			if (sizeof(canmsg) == write(fdsend, &canmsg, sizeof(canmsg)))
				printf("Message sent\n");
		}			
		else
			printf("rc(read) =%i\n",rc);
//		usleep(1000000);
	}
	close(fdsend);
	return 0;
}
