#include <linux/module.h>
#include <linux/kernel.h>
#include <asm/io.h>
#include <rtl.h>
#include <rtl_fifo.h>
#include <time.h>
#include <rtl_sched.h>
#include <rtl_sync.h>
#include <rtl_time.h>
#include <string.h>
#include <rtl_debug.h>
#include "candrv.h"
#define IODRV
#ifdef IODRV
 #include "../softplc/iodrv.h"
#endif 
#define RTL_DEBUG
static int MACId 	= 1;
MODULE_PARM(MACId, "i");
static int cout = 0;
MODULE_PARM(cout, "i");

static unsigned char TeleStart[2] = {1,0};
static unsigned char TeleStop[2] = {2,0};
static unsigned char CANInData[3] = {0,0,0};
pthread_t thread;

static int callbackfn(sCanMsg *msg)
{
	char b[26];
	int i;
	memcpy(CANInData,msg->data,3);
	if (cout)	// only if you want to see the output on the console
	{
		for (i=0;i<24;i++)
			b[i] = (msg->data[i/8] & (1 << (i%8))) ? '1' : '0';
		b[24]='\n';b[25]='\0';
//		rtl_printf("%i %i b",msg->port, msg->id);
	}
	return 0;
}

#ifndef IODRV
static void *sendtask(void *dummy)
{
	int i;
	unsigned char count=0;
	int canSendId = 0x200 + MACId;
//breakpoint();
	CANWrite(0,0,sizeof(TeleStart),TeleStart);
	pthread_make_periodic_np(pthread_self(),gethrtime(),10000*10000);
	while(1)
	{
		count++;
		CANWrite(0,canSendId,1,&count);
		for (i=0;i<10;i++)
			pthread_wait_np();           
	}		
	return NULL;
}
#else
static int iocallbackfn(eIOCommand cmd, uIOParam para)
{
	sGlobalDigital *diptr;
	int ret = -2;
	int canSendId = 0x200 + MACId;
//	rtl_printf("Weid-IOSynCB called with %i\n",cmd);
	switch(cmd)
	{
		case IOCMD_SyncDI:
		{
			int i,j;
			unsigned char *cid=&CANInData[0];
			diptr = &para.diptr[0];
			for (j=0;j<3;j++,cid++)
				for (i=0;i<8;i++)
					(diptr++)->value =  (*cid & (1 << i)) ? 1 : 0;
//					CANInData[j] =  (para.diptr[i+j*8] & (1 << i)) ? 1 : 0;
//					para.diptr[1].value = 1;// ((int)CANInData[j]) ? 1 : 0;
			ret = 0;
		}  break;
		case IOCMD_SyncDO:
		{
			unsigned char value	= 
				((para.doptr[0].value != 0) ? 1   : 0) +
				((para.doptr[1].value != 0) ? 2   : 0) +
				((para.doptr[2].value != 0) ? 4   : 0) +
				((para.doptr[3].value != 0) ? 8   : 0) +
				((para.doptr[4].value != 0) ? 16  : 0) +
				((para.doptr[5].value != 0) ? 32  : 0) +
				((para.doptr[6].value != 0) ? 64  : 0) +
				((para.doptr[7].value != 0) ? 128 : 0);
			CANWrite(0,canSendId,1,&value);
		}  break;
		case IOCMD_GetDI:
		case IOCMD_SetDO:
			return -1;
	}
	return ret;
}
#endif
int init_module(void)
{
	pthread_attr_t attr;
	struct sched_param sched_param;
#ifndef IODRV
	int thread_status;
#endif	
	TeleStart[1] = (unsigned char)MACId;
	TeleStop[1]  = (unsigned char)MACId;
	
	rtl_printf("weid24di8do on CPU %d\n",rtl_getcpuid());
	CANAddCallbackFn(0,0x180+MACId,callbackfn);
	pthread_attr_init (&attr);
	sched_param.sched_priority = 1;
	pthread_attr_setschedparam (&attr, &sched_param);
	rtl_printf("About to thread create\n");
#ifndef IODRV
	thread_status = pthread_create (&thread,  &attr, sendtask, (void *)1);
	if (thread_status != 0) {
		rtl_printf("failed to create RT-thread; errno=\n", errno);
		return -1;
	} else {
 		rtl_printf("created RT-thread\n"); 
	}
#else	
	IOAddDOFunction(8,iocallbackfn);
	IOAddDIFunction(24,iocallbackfn);
#endif	
	CANWrite(0,0,sizeof(TeleStart),TeleStart);
	return 0;
}


void cleanup_module(void)
{
    rtl_printf ("Removing module on CPU %d\n", rtl_getcpuid());
	CANWrite(0,0,sizeof(TeleStop),TeleStop);
	CANRemCallbackFn(0,0x180+MACId);
#ifndef IODRV
	pthread_delete_np (thread);
#else
	IORemDOFunction(iocallbackfn);
	IORemDIFunction(iocallbackfn);
#endif
}
