/***************************************************************************
                          codegeneratorpointers.h  -  description
                             -------------------
    begin                : Fri May 12 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CODEGENERATORPOINTERS_H
#define CODEGENERATORPOINTERS_H

/**
  *@author Ivan Martinez
  */

#include "derivablevector.h"

#include <string>
#include <set>
#include "errors.h"

class CCodeGenerator;

class CCodeGeneratorPointers : public TDerivableVector<CCodeGenerator*> {
public:
	/** Set of strings type */
	typedef set<string,less<string> > CStringsSet;

	/** Vector's const iterator */
	typedef vector<CCodeGenerator*>::const_iterator ConstIterator;

	/** Constructor */
	CCodeGeneratorPointers();

	/** Destructor */
	virtual ~CCodeGeneratorPointers();

	/** Calls AllArrange in the components. Returns if the calls succeeded.
		Stops when a call fails */
	bool AllArrange(void* ExtraParameterL = NULL);

	/** Calls AllGenerateCode(ostream&...) with the same arguments in the
		components */
	virtual void AllGenerateCode(ostream&, const CCodeGenerator*,
		const string&, void* = NULL) const;

	/** Calls AllGenerateCode(CStringsSet&...) with the same arguments in the
		components */
	virtual void AllGenerateCode(CStringsSet&, const CCodeGenerator*,
		const string&, void* = NULL) const;

	/** Calls AllReportErrors with the same argument in the components */
	void AllReportErrors(CErrors&);

	/** Returns if the vector contains the component of the argument */
	bool Contains(const CCodeGenerator*) const;

	/** Calls DeleteObjects in the components and then deletes them */
	virtual void DeleteObjects();

	/** Returns the number of components derived from the class of the
		argument */
	virtual unsigned int DescendantsOf(const string&) const;

	/** Calls RepeatedValue with the same arguments in the components.
		Returns if one of the calls returned true */
	virtual bool RepeatedValue(const CCodeGenerator*, const string&, const string&) const;
};

#endif
