/***************************************************************************
                          dualvariable.h  -  description
                             -------------------
    begin                : Tue Aug 1 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DUALVARIABLE_H
#define DUALVARIABLE_H

#include "variable.h"

/**
  *@author Ivan Martinez
  */

class CDualVariable : public CVariable  {
public:
	/** Constructor */
	CDualVariable();

	/** Destructor */
	~CDualVariable();

	/** Returns TRUE if the argument is "CDualVariable" or one of its base
		classes, otherwise FALSE */
	virtual bool DescendantOf(const string&) const;

	/** Returns the identifier of the global variable */
	const string& GlobalIdentifier() const;
protected: // Protected methods
	/** Generates the declaration of the global variable */
	virtual void GenerateGlobalDeclarations(ostream&, void* = NULL) const;

	/** Generates an assignation from the global to the local variable */
	virtual void GenerateGlobalReading(ostream&, void* = NULL) const;

	/** Generates an assignation from the local to the global variable */
	virtual void GenerateGlobalWriting(ostream&, void* = NULL) const;

	/** Sets the identifier of the global variable */
	void SetGlobalIdentifier(const string&);
private: // Private attributes
	/** Identifier of the global variable */
	string m_GlobalIdentifier;
};

#endif
