/***************************************************************************
                          function.h  -  description
                             -------------------
    begin                : Tue Feb 15 2000
    copyright            : (C) 2000 by 
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef FUNCTION_H
#define FUNCTION_H

#include "block.h"

/**
  *@author Ivan Martinez
  */

class CFunction : public CBlock  {
public:
	/** Constructor */
	CFunction();

	/** Destructor */
	virtual ~CFunction();

	/** Returns "CFuncion" */
	virtual string Class() const;

	/** Returns the C code that implements the function */
	const string& Code() const;

	/** Returns if the argument is "CFunction" or one of its base	classes */
	virtual bool DescendantOf(const string&) const;

	/** Sets the C code that implements the function */
	bool SetCode(const string&);
protected:
	/** Generates the C code that implements the function */
	virtual void GenerateBlockCode(ostream&) const;

	/** Calls a code generation function depending on the arguments */
	virtual void GenerateCode(ostream&, const CCodeGenerator*, const string&,
		void* = NULL) const;

	/** Generates the function that executes the block */
	virtual void GenerateFunctions(ostream&, void* = NULL) const;

	/** Generates the call to the initialization function */
	virtual void GenerateInitializationCall(ostream&, void* = NULL) const;

	/** Generates the function that performs the variables initialization */
	virtual void GenerateInitializationFunctions(ostream&, void* = NULL) const;
private: // Private attributes
	/** C code that implements the function */
	string m_Code;
};

#endif
