/***************************************************************************
                          functioninput.h  -  description
                             -------------------
    begin                : Mon Mar 27 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef FUNCTIONINPUT_H
#define FUNCTIONINPUT_H

#include "input.h"

/**
  *@author Ivan Martinez
  */

#include "loggingfifo.h"
#include "function.h"

class CBlock;

class CFunctionInput : public CInput {
public:
	/** Constructor */
	CFunctionInput();

	/** Destructor */
	~CFunctionInput();

	/** If the paramater is DescendantOf("CFunction"),
		sets it as the container of this component and returns true.
		If it isn't, generates the error 101 and retuns false */
	virtual bool AcceptsContainer(const CCodeGenerator*);

	/** Returns "CFunctionInput" */
	virtual string Class() const;

	/** Returns the container of this component */
	const CFunction* Container() const;

	/** Returns TRUE if the argument is "CFunctionInput" or one of its base
		classes, otherwise FALSE */
	virtual bool DescendantOf(const string&) const;

	/** Returns the name of this component */
	const string& Name() const;

	/** Sets the variable's initialization */
	bool SetInitialization(const string&);

	/** Sets the name of this component */
	bool SetName(const string&);

	/** Sets the type of the variable */
	bool SetType(const string&);
protected:
	/** If IllegalCIdentifier(Identifier()) generates the error 600.
		If the variable is not connected to an output generates the error 1400.
		Otherwise sets the identifier of the global variable */
	virtual void Arrange(void* = NULL);

	/** Calls a code generation function depending on the arguments */
	virtual void GenerateCode(ostream&, const CCodeGenerator*, const string&,
		void* = NULL) const;
};

#endif
