/***************************************************************************
                          rttaskc.cpp  -  description
                             -------------------
    begin                : Fri May 12 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "rttask.h"

#include "rtmodule.h"

CRTTask::CRTTask()
{
	m_FloatingPoint = "1";
	m_FunctionArgument = "";
	m_FunctionIdentifier = "";
	m_FunctionParameterIdentifier = "";
	m_PThreadAttrIdentifier = "";
	m_Period = "";
  m_Priority = "";
	m_SchedParamIdentifier = "";
	m_StartTime = "";
  m_Scheduling = PERIOD;
	m_IRQNumber = "0";
}
CRTTask::~CRTTask()
{
}

void CRTTask::GenerateThreadDeclaration(ostream& StreamL,
	void* ExtraParameterL = NULL) const
{
	if (Scheduling() == PERIOD)
		StreamL << "pthread_t " << Identifier() << ";" << endl;
}

void CRTTask::GenerateInitialization(ostream& StreamL,
	void* ExtraParameterL = NULL) const
{
	if (Scheduling() == PERIOD)
	{
	  StreamL << "  pthread_attr_init(&" <<
			PThreadAttrIdentifier() << ");" << endl;
		StreamL << "  " << SchedParamIdentifier() << ".sched_priority = " <<
			Priority() << ";" << endl;
		StreamL << "  pthread_attr_setschedparam(&" << PThreadAttrIdentifier() <<
			", &" << SchedParamIdentifier() << ");" << endl;
		StreamL << "  pthread_create(&" << Identifier() << ", &" <<
			PThreadAttrIdentifier() << ", " << FunctionIdentifier() <<
			", (void *)" << FunctionArgument() << ");" << endl;
	  StreamL << "  pthread_setfp_np(" << Identifier() << ", " <<
			FloatingPoint() << ");" << endl;
		StreamL << "  pthread_make_periodic_np(" << Identifier() << ", " <<
			StartTime() << ", " << Period() << ");" << endl;
	}
	else
	{
		StreamL << "  rtl_stop_interrupts();" << endl;
		StreamL << "  rtl_request_irq(" << IRQNumber() << ", &" <<
			FunctionIdentifier() << ");" << endl;
		StreamL << "  rtl_allow_interrupts();" << endl;
	}
}

void CRTTask::SetPThreadAttrIdentifier(const string& IdentifierL)
{
	m_PThreadAttrIdentifier = IdentifierL;
}

const string& CRTTask::PThreadAttrIdentifier() const
{
	return m_PThreadAttrIdentifier;
}

void CRTTask::SetStartTime(const string& StartTimeL)
{
	m_StartTime = StartTimeL;
}

const string& CRTTask::StartTime() const
{
	return m_StartTime;
}

void CRTTask::SetSchedParamIdentifier(const string& Identifier)
{
	m_SchedParamIdentifier = Identifier;
}

const string& CRTTask::SchedParamIdentifier() const
{
	return m_SchedParamIdentifier;
}

void CRTTask::SetPriority(const string& Priority)
{
	m_Priority = Priority;
}

const string& CRTTask::Priority() const
{
	return m_Priority;
}

void CRTTask::SetFunctionIdentifier(const string& Identifier)
{
	m_FunctionIdentifier = Identifier;
}

const string& CRTTask::FunctionIdentifier() const
{
	return m_FunctionIdentifier;
}

void CRTTask::SetFunctionArgument(const string& Argument)
{
	m_FunctionArgument = Argument;
}

const string& CRTTask::FunctionArgument() const
{
	return m_FunctionArgument;
}

void CRTTask::SetPeriod(const string& Period)
{
	m_Period = Period;
}

const string& CRTTask::Period() const
{
	return m_Period;
}

void CRTTask::SetFunctionParameterIdentifier(const string& Identifier)
{
	m_FunctionParameterIdentifier = Identifier;
}

const string& CRTTask::FunctionParameterIdentifier() const
{
	return m_FunctionParameterIdentifier;
}

void CRTTask::SetFloatingPoint(const string& FloatingPoint)
{
	m_FloatingPoint = FloatingPoint;
}

const string& CRTTask::FloatingPoint() const
{
	return m_FloatingPoint;
}

bool CRTTask::DescendantOf(const string& ClassNameL) const
{
	if (ClassNameL == "CRTTask")
		return true;
	else
		return CCodeGenerator::DescendantOf(ClassNameL);
}

void CRTTask::GenerateIncludes(CStringsSet& StringsSetL,
	void* ExtraParameterL = NULL) const
{
	if (Scheduling() == PERIOD)
	{
		StringsSetL.insert("#include <pthread.h>");
		StringsSetL.insert("#include <rtl_sched.h>");
	}
	else
		StringsSetL.insert("#include <rtl_sync.h>");
}

void CRTTask::GenerateDestruction(ostream& StreamL, void* ExtraParameterL = NULL) const
{
	if (Scheduling() == PERIOD)
		StreamL << "  pthread_delete_np(" << Identifier() << ");" << endl;
}

const CRTModule* CRTTask::Container() const
{
	return (CRTModule*)CCodeGenerator::Container();
}

void CRTTask::GenerateWakeUp(ostream& StreamL, void* ExtraParameterL = NULL) const
{
	if (Scheduling() == PERIOD)
		StreamL << "  pthread_wakeup_np(" << Identifier() << ");" << endl;
	else
		StreamL << "  rtl_hard_enable_irq(" << IRQNumber() << ");" << endl;
}

void CRTTask::SetScheduling(eScheduling Scheduling)
{
	m_Scheduling = Scheduling;
}

void CRTTask::SetIRQNumber(const string& IRQNumber)
{
	m_IRQNumber = IRQNumber;
}

const string& CRTTask::IRQNumber() const
{
  return m_IRQNumber;
}

enum CRTTask::eScheduling CRTTask::Scheduling() const
{
	return m_Scheduling;
}

void CRTTask::GenerateSuspend(ostream& StreamL, void* ExtraParameterL = NULL) const
{
	if (Scheduling() == PERIOD)
		StreamL << "  pthread_suspend_np(" << Identifier() << ");" << endl;
	else
		StreamL << "  rtl_hard_disable_irq(" << IRQNumber() << ");" << endl;
}

void CRTTask::GenerateInitializationVariablesDeclaration(CStringsSet& StringsSetL,
	void* ExtraParameterL = NULL) const
{
	if (Scheduling() == PERIOD)
	{
	  StringsSetL.insert("  pthread_attr_t " + PThreadAttrIdentifier() + ";");
	  StringsSetL.insert("  struct sched_param " + SchedParamIdentifier() + ";");
	}
}
