/***************************************************************************
                          settingfifo.h  -  description
                             -------------------
    begin                : Thu Jun 1 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SETTINGFIFO_H
#define SETTINGFIFO_H

#include "variablefifo.h"

/**
  *@author Ivan Martinez
  */

class CInitializableVariable;

class CSettingFifo : public CVariableFifo  {
public:
	/** Constructor */
	CSettingFifo();

	/** Destructor */
	~CSettingFifo();

	/** If the paramater is not DescendantOf("CInitializableVariable"),
		generates the error 101 and returns false. If it is, but the container
		already contains a setting fifo, generates the error 2000 and returns
		false. Otherwise sets the parameter as the container of this component
		and returns true */
	virtual bool AcceptsContainer(const CCodeGenerator*);

	/** Returns "CSettingFifo" */
	virtual string Class() const;

	/** Returns the container of this component */
	const CInitializableVariable* Container() const;

	/** Returns TRUE if the argument is "CSettingFifo" or one of its base
		classes, otherwise FALSE */
	virtual bool DescendantOf(const string&) const;

	/** Returns the number of the associated device file "/dev/rtf*" */
	unsigned int Number() const;

	/** Sets the number of the associated device file "/dev/rtf*" */
	bool SetNumber(unsigned int);

	/** Sets the size in bytes of the fifo */
	bool SetSize(int);

	/** Returns the size in bytes of the fifo */
	int Size();
protected:
	/** Sets the type of access to the fifo as READ, the identifier of
		the fifo, and "buffer" and "buffer_size" parameters for "read" and
		"write" */
	virtual void Arrange(void* = NULL);

	/** Calls a code generation function depending on the arguments */
	virtual void GenerateCode(ostream&, const CCodeGenerator*,
		const string&, void* = NULL) const;

	/** Generates fifo reading into the global variable preventing
		concurrent access of such variable */
	virtual void GenerateLockedFifoReading(ostream&, void* = NULL) const;
};

#endif
