/***************************************************************************
                          systemvariable.h  -  description
                             -------------------
    begin                : Tue Apr 18 2000
    copyright            : (C) 2000 by 
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SYSTEMVARIABLEC_H
#define SYSTEMVARIABLEC_H

#include "initializablevariable.h"

/**
  *@author Ivan Martinez
  */

#include "loggingfifo.h"
#include "settingfifo.h"
#include <iostream>

class CDynamicSystem;

class CSystemVariable : public CInitializableVariable {
public:
	/** Constructor */
	CSystemVariable();

	/** Destructor */
	~CSystemVariable();

	/** If the paramater is DescendantOf("CDynamicSystem"),
		sets it as the container of this component and returns true.
		If it isn't, generates the error 101 and retuns false */
	virtual bool AcceptsContainer(const CCodeGenerator*);

	/** Returns "CSystemVariable" */
	virtual string Class() const;

	/** Returns the container of this component */
	const CDynamicSystem* Container() const;

	/** Returns TRUE if the argument is "CSystemVariable" or one of its base
		classes, otherwise FALSE */
	virtual bool DescendantOf(const string&) const;

	/** Returns the identifier of the lock that prevents concurrent access to
		the global variable */
	const string& LockIdentifier() const;

	/** Returns the name of this component */
	const string& Name() const;

	/** Sets the variable's initialization */
	bool SetInitialization(const string&);

	/** Sets the name of this component */
	bool SetName(const string&);

	/** Sets the type of the variable */
	bool SetType(const string&);
protected:
	/** Calls a code generation function depending on the arguments */
	virtual void GenerateCode(ostream&, const CCodeGenerator*, const string&,
		void* = NULL) const;

	/** Calls a code generation function depending on the arguments.
		Lines repeated in the same section are eliminated */
	virtual void GenerateCode(CStringsSet&, const CCodeGenerator*, const string&,
		void* = NULL) const;

	/** Generates the declaration of the global variable and the lock that
		prevents concurrent access to it */
	virtual void GenerateGlobalDeclarations(ostream&, void* = NULL) const;

	/** Generates an assignation from the global to the local variable */
	virtual void GenerateGlobalReading(ostream&, void* = NULL) const;

	/** Generates an assignation from the local to the global variable */
	virtual void GenerateGlobalWriting(ostream&, void* = NULL) const;

	/** Generates the declaration of include directives */
	virtual void GenerateIncludes(CStringsSet&, void* = NULL) const;

	/** Generates the static local declaratin of the flags for the lock */
	virtual void GenerateLocalDeclarations(CStringsSet&, void* = NULL) const;

	/** Generates the static declaration of the local variable */
	virtual void GenerateLocalDeclarations(ostream&, void* = NULL) const;

	/** Sets the identifier of the lock that prevents concurrent access to
		the global variable */
	void SetLockIdentifier(const string&);
private: // Private attributes
	/** Identifier of the lock that prevents concurrent access to
		the global variable */
	string m_LockIdentifier;
};

#endif
