/***************************************************************************
                          blockvariable.cpp  -  description
                             -------------------
    begin                : Fri Mar 23 2001
    copyright            : (C) 2001 by Ivan Martinez
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "blockvariable.h"

#include "loop.h"

CBlockVariable::CBlockVariable()
{
}

CBlockVariable::~CBlockVariable()
{
}

bool CBlockVariable::AcceptsContainer(const CCodeGenerator* pContainer)
{
	if (pContainer->DescendantOf("CBlock"))
	{
		SetContainer(pContainer);
		return true;
	}
	else
	{
		AddError(101);
		return false;
	}
}

void CBlockVariable::Arrange(void *pExtraParameter)
{
	if (IllegalCIdentifier(Identifier()))
		AddError(600);
	else
		if (IllegalCIdentifier(Type()))
			AddError(602);
		else
			SetGlobalIdentifier("blockvar_" +
				Container()->Container()->Identifier() + "_" + Container()->Name() +
				"_" + Identifier());
}

string CBlockVariable::Class() const
{
	return "CBlockVariable";
}

const CBlock* CBlockVariable::Container() const
{
	return (CBlock*) CVariable::Container();
}

bool CBlockVariable::DescendantOf(const string& ClassName) const
{
	if (ClassName == "CBlockVariable")
		return true;
	else
		return CInitializableVariable::DescendantOf(ClassName);
}

void CBlockVariable::GenerateCode(ostream& Stream,
	const CCodeGenerator* pCaller, const string& Section,
	void* pExtraParameter) const
{
	if (pCaller->DescendantOf("CDynamicSystem"))
	{
		if (Section == "global variables declaration")
			GenerateGlobalDeclarations(Stream, pExtraParameter);
	}
	else
		if (pCaller->DescendantOf("CBlock"))
		{
			if (Section == "initialized variables declaration")
				GenerateInitializationDeclaration(Stream, pExtraParameter);
			else
				if (Section == "variables initialization")
					GenerateInitialization(Stream, pExtraParameter);
				else
					if (Section == "initialized variables writing")
						GenerateInitializationWriting(Stream, pExtraParameter);
					else
						if (Section == "local IO variables declaration")
							GenerateLocalDeclaration(Stream, pExtraParameter);
						else			
							if (Section == "inputs reading")
								GenerateGlobalReading(Stream, pExtraParameter);
		}
}

const string & CBlockVariable::Name() const
{
	return Identifier();
}

bool CBlockVariable::SetInitialization(const string& Initialization)
{
	CInitializableVariable::SetInitialization(Initialization);

	return true;
}

bool CBlockVariable::SetName(const string& Name)
{
	SetIdentifier(Name);

	if (IllegalCIdentifier(Name))
	{
		AddError(600);
		return false;
	}	
	else
		return true;
}

bool CBlockVariable::SetType(const string& Type)
{
	CVariable::SetType(Type);

	if (IllegalCIdentifier(Type))
	{
		AddError(602);
		return false;
	}	
	else
		return true;
}