/***************************************************************************
                          blockvariable.h  -  description
                             -------------------
    begin                : Fri Mar 23 2001
    copyright            : (C) 2001 by Ivan Martinez
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef BLOCKVARIABLE_H
#define BLOCKVARIABLE_H

#include "initializablevariable.h"

/**
  *@author Ivan Martinez
  */

#include "block.h"

class CBlockVariable : public CInitializableVariable  {
public: 
	/** Constructor */
	CBlockVariable();
	
	/** Destructor */
	~CBlockVariable();
	
	/** If the paramater is DescendantOf("CBlockVariable"),
		sets it as the container of this component and returns true.
		If it isn't, generates the error 101 and retuns false */
	virtual bool AcceptsContainer(const CCodeGenerator*);

	/** Returns "CBlockVariable" */
	virtual string Class() const;

	/** Returns the container of this component */
	const CBlock* Container() const;

	/** Returns TRUE if the argument is "CBlockVariable" or one of its
		base classes, otherwise FALSE */
	virtual bool DescendantOf(const string&) const;

	/** Returns the name of this component */
	const string & Name() const;

	/** Sets the variable's initialization */
	bool SetInitialization(const string&);
	
	/** Sets the name of this component */
	bool SetName(const string&);
	
	/** Sets the type of the variable */
	bool SetType(const string&);
protected: // Protected methods
	/** If IllegalCIdentifier(Identifier()) generates the error 600.
		If IllegalCIdentifier(Type()) generates the error 602. Otherwise,
		sets the identifier of the global variable */
	virtual void Arrange(void*);

	/** Calls a code generation function depending on the arguments */
	virtual void GenerateCode(ostream&, const CCodeGenerator*, const string&,
		void*) const;
};

#endif