/***************************************************************************
                          derivablevector.h  -  description
                             -------------------
    begin                : Fri May 12 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DERIVABLEVECTOR_H
#define DERIVABLEVECTOR_H

/**
  *@author Ivan Martinez
  */

#include <vector>


template <class Type>
class TDerivableVector {
public:
	/** Iterator */
	typedef vector<Type>::iterator Iterator;

	/** Const iterator */
	typedef vector<Type>::const_iterator ConstIterator;

	/** Constructor */
	TDerivableVector();

	/** Destructor */
	virtual ~TDerivableVector();

	/** Copies all the elements of the vector of the arguments
		at the end of this one in the same order */
	void Append(const TDerivableVector<Type>&);

	/** First element of the vector */
	ConstIterator Begin() const;

	/** First element of the vector */
	Iterator Begin();

	/** Deletes the elements of the vector */
	void Clear();

	/** Returns if the vector is empty */
	bool Empty() const;

	/** Position after the last element of the vector */
	ConstIterator End() const;

	/** Position after the last element of the vector */
	Iterator End();

  /** Removes the last element */
  void PopBack();
	
	/** Adds an element at the end of the vector */
	void PushBack(Type);

	/** Number of elements in the vector */
	unsigned int Size() const;

	/** Indexed access to the elements of the vector */
	Type operator[](unsigned int);

	/** Indexed access to the elements of the vector */
	Type operator[](unsigned int) const;
protected:
	/** Vector */
	vector<Type> m_Vector;
};

#include "derivablevector.cpp"

#endif
