/***************************************************************************
                          error.cpp  -  description
                             -------------------
    begin                : Mon Jul 17 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "error.h"

CError::CError(){
}
CError::~CError(){
}

void CError::SetLocation(const CCodeGenerator* Location)
{
	m_pLocation = Location;
}

const CCodeGenerator* CError::Location() const
{
	return m_pLocation;
}

void CError::SetNumber(unsigned int uNumber)
{
	m_uNumber = uNumber;
}

unsigned int CError::Number() const
{
	return m_uNumber;
}

const string CError::Message() const
{
	switch (Number())
	{
		/* 1XX: Error in CCodeGenerator's members */
		case 100:
			return "Identifier is not a valid C identifier";
		case 101:
			return "Class is not a suitable container for this component";
		case 102:
			return "Couldn't add component";
		case 103:
			return "Component can't be added to other components by user";
		/* 2XX: Errors in CRTModule's members */
		/* 3XX: Errors in CRTTask's members */
		case 300:
			return "Period must be greater than 0";
		/* 4XX: Errors in CRTFifo's members */
		case 400:
			return "Fifo's number must be greater or equal than 0";
		case 401:
			return "Fifo's size must be greater than 0";
		/* 5XX: Errors in CBlock' members */
		case 500:
			return "Block's name is not a valid C identifier";
		case 501:
			return "Name repeated in the loop";
		case 502:
			return "Block couldn't be ordered";
		/* 6XX: Errors in CVariable's members */
		case 600:
			return "Variable's name is not a valid C identifier";
		case 601:
			return "Name repeated in the scope";
		case 602:
			return "Variable's type is not a valid C identifier";
		/* 7XX: Errors in CInitializableVariable's members */
		/* 8XX: Errors in CVariableFifo's members */
		/* 9XX: Errors in CDynamicSystem's members */
		case 900:
			return "Dynamic sytem's time must be greater than 0";
		/* 10XX: Errors in CLoop's members */
		case 1000:
			return "Name repeated in the scope";
		/* 11XX: Errors in CFunction's members */
		/* 12XX: Errors in CSystemVariable's members*/
		/* 13XX: Errors in CLoopVariable's members */
		/* 14XX: Errors in CInput's members */
		case 1400:
			return "Input not linked";
		case 1401:
			return "Redundant initialization in input variable";
		/* 15XX: Errors in COutput's members */
		/* 16XX: Errors in CBlockVariable's members */
		/* 17XX: Errors in CCommandingFifo's members */
		case 1700:
			return "There can't be more than one commanding fifo in the system";
		/* 18XX: Errors in CReportingFifo's members */
		case 1800:
			return "There can't be more than one reporting fifo in the loop";
		/* 19XX: Errors in CLoggingFifo's members */
		case 1900:
			return "There can't be more than one logging fifo in the variable";
		case 1901:
			return "Sampling rate must be between 0 and 1";
		/* 20XX: Errors in CSettingFifo's members */
		case 2000:
			return "There can't be more than one setting fifo in the variable";
		/* 21XX: Errors in CProtectedSharedMemory's members */
		/* 22XX: Errors it CVariableSharedMemory's members */
		/* 23XX: Errors in CCommandingSharedMemory's members */
		/* 24XX: Errors in CReportingSharedMemory's members */
		/* 25XX: Errors in CLoggingSharedMemory's members */
		/* 26XX: Errors in CSettingSharedMemory's members */
		case 2600:
			return "There can't be more than one setting shared memory in"
				" the variable";
		/* 27XX: Errors in CFixedInput's members */
		/* 28XX: Errors in CFixedOutput's members */
		/* 29XX: Errors in CGain's members */
		case 2900:
			return "Variable factor is not a valid C identifier";
		/* 30XX: Errors in CFunctionInput's members */
		/* 31XX: Errors in CFunctionOutput's members */
		/* 32XX: Errors in CSummingInput's members */
		/* 33XX: Errors in CBlockVariable's members */
		/* 34XX: Errors in CFixedOperationBlock's members */
		case 3400:
			return "IOs type is not a valid C identifier";
		/* 35XX: Errors in CSummingPoint's members */
		case 3500:
			return "Input index must be between 1 and the number of inputs";
		/* 36XX: Errors in CGain's members */
		/* 37XX: Errors in CIntegrator's members */
		case 3700:
			return "Variable input factor is not a valid C identifier";
		case 3701:
			return "Variable state factor is not a valid C identifier";
		default:
			return "Unknown error";
	}
}
