/***************************************************************************
                          gain.cpp  -  description
                             -------------------
    begin                : Fri Feb 23 2001
    copyright            : (C) 2001 by Ivan Martinez
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "gain.h"

#include "loop.h"
#include "dynamicsystem.h"

CGain::CGain()
{
	CFixedInput* Input;
	CFixedOutput* Output;
	
	Input = new CFixedInput(this);
	Output = new CFixedOutput(this);
	Input->SetName("input");
	Output->SetName("output");
	AddFixedComponent(Input);
	AddFixedComponent(Output);
	
	m_FactorType = CONSTANT;
	m_rConstantFactor = 1;
	m_VariableFactor = "";
}

CGain::~CGain()
{
}

string CGain::Class() const
{
	return "CGain";
}

float CGain::ConstantFactor() const
{
	if (m_FactorType == CONSTANT)
		return m_rConstantFactor;
	else
		return 0;
}

bool CGain::DescendantOf(const string& ClassName) const
{
	if (ClassName == "CGain")
		return true;
	else
		return CFixedOperationBlock::DescendantOf(ClassName);
}

void CGain::GenerateFunctions(ostream& Stream,
	void* pExtraParameter = NULL) const
{
	CStringsSet StringsSet;

	Stream << "void " << Identifier() << "(void)" <<	endl;
	Stream << "{" << endl;

	if (m_FactorType == VARIABLE)
	{
		StringsSet.clear();
		Container()->Container()->ComponentsGenerateCode(StringsSet, this,
			"flags declaration", pExtraParameter);
		copy(StringsSet.begin(), StringsSet.end(),
			ostream_iterator<string>(Stream, "\n"));
		Container()->Container()->ComponentsGenerateCode(Stream, this,
			"local system variable declarations", pExtraParameter);
		Container()->ComponentsGenerateCode(Stream, this,
			"local loop variable declarations", pExtraParameter);
			
		Container()->Container()->ComponentsGenerateCode(Stream, this,
			"system variables reading", pExtraParameter);
		Container()->ComponentsGenerateCode(Stream, this,
			"loop variables reading", pExtraParameter);
	}

	Stream << "  " << Output()->GlobalIdentifier() << " = " <<
		Input()->GlobalIdentifier() << " * ";
	if (m_FactorType == CONSTANT)
		Stream << ConstantFactor();
	else
		Stream << VariableFactor();
	Stream << ";" << endl;

	if (m_FactorType == VARIABLE)
	{
		Container()->ComponentsGenerateCode(Stream, this,
			"loop variables writing", pExtraParameter);
		Container()->Container()->ComponentsGenerateCode(Stream, this,
			"system variables writing", pExtraParameter);
	}

	Stream << "}" << endl;
}

CFixedInput* CGain::Input() const
{
	return (CFixedInput*) *(FixedComponents().Begin());
}

CFixedOutput* CGain::Output() const
{
	return (CFixedOutput*) FixedComponents()[1];
}

bool CGain::SetConstantFactor(float Factor)
{
	m_FactorType = CONSTANT;
	m_rConstantFactor = Factor;
	return true;
}

bool CGain::SetVariableFactor(const string& Factor)
{
	if (IllegalCIdentifier(Factor))
	{
		AddError(2900);
		return false;
	}	
	else
	{
		m_FactorType = VARIABLE;
		m_VariableFactor = Factor;
		return true;
	}
}

const string CGain::VariableFactor() const
{
	if (m_FactorType == CONSTANT)
		return m_VariableFactor;
	else
		return "";
}