/***************************************************************************
                          loggingfifo.h  -  description
                             -------------------
    begin                : Tue May 23 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LOGGINGFIFO_H
#define LOGGINGFIFO_H

#include "variablefifo.h"

/**
  *@author Ivan Martinez
  */

class CDualVariable;

class CLoggingFifo : public CVariableFifo  {
public:
	/** Constructor */
	CLoggingFifo();

	/** Destructor */
	virtual ~CLoggingFifo();

	/** If the paramater is not DescendantOf("CDualVariable"),
		generates the error 101 and returns false. If it is, but the container
		already contains a logging fifo, generates the error 1900 and returns
		false. Otherwise sets the parameter as the container of this component
		and returns true */
	virtual bool AcceptsContainer(const CCodeGenerator*);

	/** Returns "CLoggingFifo" */
	virtual string Class() const;

	/** Returns the container of this component */
	const CDualVariable* Container() const;

	/** Returns TRUE if the argument is "CLoggingFifo" or one of its base
		classes, otherwise FALSE */
	virtual bool DescendantOf(const string&) const;

	/** Returns the number of the associated device file "/dev/rtf*" */
	unsigned int Number() const;

	/** Returns the rate to write values in the fifo, from 0 to 1 */
	float SamplingRate() const;

	/** Sets the number of the associated device file "/dev/rtf*" */
	bool SetNumber(unsigned int);

	/** Sets the rate to write values in the fifo, from 0 to 1 */
	bool SetSamplingRate(float);

	/** Sets the size in bytes of the fifo */
	bool SetSize(int);

	/** Returns the size in bytes of the fifo */
	int Size() const;
protected:
	/** Sets the type of access to the fifo as WRITE, the identifier of
		the fifo, and "buffer" and "buffer_size" parameters for "read" and
		"write" */
	virtual void Arrange(void* = NULL);

	/** Calls a code generation function depending on the arguments */
	virtual void GenerateCode(ostream&, const CCodeGenerator*, const string&,
		void* = NULL) const;

  /**  */
  void GenerateFifoWriting(ostream&, void * = NULL) const;

	/** Generates fifo reading from the global variable preventing
		concurrent access of such variable */
	virtual void GenerateLockedFifoWriting(ostream&, void* = NULL) const;

	/** Generates the declaration of the sampling-counter variable */
	virtual void GenerateSamplingCounterDeclaration(ostream&,
		void* = NULL) const;

  /**  */
  void GenerateSamplingCounterInitialization(ostream&, void* = NULL) const;

	/** Returns the identifier of the sampling-counter variable */
	const string& SamplingCounterIdentifier() const;

	/**  */
	void SetSamplingCounterIdentifier(const string&);
private: // Private attributes
	/** Rate to write values in the fifo, from 0 to 1 */
	float m_rSamplingRate;

	/** Identifier of the sampling-counter variable */
	string m_SamplingCounterIdentifier;
};

#endif
