/***************************************************************************
                          protectedsharedmemory.cpp  -  description
                             -------------------
    begin                : Tue Feb 13 2001
    copyright            : (C) 2001 by Ivan Martinez
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "protectedsharedmemory.h"

CProtectedSharedMemory::CProtectedSharedMemory()
{
}

CProtectedSharedMemory::~CProtectedSharedMemory()
{
}
/**  */
void CProtectedSharedMemory::GenerateMemoryWriting(ostream& Stream,
	void* pExtraParameter = NULL) const
{
	GenerateLockAcquisition(Stream, pExtraParameter);
	Stream << "        memcpy(" << Identifier() << ", " <<
		KernelMemoryPointer() << ", " << DataSize() << ");" << endl;
	GenerateLockRelease(Stream, pExtraParameter);
}
/**  */
void CProtectedSharedMemory::GenerateMemoryReading(ostream& Stream,
	void* pExtraParameter = NULL) const
{
	GenerateLockAcquisition(Stream, pExtraParameter);
	Stream << "        memcpy(" << KernelMemoryPointer() << ", " <<
		Identifier() << ", " << DataSize() << ");" << endl;
	GenerateLockRelease(Stream, pExtraParameter);
}
/**  */
const string& CProtectedSharedMemory::KernelMemoryPointer() const
{
	return m_KernelMemoryPointer;
}

void CProtectedSharedMemory::SetKernelMemoryPointer(
	const string& KernelMemoryPointer)
{
	m_KernelMemoryPointer = KernelMemoryPointer;
}

const string& CProtectedSharedMemory::DataSize() const
{
	return m_DataSize;
}

void CProtectedSharedMemory::SetDataSize(const string& DataSize)
{
	m_DataSize = DataSize;
}

void CProtectedSharedMemory::GenerateDeclaration(ostream& Stream,
	void* pExtraParameter = NULL) const
{
	Stream << DataType() << "* " << Identifier() << ";" << endl;
	Stream << LockType() << "* " << LockIdentifier() << ";" << endl;
}

const string& CProtectedSharedMemory::DataType() const
{
	return m_DataType;
}

const string& CProtectedSharedMemory::LockSize() const
{
	return m_LockSize;
}

const string& CProtectedSharedMemory::LockType() const
{
	return m_LockType;
}

void CProtectedSharedMemory::SetDataType(const string& DataType)
{
	m_DataType = DataType;
}

void CProtectedSharedMemory::SetLockSize(const string& LockSize)
{
	m_LockSize = LockSize;
}

void CProtectedSharedMemory::SetLockType(const string& LockType)
{
	m_LockType = LockType;
}

const string& CProtectedSharedMemory::LockIdentifier() const
{
	return m_LockIdentifier;
}

void CProtectedSharedMemory::SetLockIdentifier(const string& LockIdentifier)
{
	m_LockIdentifier = LockIdentifier;
}

const string& CProtectedSharedMemory::DataOffset() const
{
	return m_DataOffset;
}

void CProtectedSharedMemory::SetDataOffset(const string& DataOffset)
{
	m_DataOffset = DataOffset;
}

void CProtectedSharedMemory::GenerateCode(CStringsSet& StringsSet,
	const CCodeGenerator* pCaller, const string& Section,
	void* pExtraParameter = NULL) const
{
	if (pCaller->DescendantOf("CDynamicSystem"))
		if (Section == "includes")
			GenerateIncludes(StringsSet, pExtraParameter);
}

void CProtectedSharedMemory::GenerateIncludes(CStringsSet& StringsSet,
	void* pExtraParameter = NULL) const
{
	StringsSet.insert("#include <asm/page.h>");
}
