/***************************************************************************
                          protectedsharedmemory.h  -  description
                             -------------------
    begin                : Tue Feb 13 2001
    copyright            : (C) 2001 by Ivan Martinez
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PROTECTEDSHAREDMEMORY_H
#define PROTECTEDSHAREDMEMORY_H

#include "codegenerator.h"

/**
  *@author Ivan Martinez
  */

class CProtectedSharedMemory : public CCodeGenerator  {
public:
	/** Constructor */
	CProtectedSharedMemory();

	/** Destructor */
	~CProtectedSharedMemory();

	/** Returns the offset of the data in shared memory */
	const string& DataOffset() const;
	
	/** Returns the size of the shared data */
	const string& DataSize() const;

	/** Returns the type of the shared data */
	const string& DataType() const;

	/** Returns the identifier of the kernel memory pointer */
	const string& KernelMemoryPointer() const;
	
	/** Returns the identifier of the lock that prevents concurrent access to
		the shared data */
	const string& LockIdentifier() const;

	/** Returns the size of the lock that prevents concurrent access to
		the shared data */
	const string& LockSize() const;

	/** Returns the type of the lock that prevents concurrent access to
		the shared data */
	const string& LockType() const;
protected: // Protected methods
	/** Generates code for the section of the component indicated in the
		arguments. Lines repeated in the same section are eliminated */
	virtual void GenerateCode(CStringsSet&, const CCodeGenerator*,
		const string&, void* = NULL) const;

  /** Generates pointers declaration */
	virtual void GenerateDeclaration(ostream&, void* = NULL) const;

	/** Generates "include" directives declaration */
	virtual void GenerateIncludes(CStringsSet&, void* = NULL) const;
	
	/** Generates shared memory and lock initialization */
	virtual void GenerateInitialization(ostream&, void* = NULL) const = 0;
	
	/** Generates the acquisition of the lock that prevents concurrent
		access to the shared data */
	virtual void GenerateLockAcquisition(ostream&, void* = NULL) const = 0;

	/** Generates the release of the lock that prevents concurrent access
		to the shared data */
	virtual void GenerateLockRelease(ostream&, void* = NULL) const = 0;
	
	/** Generates the copy of the shared data to the address pointed by
		the kerner memory pointer */
	virtual void GenerateMemoryReading(ostream&, void* = NULL) const;

	/** Generates the copy of the data pointed by the kernel memory pointer
		to the shared memory */
	virtual void GenerateMemoryWriting(ostream&, void* = NULL) const;

	/** Sets the offset of the data in shared memory */
	void SetDataOffset(const string&);
	
	/** Sets the size of the shared data */
	void SetDataSize(const string&);

	/** Sets the type of the shared data */
	void SetDataType(const string&);

	/** Sets the identifier of the kernel memory pointer */
	void SetKernelMemoryPointer(const string&);
	
	/** Sets the identifier of the lock that prevents concurrent access to
		the shared data */
	void SetLockIdentifier(const string&);
	
	/** Sets the size of the lock that prevents concurrent access to
		the shared data */
	void SetLockSize(const string&);

	/** Sets the type of the lock that prevents concurrent access to
		the shared data */
	void SetLockType(const string&);
private: // Private attributes
	/** Offset of the data in shared memory */
	string m_DataOffset;

	/** Shared data size */
	string m_DataSize;

	/** Shared data type */
	string m_DataType;

	/** Identifier of the kernel memory pointer */
	string m_KernelMemoryPointer;

	/** Identifier of the lock that prevents concurrent access to the shared
		data */
	string m_LockIdentifier;

	/** Size of the lock that prevents concurrent access to the shared data */
	string m_LockSize;

	/** Type of the lock that prevents concurrent access to the shared data */
	string m_LockType;
};

#endif
