/***************************************************************************
                          rtfifo.h  -  description
                             -------------------
    begin                : Tue May 23 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef RTFIFO_H
#define RTFIFO_H

#include "codegenerator.h"

/**
  *@author Ivan Martinez
  */

class CRTFifo : public CCodeGenerator  {
public:
	/** Types of access from the realtime task */
	enum EMode {READ, WRITE};

	/** Constructor */
	CRTFifo();

	/** Destructor */
	virtual ~CRTFifo();

	/** Returns the "buffer" parameter for "read" and "write" */
	const string& Buffer() const;

	/** Returns the "buffer_size" parameter for "read" and "write" */
	const string& BufferSize() const;

	/** Returns if the argument is "CRTFifo" or "CCodeGenerator" */
	virtual bool DescendantOf(const string&) const;

	/** Returns the type of access from the realtime task */
	EMode Mode() const;

	/** Returns the number of the associated device file "/dev/rtf*" */
	const string& Number() const;

	/** Returns the "size" parameter for "rtf_create" */
	const string& Size() const;
protected:
	/** Generates fifo's declaration */
	virtual void GenerateDeclaration(ostream&, void* = NULL) const;

	/** Generates fifo's destruction */
	virtual void GenerateDestruction(ostream&, void* = NULL) const;

	/** Generates fifo's reading */
	virtual void GenerateFifoReading(ostream&, void* = NULL) const;

	/** Generates fifo's writing */
	virtual void GenerateFifoWriting(ostream&, void* = NULL) const;

	/** Generates "include" directives declaration */
	virtual void GenerateIncludes(CStringsSet&, void* = NULL) const;

	/** Generates fifo's initialization */
	virtual void GenerateInitialization(ostream&, void* = NULL) const;

	/** Sets the "buffer" paramenter for "read" and "write" */
	void SetBuffer(const string&);

	/** Sets the "buffer_size" parameter for "read" and "write" */
	void SetBufferSize(const string&);

	/** Sets the type of access from the realtime task */
	void SetMode(EMode);

	/** Sets the number of the associated device file "/dev/rtf*" */
	void SetNumber(const string&);

	/** Sets the "size" parameter for "rtf_create" */
	void SetSize(const string&);
private: // Private attributes
	/** Size in bytes of the fifo */
	string m_Size;

	/** Number of the associated device file  */
	string m_Number;

	/** Type of access of the associated device file */
	EMode m_Mode;

	/** Buffer for fifo reading/writing */
	string m_Buffer;

	/** Size in bytes of the buffer for fifo reading/writing */
	string m_BufferSize;
};

#endif
