/***************************************************************************
                          summingpoint.cpp  -  description
                             -------------------
    begin                : Tue Mar 13 2001
    copyright            : (C) 2001 by Ivan Martinez
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "summingpoint.h"

#include <strstream>

CSummingPoint::CSummingPoint()
{
	CFixedOutput* Output;
	
	Output = new CFixedOutput(this);
	Output->SetName("output");
	AddFixedComponent(Output);	
}

CSummingPoint::~CSummingPoint()
{
}

string CSummingPoint::Class() const
{
	return "CSummingPoint";
}

bool CSummingPoint::DescendantOf(const string& ClassName) const
{
	if (ClassName == "CSummingPoint")
		return true;
	else
		return CFixedOperationBlock::DescendantOf(ClassName);
}

void CSummingPoint::GenerateFunctions(ostream& Stream,
	void* pExtraParameter) const
{
	CStringsSet StringsSet;
	CSummingInput* pInput;
	CCodeGeneratorPointers::ConstIterator InputIterator;

	Stream << "void " << Identifier() << "(void)" <<	endl;
	Stream << "{" << endl;

	Stream << "  " << Output()->GlobalIdentifier() << " =";
	InputIterator = FixedComponents().Begin();
	InputIterator++;
	while (InputIterator != FixedComponents().End())
	{
		
		pInput = (CSummingInput*)(*InputIterator);
		if (pInput->Negated())
			Stream << " - ";
		else
			Stream << " + ";
		Stream << pInput->GlobalIdentifier();
		InputIterator++;
	}
	Stream << ";" << endl;

	Stream << "}" << endl;
}

CSummingInput* CSummingPoint::Input(unsigned int uIndex)
{
	if ((uIndex >= 1) || (uIndex <= NumberOfInputs()))
		return (CSummingInput*) FixedComponents()[uIndex];
	else
	{
		AddError(3500);
		return NULL;
	}
}

CFixedOutput* CSummingPoint::Output() const
{
	return (CFixedOutput*) *(FixedComponents().Begin());
}

unsigned int CSummingPoint::NumberOfInputs() const
{
	return FixedComponents().Size() - 1;
}

bool CSummingPoint::SetNumberOfInputs(unsigned int uNumber)
{
	CSummingInput* pInput;
	strstream* pName;
	
	while (NumberOfInputs() < uNumber)
	{
		pInput = new CSummingInput(this);
		pName = new strstream();
		
		*pName << "input" << (NumberOfInputs() + 1) << ends;
		pInput->SetName(pName->str());
		
		delete pName;
		AddFixedComponent(pInput);
	}
	
	while (NumberOfInputs() > uNumber)
		FixedComponents().PopBack();
}

