/***************************************************************************
                          block.cpp  -  description
                             -------------------
    begin                : Fri Feb 11 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <block.h>

#include <dynamicsystem.h>
#include <loop.h>
#include <input.h>

CBlock::CBlock()
{
	m_Name = "";
}

CBlock::~CBlock()
{
}

void CBlock::Arrange(void* ParameterL = NULL)
{
	if (IllegalCIdentifier(Name()))
		AddError(500);
	else
		SetIdentifier("block_" + Container()->Identifier() + "_" + Name());
}

bool CBlock::SetName(const string& Name)
{
	m_Name = Name;

	if (IllegalCIdentifier(Name))
	{
		AddError(500);
		return false;
	}	
	else
		return true;
}

const string& CBlock::Name() const
{
	return m_Name;
}

bool CBlock::DescendantOf(const string& ClassName) const
{
	if (ClassName == "CBlock")
		return true;
	else
		return CCodeGenerator::DescendantOf(ClassName);
}

void CBlock::GenerateCall(ostream& Stream,
	void* ExtraParameter = NULL) const
{
	Stream << "      " << Identifier() << "();" << endl;
}

bool CBlock::CandidateForOrdenation(
	const CCodeGeneratorPointers& OrderedComponents) const
{
	CCodeGeneratorPointers::ConstIterator ComponentPIL;
	bool InputsInitializedOrFromOrderedL;
	CInput* InputVariablePL;

	if (!OrderedComponents.Contains(this))
	{
		ComponentPIL = Components().Begin();

		InputsInitializedOrFromOrderedL = true;
		while ((ComponentPIL != Components().End()) &&
			InputsInitializedOrFromOrderedL)
		{
			if ((*ComponentPIL)->DescendantOf("CInput"))
			{
				InputVariablePL = (CInput*)(*ComponentPIL);

				InputsInitializedOrFromOrderedL = InputVariablePL->Initialized() ||
					InputVariablePL->ConnectedToOrderedBlock(OrderedComponents);
			}
			ComponentPIL++;
		}
		return InputsInitializedOrFromOrderedL;
	}
	else
		return false;
}

const CLoop* CBlock::Container() const
{
	return (CLoop*)CCodeGenerator::Container();
}

void CBlock::ComponentsDependingArrange(void* pExtraParameter = NULL)
{
	if (Container()->RepeatedValue(this, "name", Name()))
		AddError(501);
}

bool CBlock::RepeatedValue(const CCodeGenerator* pCaller,
	const string& ValueName, const string& Value) const
{
	if ((pCaller != this) && (ValueName == "name") && (Value == Name()))
		return true;
	else
		return Components().RepeatedValue(pCaller, ValueName, Value) ||
			FixedComponents().RepeatedValue(pCaller, ValueName, Value);
}

bool CBlock::AcceptsContainer(const CCodeGenerator* pContainer)
{
	if (pContainer->DescendantOf("CLoop"))
	{
		SetContainer(pContainer);
		return true;
	}
	else
	{
		AddError(101);
		return false;
	}
}

void CBlock::GenerateInitializationFunctions(ostream& Stream,
	void* pExtraParameter = NULL) const
{
	Stream << "void " << Container()->Identifier() << "_" << Name() <<
		"_vars_init(void)" << endl;
	Stream << "{" << endl;

	ComponentsGenerateCode(Stream, "initialized variables declaration",
		pExtraParameter);
	ComponentsGenerateCode(Stream, "variables initialization",
		pExtraParameter);
	ComponentsGenerateCode(Stream, "initialized variables writing",
		pExtraParameter);

	Stream << "}" << endl;
}

void CBlock::GenerateInitializationCall(ostream& Stream,
	void* pExtraParameter = NULL) const
{
	Stream << "  " << Container()->Identifier() << "_" << Name() <<
		"_vars_init();" << endl;
}

void CBlock::GenerateCode(ostream& Stream, const CCodeGenerator* pCaller,
	const string& Section, void* pExtraParameter = NULL) const
{
	if (pCaller->DescendantOf("CDynamicSystem"))
	{
		if (Section == "initialization functions declaration")
			GenerateInitializationFunctions(Stream, pExtraParameter);
		else
			if (Section == "initialization functions call")
				GenerateInitializationCall(Stream, pExtraParameter);
			else
				if (Section == "functions declaration")
					GenerateFunctions(Stream, pExtraParameter);
	}
	else
		if (pCaller->DescendantOf("CLoop"))
			if (Section == "processes call")
				GenerateCall(Stream, pExtraParameter);
}
