/***************************************************************************
                          block.h  -  description
                             -------------------
    begin                : Fri Feb 11 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef BLOCK_H
#define BLOCK_H

#include <codegenerator.h>

/**
  *@author Ivan Martinez
  */

class CLoop;

class CBlock : public CCodeGenerator {
public:
	/** Constructor */
	CBlock();

	/** Destructor */
	virtual ~CBlock();

	/** If the paramater is DescendantOf("CLoop"),
		sets it as the container of this component and returns true.
		If it isn't, generates the error 101 and retuns false */
	virtual bool AcceptsContainer(const CCodeGenerator*);

	/** Returns if the block isn't ordered and all its inputs are
		initialized or connected to ordered blocks */
	virtual bool CandidateForOrdenation(const CCodeGeneratorPointers&) const;

	/** Returns the container of this component */
	const CLoop* Container() const;

	/** Returns TRUE if the argument is "CBlock" or "CCodeGenerator",
		otherwise FALSE */
	virtual bool DescendantOf(const string&) const;

	/** Returns the name of this component */
	const string& Name() const;

	/** If the first argument is not this component, the second is "name"
		and the third is Name(), returns true. If not, calls the function
		with the same arguments in the contained components and returns if
		one of the calls returned true. */
	virtual bool RepeatedValue(const CCodeGenerator*, const string&,
		const string&) const;

	/** Sets the name */
	bool SetName(const string&);
protected:
	/** If IllegalCIdentifier(Name()) generates the error 500.
		If not,	sets the identifier */
	virtual void Arrange(void* = NULL);

	/** Generates the error 501 if the value of Name() is repeated
		in any other component of	the same loop */
	virtual void ComponentsDependingArrange(void* = NULL);

	/** Generates the call of the function that contains
		the operations of the block */
	virtual void GenerateCall(ostream&, void* = NULL) const;
	
	/** Calls a code generation function depending on the arguments */
	virtual void GenerateCode(ostream&, const CCodeGenerator*,
		const string&, void* = NULL) const;
	
	/** Generates the function that executes the block */
  virtual void GenerateFunctions(ostream&, void* = NULL) const = 0;

	/** Generates the call to the initialization function */
	virtual void GenerateInitializationCall(ostream&, void* = NULL) const;
	
	/** Generates the function that performs the variables initialization */
	virtual void GenerateInitializationFunctions(ostream&, void* = NULL) const;
private: // Private attributes
	/** Name */
	string m_Name;
};

#endif
