/***************************************************************************
                          comedichannel.cpp  -  description
                             -------------------
    begin                : Tue Mar 6 2001
    copyright            : (C) 2001 by Ivan Martinez
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <comediinput.h>

#include <strstream>

CComediChannel::CComediChannel()
{
	m_ComediChannel = 0;
	m_ComediMinor = 0;
	m_ComediRange = 0;
	m_ComediReference = 0;
	m_ComediSubdevice = 0;
	m_Offset = 0;
	m_Range = 1;
}

CComediChannel::~CComediChannel()
{
}

string CComediChannel::Class() const
{
	return "CComediChannel";
}

unsigned int CComediChannel::ComediChannel() const
{
	return m_ComediChannel;
}

unsigned int CComediChannel::ComediMinor() const
{
	return m_ComediMinor;
}

unsigned int CComediChannel::ComediRange() const
{
	return m_ComediRange;
}

unsigned int CComediChannel::ComediReference() const
{
	return m_ComediReference;
}

unsigned int CComediChannel::ComediSubdevice() const
{
	return m_ComediSubdevice;
}

bool CComediChannel::DescendantOf(const string & ClassName) const
{
	if (ClassName == "CComediChannel")
		return true;
	else
		return CFixedOperationBlock::DescendantOf(ClassName);
}

void CComediChannel::GenerateCode(CStringsSet& StringsSet,
	const CCodeGenerator* pCaller, const string& Section,
	void* pExtraParameter) const
{
	if (pCaller->DescendantOf("CDynamicSystem"))
	{
		if (Section == "includes")
			GenerateIncludes(StringsSet, pExtraParameter);
		else
			if (Section == "comedi devices opening")
				GenerateDevicesOpening(StringsSet, pExtraParameter);
			else
				if (Section == "comedi subdevices locking")
					GenerateSubdevicesLocking(StringsSet, pExtraParameter);
				else
					if (Section == "comedi subdevices unlocking")
						GenerateSubdevicesUnlocking(StringsSet, pExtraParameter);
					else					
						if (Section == "comedi devices closing")
							GenerateDevicesClosing(StringsSet, pExtraParameter);
	}
}

void CComediChannel::GenerateCode(ostream& Stream,
	const CCodeGenerator* pCaller , const string& Section,
	void* pExtraParameter) const
{
	if (pCaller->DescendantOf("CDynamicSystem"))
	{
			if (Section == "module I/Os declaration")
				GenerateDeclarations(Stream, pExtraParameter);
			else
				if (Section == "functions declaration")
					GenerateFunctions(Stream, pExtraParameter);
				else
					if (Section == "module I/Os initialization")
						GenerateInitialization(Stream, pExtraParameter);
	}
	else
		if (pCaller->DescendantOf("CLoop"))
			if (Section == "processes call")
				GenerateCall(Stream, pExtraParameter);
}

void CComediChannel::GenerateDeclarations(ostream& Stream,
	void* pExtraParameter) const
{
	Stream << "lsampl_t " << MaximumData() << ";" << endl;
}

void CComediChannel::GenerateDevicesClosing(CStringsSet & StringsSet,
	void * pExtraParameter) const
{
	strstream Stream;

	Stream << "  comedi_close(" << ComediMinor() << ");" << ends;

	StringsSet.insert(Stream.str());
}

void CComediChannel::GenerateDevicesOpening(CStringsSet & StringsSet,
	void * pExtraParameter) const
{
	strstream Stream;

	Stream << "  comedi_open(" <<	ComediMinor() << ");" << ends;

	StringsSet.insert(Stream.str());
}

void CComediChannel::GenerateIncludes(CStringsSet& StringsSet,
	void* pExtraParameter) const
{
	StringsSet.insert("#include <linux/comedilib.h>");
}

void CComediChannel::GenerateInitialization(ostream& Stream,
	void* pExtraParameter) const
{
	Stream << "  " << MaximumData() << " = comedi_get_maxdata(" <<
		ComediMinor() << ", " << ComediSubdevice() << ", " << ComediChannel() <<
		");" << endl;
}

void CComediChannel::GenerateSubdevicesLocking(CStringsSet & StringsSet,
	void * pExtraParameter) const
{
	strstream Stream;

	Stream << "  comedi_lock(" <<	ComediMinor() << ", " <<
		ComediSubdevice() << ");" << ends;

	StringsSet.insert(Stream.str());
}

void CComediChannel::GenerateSubdevicesUnlocking(CStringsSet & StringsSet,
	void * pExtraParameter) const
{
	strstream Stream;

	Stream << "  comedi_unlock(" << ComediMinor() << ", " <<
		ComediSubdevice() << ");" << ends;

	StringsSet.insert(Stream.str());
}

string CComediChannel::MaximumData() const
{
	return "maxdata_" + Identifier();
}

float CComediChannel::Offset() const
{
	return m_Offset;
}

float CComediChannel::Range() const
{
	return m_Range;
}

bool CComediChannel::SetComediChannel(unsigned int Channel)
{
	m_ComediChannel = Channel;

	return true;
}

bool CComediChannel::SetComediMinor(unsigned int Minor)
{
	m_ComediMinor = Minor;

	return true;
}

bool CComediChannel::SetComediSubdevice(unsigned int Subdevice)
{
	m_ComediSubdevice = Subdevice;

	return true;
}

bool CComediChannel::SetComediRange(unsigned int Range)
{
	m_ComediRange = Range;

	return true;
}

bool CComediChannel::SetComediReference(unsigned int Reference)
{
	m_ComediReference = Reference;

	return true;
}

bool CComediChannel::SetOffset(float Offset)
{
	m_Offset = Offset;

	return true;
}

bool CComediChannel::SetRange(float Range)
{
	m_Range = Range;
	
	return true;
}
