/***************************************************************************
                          comedichannel.h  -  description
                             -------------------
    begin                : Tue Mar 6 2001
    copyright            : (C) 2001 by Ivan Martinez
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef COMEDICHANNEL_H
#define COMEDICHANNEL_H

#include <fixedoperationblock.h>

/**
  *@author Ivan Martinez
  */

#include <fixedoutput.h>

class CComediChannel : public CFixedOperationBlock  {
public:
	/** Constructor */
	CComediChannel();
	
	/** Destructor */
	~CComediChannel();

	/** Returns "CComediChannel" */
	virtual string Class() const;

	/** Returns the "chan" argument for comedilib function calls */
	unsigned int ComediChannel() const;

	/** Returns the "minor" argument for comedilib funcion calls */
	unsigned int ComediMinor() const;

	/** Returns the "range" argument for comedilib function calls */
	unsigned int ComediRange() const;

	/** Returns the "aref" argument for comedilb function calls */
	unsigned int ComediReference() const;

	/** Returns the "subdev" argument for comedilib function calls */
	unsigned int ComediSubdevice() const;

	/** Returns TRUE if the argument is "CComediChannel" or one of its
		base classes, otherwise FALSE */
	virtual bool DescendantOf(const string &) const;

	/** Returns the offset of the block's input or output */
	float Offset() const;

	/** Returns the range of the block's input or output */
	float Range() const;

	/** Sets the "chan" argument for comedilib function calls */
	bool SetComediChannel(unsigned int);

	/** Sets the "minor" argument for comedilib function calls */
	bool SetComediMinor(unsigned int);

	/** Sets the "range" argument for comedilib function calls */
	bool SetComediRange(unsigned int);

	/** Sets the "aref" argument for comedilib function calls */
	bool SetComediReference(unsigned int);

	/** Sets the "subdev" argument for comedilib function calls */
	bool SetComediSubdevice(unsigned int);

	/** Sets the offset of the block's input or output */
	bool SetOffset(float);

	/** Sets the range of the block's input or output */
	bool SetRange(float );
protected: // Protected methods
	/** Generates code for the section of the component indicated in the
		arguments. Lines repeated in the same section are eliminated */
	virtual void GenerateCode(CStringsSet&, const CCodeGenerator*,
		const string& , void*) const;

	/** Calls a code generation function depending on the arguments */
	virtual void GenerateCode(ostream& , const CCodeGenerator* ,
		const string& , void*) const;

	/** Generates MaximumData() variable declaration */
	virtual void GenerateDeclarations(ostream&, void*) const;

	/** Generates Comedi device closing */
	virtual void GenerateDevicesClosing(CStringsSet &, void *) const;

	/** Generates Comedi device opening */
	virtual void GenerateDevicesOpening(CStringsSet &, void *) const;

	/** Generates "include" directives declaration */
	virtual void GenerateIncludes(CStringsSet & , void * ) const;

	/** Generates MaximumData() variable initialization */
	virtual void GenerateInitialization(ostream&, void*) const;

	/** Generates Comedi device locking  */
	virtual void GenerateSubdevicesLocking(CStringsSet &, void *) const;

	/** Generates Comedi device unlocking */
	virtual void GenerateSubdevicesUnlocking(CStringsSet &, void *) const;

	/** Returns the variable that stores the maximum value in the Comedi
		channel */
	string MaximumData() const;
private: // Private attributes
	/** "chan" argument for comedilib function calls */
	unsigned int m_ComediChannel;

	/** "minor" argument for comedilib function calls */
	unsigned int m_ComediMinor;

	/** "range" argument for comedilb function calls */
	unsigned int m_ComediRange;

	/** "aref" argument for comedilib function calls */
	unsigned int m_ComediReference;

	/** "subdevice" argument for comedlib function calls */
	unsigned int m_ComediSubdevice;

	/** Offset of the input or output variable */
	float m_Offset;

	/** Range of the input or output variable */
	float m_Range;
};

#endif
