/***************************************************************************
                          dynamicsystem.h  -  description
                             -------------------
    begin                : Thu Feb 10 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DYNAMICSYSTEM_H
#define DYNAMICSYSTEM_H

#include <rtmodule.h>

/**
  *@author Ivan Martinez
  */

#include <function.h>
#include <time.h>
#include <vector>
#include <string>
#include <loop.h>
#include <variable.h>

class CDynamicSystem : public CRTModule {
public:
	/** Constructor */
	CDynamicSystem();

	/** Destructor */
	virtual ~CDynamicSystem();

	/** Generates the error 101 and retuns false */
	virtual bool AcceptsContainer(const CCodeGenerator*);

	/** Returns FALSE if this component reported errors. If not, calls
		AllArrange for all its components and returns FALSE if any component
		reported errors. If no error was reported, calls
		ComponentsDependingArrange.	Returns FALSE if this call reported errors,
		TRUE in other case */
	bool ArrangeAll(void* = NULL);

	/** Returns "CDynamicSystem" */
	virtual string Class() const;

	/** Returns if the argument is "CDynamicSystem" or one of its base
		classes */
	virtual bool DescendantOf(const string&) const;

	/** Calls a code generation function depending on the arguments.
		Lines repeated in the same section are eliminated */
	virtual void GenerateCode(ostream&, const CCodeGenerator*, const string&,
		void* = NULL) const;

	/** Generates the C source code for the module that implements the
		dynamic system */
	virtual void GenerateSource(ostream&, void* = NULL) const;

	/** Returns the aditional "include" directives for the module */
	const string& Includes() const;

	/** Sets the aditional "include" directives for the module */
	bool SetIncludes(const string& Includes);

	/** Sets the maximum execution time in nanoseconds */
	bool SetTime(long long);

	/** Returns the maximum execution time in nanoseconds */
	long long Time() const;
private: // Private attributes
	/** Aditional "include" directives for the module */
	string m_Includes;

	/** Maximum execution time in nanoseconds */
	long long m_llTime;
};

#endif
