/***************************************************************************
                          gain.h  -  description
                             -------------------
    begin                : Fri Feb 23 2001
    copyright            : (C) 2001 by Ivan Martinez
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GAIN_H
#define GAIN_H

#include <fixedoperationblock.h>

/**
  *@author Ivan Martinez
  */

#include <fixedinput.h>
#include <fixedoutput.h>

class CGain : public CFixedOperationBlock  {
public:
	/** Type of the gain factor */
	enum EFactorType {CONSTANT, VARIABLE};
	
	/** Constructor */
	CGain();

	/** Destructor */	
	~CGain();
	
	/** Returns "CGain" */
	virtual string Class() const;
	
	/** Returns the gain factor when it's a constant */
	float ConstantFactor() const;
	
	/** Returns TRUE if the argument is "CGain" or one of its
		base classes, otherwise FALSE */
	virtual bool DescendantOf(const string&) const;
	
	/** Returns the input */
	CFixedInput* Input() const;
	
	/** Returns the output */
	CFixedOutput* Output() const;

	/** Sets the gain factor as a constant */
	bool SetConstantFactor(float);
	
	/** Sets the gain factor as a variable */
	bool SetVariableFactor(const string&);

	/** Returns the gain factor when it's a variable */
	const string VariableFactor() const;
protected: // Protected methods
	/** Generates the function that executes the block */
	virtual void GenerateFunctions(ostream&, void* = NULL) const;
private: // Private attributes
	/** Type of the gain factor */
	EFactorType m_FactorType;
	
	/** Gain factor when it's a constant */
	float m_rConstantFactor;
	
	/** Gain factor when it's a variable */
	string m_VariableFactor;
};

#endif
