/***************************************************************************
                          initializablevariable.h  -  description
                             -------------------
    begin                : Mon Jun 5 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef INITIALIZABLEVARIABLE_H
#define INITIALIZABLEVARIABLE_H

#include <dualvariable.h>

/**
  *@author Ivan Martinez
  */

class CInitializableVariable : public CDualVariable  {
public:
	/** Constructor */
	CInitializableVariable();

	/** Destructor */
	~CInitializableVariable();

	/** Returns TRUE if the argument is "CDualVarible" or one of its base
		classes, otherwise FALSE */
	virtual bool DescendantOf(const string&) const;

	/** Returns the variable's initialization */
	const string& Initialization() const;

	/** Returns if the variable has initialization */
	bool Initialized() const;
protected:
	/** Generates the variable's initialization */
	virtual void GenerateInitialization(ostream&, void* = NULL) const;

	/** Generates the local variable declaration if Initialized() */
	virtual void GenerateInitializationDeclaration(ostream&,
		void* = NULL) const;

	/** Generates an assignation from the local to the global variable if
		Initialized() */
	virtual void GenerateInitializationWriting(ostream&, void* = NULL) const;

	/** Sets the variable's initialization */
	void SetInitialization(const string&);
private: // Private attributes
	/** Variables's initialization */
	string m_Initialization;
};

#endif
