/***************************************************************************
                          input.cpp  -  description
                             -------------------
    begin                : Mon Mar 27 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <input.h>

#include <block.h>
#include <loop.h>

CInput::CInput()
{
	m_pConnectedOutput = NULL;
}

CInput::~CInput()
{
}

const COutput* CInput::ConnectedOutput() const
{
	return m_pConnectedOutput;
}

bool CInput::Connect(COutput* pOutput)
{
	m_pConnectedOutput = pOutput;

	return true;
}

bool CInput::DescendantOf(const string& ClassNameL) const
{
	if (ClassNameL == "CInput")
		return true;
	else
		return CInitializableVariable::DescendantOf(ClassNameL);
}

void CInput::Arrange(void* pExtraParameter = NULL)
{
	if (IllegalCIdentifier(Identifier()))
		AddError(600);
	else
		if (IllegalCIdentifier(Type()))
			AddError(602);
		else
			if (Connected())
				SetGlobalIdentifier("iovar_" + Container()->Container()->Identifier() +
					"_" + ConnectedOutput()->Container()->Name() + "_" +
					ConnectedOutput()->Identifier());
			else
				AddError(1400);
}

bool CInput::Connected() const
{
	return (ConnectedOutput() != NULL);
}

bool CInput::ConnectedToOrderedBlock(const CCodeGeneratorPointers& OrderedComponents) const
{
	if (Connected())
		return OrderedComponents.Contains(ConnectedOutput()->Container());
	else
		return false;
}

bool CInput::SetInitialization(const string& Initialization)
{
	CInitializableVariable::SetInitialization(Initialization);

	return true;
}

bool CInput::SetName(const string& Name)
{
	SetIdentifier(Name);

	if (IllegalCIdentifier(Name))
	{
		AddError(600);
		return false;
	}	
	else
		return true;
}

const string& CInput::Name() const
{
	return Identifier();
}

void CInput::GenerateCode(ostream& Stream, const CCodeGenerator* pCaller,
	const string& Section, void* pExtraParameter = NULL) const
{
	if (pCaller->DescendantOf("CBlock"))
	{
		if (Section == "initialized variables declaration")
			GenerateInitializationDeclaration(Stream, pExtraParameter);
		else
			if (Section == "variables initialization")
				GenerateInitialization(Stream, pExtraParameter);
			else
				if (Section == "initialized variables writing")
					GenerateInitializationWriting(Stream, pExtraParameter);
				else
					if (Section == "local IO variables declaration")
						GenerateLocalDeclaration(Stream, pExtraParameter);
					else			
						if (Section == "inputs reading")
							GenerateGlobalReading(Stream, pExtraParameter);
	}
}
