/***************************************************************************
                          input.h  -  description
                             -------------------
    begin                : Mon Mar 27 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef INPUT_H
#define INPUT_H

#include <initializablevariable.h>

/**
  *@author Ivan Martinez
  */

#include <loggingfifo.h>
#include <output.h>

class CBlock;

class CInput : public CInitializableVariable {
public:
	/** Constructor */
	CInput();

	/** Destructor */
	~CInput();

	/** Connects this input to the output of the parameters */
	bool Connect(COutput*);

	/** Returns if this input is connected */
	bool Connected() const;

	/** Returns the output connected to this input */
	const COutput* ConnectedOutput() const;

	/** Returns if the connected output belongs to an ordered block */
	bool ConnectedToOrderedBlock(const CCodeGeneratorPointers&) const;

	/** Returns TRUE if the argument is "CInput" or one of its base classes,
		otherwise FALSE */
	virtual bool DescendantOf(const string&) const;

	/** Returns the name of this component */
	const string& Name() const;
	
	/** Sets the variable's initialization and returns TRUE */
	bool SetInitialization(const string&);
	
	/** Sets the name of this component */
	bool SetName(const string&);
protected:
	/** If IllegalCIdentifier(Identifier()) generates the error 600.
		If IllegalCIdentifier(Type()) generates the error 602. If the variable
		is not connected to an output generates the error 1400. Otherwise sets
		the identifier of the global variable */
	virtual void Arrange(void* = NULL);

	/** Calls a code generation function depending on the arguments */
	virtual void GenerateCode(ostream&, const CCodeGenerator*, const string&,
		void* = NULL) const;
private: // Private attributes
	/** Output connected to this input */
	const COutput* m_pConnectedOutput;
};

#endif
