/***************************************************************************
                          reportingfifo.h  -  description
                             -------------------
    begin                : Thu Jun 1 2000
    copyright            : (C) 2000 by Ivan Martinez
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef REPORTINGFIFO_H
#define REPORTINGFIFO_H

#include <rtfifo.h>

/**
  *@author Ivan Martinez
  */

class CLoop;

class CReportingFifo : public CRTFifo  {
public:
	/** Constructor */
	CReportingFifo();

	/** Destructor */
	~CReportingFifo();

	/** If the paramater is not DescendantOf("CLoop"),
		generates the error 101 and returns false. If it is, but the container
		already contains a reporting fifo, generates the error 1800 and returns
		false. Otherwise sets the parameter as the container of this component
		and returns true */
	virtual bool AcceptsContainer(const CCodeGenerator*);

	/** Returns "CReportingFifo" */
	virtual string Class() const;

	/** Returns the container of this component */
	const CLoop* Container() const;

	/** Returns TRUE if the argument is "CReportingFifo" or one of its base
		classes, otherwise FALSE */
	virtual bool DescendantOf(const string&) const;

	/** Returns the number of the associated device file "/dev/rtf*" */
	unsigned int Number() const;

	/** Sets the number of the associated device file "/dev/rtf*" */
	bool SetNumber(unsigned int);

	/** Sets the size in bytes of the fifo */
	bool SetSize(int);

	/** Returns the size in bytes of the fifo */
	int Size();
protected:
	/** Sets the type of access to the fifo as WRITE and the identifier of
		the fifo */
	virtual void Arrange(void* = NULL);

	/** Calls a code generation function depending on the arguments */
	virtual void GenerateCode(ostream&, const CCodeGenerator*, const string&,
		void* = NULL) const;

	/** Calls a code generation function depending on the arguments.
		Lines repeated in the same section are eliminated */
	virtual void GenerateCode(CStringsSet&, const CCodeGenerator*,
		const string&, void* = NULL) const;

	/** Generates logging of the time at the end of an execution of the loop */
	virtual void GenerateEndTimeLog(ostream&, void* = NULL) const;

	/** Generates "include" directives declaration */
	virtual void GenerateIncludes(CStringsSet&, void* = NULL) const;

	/** Generates the variables that store the message and the time
		to be writed in the fifo */
	virtual void GenerateLoopDeclarations(CStringsSet&, void* = NULL) const;

	/** Generates logging of the time at the begin of an execution of
		the loop */
	virtual void GenerateStartTimeLog(ostream&, void* = NULL) const;
};

#endif