/***************************************************************************
                          settingsharedmemory.cpp  -  description
                             -------------------
    begin                : Tue Feb 13 2001
    copyright            : (C) 2001 by Ivan Martinez
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <settingsharedmemory.h>

#include <initializablevariable.h>
#include <systemvariable.h>

CSettingSharedMemory::CSettingSharedMemory()
{
}

CSettingSharedMemory::~CSettingSharedMemory()
{
}

bool CSettingSharedMemory::AcceptsContainer(const CCodeGenerator* pContainer)
{
	if (pContainer->DescendantOf("CInitializableVariable"))
	{
		if (pContainer->Components().DescendantsOf("CSettingSharedMemory"))
		{
			AddError(2600);
			return false;
		}
		else
		{
			SetContainer(pContainer);
			return true;
		}
	}
	else
	{
		AddError(101);
		return false;
	}
}

void CSettingSharedMemory::Arrange(void* pExtraParameter = NULL)
{
	SetIdentifier(string("setsharedmem_" + Container()->Identifier()));
	SetKernelMemoryPointer("&" + Container()->GlobalIdentifier());
	SetDataType(Container()->Type());
	SetDataSize("sizeof(" + DataType() + ")");
	SetLockIdentifier("sharedmemlock_" + Container()->Identifier());
	SetLockType("char");
	SetLockSize("sizeof(char)");
}

string CSettingSharedMemory::Class() const
{
	return "CSettingSharedMemory";
}

const CInitializableVariable* CSettingSharedMemory::Container() const
{
	return (CInitializableVariable*)CProtectedSharedMemory::Container();
}

void CSettingSharedMemory::GenerateCode(ostream& Stream,
	const CCodeGenerator* pCaller, const string& Section,
	void* pExtraParameter = NULL) const
{
	if (pCaller->DescendantOf("CDynamicSystem"))
	{
		if (Section == "module I/Os declaration")
			GenerateDeclaration(Stream, pExtraParameter);
		else
			if (Section == "module I/Os initialization")
				GenerateInitialization(Stream, pExtraParameter);
	}
	else
		if (pCaller->DescendantOf("CLoop"))
			if (Section == "variables set")
				GenerateMemoryReading(Stream, pExtraParameter);
			else
				if (Section == "global variables set")
					GenerateLockedMemoryReading(Stream, pExtraParameter);
}

void CSettingSharedMemory::GenerateLockedMemoryReading(ostream& Stream,
	void* pExtraParameter = NULL) const
{
	if (Container()->Class() == "CSystemVariable")
	{
		GenerateLockAcquisition(Stream, pExtraParameter);
		Stream << "        rtl_spin_lock_irqsave(&" <<
			((CSystemVariable*)Container())->LockIdentifier() << ", flags);" <<
			endl;
		Stream << "        memcpy(" << KernelMemoryPointer() << ", " <<
			Identifier() << ", " << DataSize() << ");" << endl;
		Stream << "        rtl_spin_unlock_irqrestore(&" <<
			((CSystemVariable*)Container())->LockIdentifier() << ", flags);" <<
			endl;
		GenerateLockRelease(Stream, pExtraParameter);
	}
}
