/***************************************************************************
                          systemvariable.cpp  -  description
                             -------------------
    begin                : Tue Apr 18 2000
    copyright            : (C) 2000 by 
    email                : ivanmr@altavista.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <systemvariable.h>
#include <dynamicsystem.h>

CSystemVariable::CSystemVariable()
{
}

CSystemVariable::~CSystemVariable()
{
}

bool CSystemVariable::AcceptsContainer(const CCodeGenerator* pContainer)
{
	if (pContainer->DescendantOf("CDynamicSystem"))
	{
		SetContainer(pContainer);
		return true;
	}
	else
	{
		AddError(101);
		return false;
	}
}

string CSystemVariable::Class() const
{
	return "CSystemVariable";
}

const CDynamicSystem* CSystemVariable::Container() const
{
	return (CDynamicSystem*)CCodeGenerator::Container();
}

bool CSystemVariable::DescendantOf(const string& ClassNameL) const
{
	if (ClassNameL == "CSystemVariable")
		return true;
	else
		return CInitializableVariable::DescendantOf(ClassNameL);
}

void CSystemVariable::GenerateCode(CStringsSet& StringsSetL,
	const CCodeGenerator* CallerPL, const string& SectionL,
	void* ExtraParameterL= NULL) const
{
	if (CallerPL->DescendantOf("CDynamicSystem"))
	{
		if (SectionL == "includes")
			GenerateIncludes(StringsSetL, ExtraParameterL);
  }
	else
		if (CallerPL->DescendantOf("CBlock"))
			if (SectionL == "flags declaration")
				GenerateFlagsDeclaration(StringsSetL, ExtraParameterL);
}

void CSystemVariable::GenerateCode(ostream& StreamL, const CCodeGenerator* CallerPL,
	const string& SectionL, void* ExtraParameterL = NULL) const
{
	if (CallerPL->DescendantOf("CDynamicSystem"))
	{
		if (SectionL == "global variables declaration")
			GenerateGlobalDeclarations(StreamL, ExtraParameterL);
		else
			if (SectionL == "initialized variables declaration")
				GenerateInitializationDeclaration(StreamL, ExtraParameterL);
			else
				if (SectionL == "system variables initialization")
					GenerateInitialization(StreamL, ExtraParameterL);
				else
					if (SectionL == "initialized variables writing")
						GenerateInitializationWriting(StreamL, ExtraParameterL);
	}
	else
		if (CallerPL->DescendantOf("CBlock"))
			if (SectionL == "local system variable declarations")
				GenerateLocalDeclaration(StreamL, ExtraParameterL);
			else			
				if (SectionL == "system variables reading")
					GenerateGlobalReading(StreamL, ExtraParameterL);
				else
					if (SectionL == "system variables writing")
						GenerateGlobalWriting(StreamL, ExtraParameterL);
}

void CSystemVariable::GenerateFlagsDeclaration(CStringsSet& StringsSet,
	void* pExtraParameter = NULL) const
{
	StringsSet.insert("  static unsigned long flags;");
}

void CSystemVariable::GenerateGlobalDeclarations(ostream& StreamL,
	void* ExtraParameterL = NULL) const
{
	StreamL << "spinlock_t " << LockIdentifier() << " = SPIN_LOCK_UNLOCKED;" << endl;
	StreamL << Type() << " " << GlobalIdentifier() << ";" << endl;
}

void CSystemVariable::GenerateGlobalReading(ostream& StreamL,
	void* ExtraParameterL = NULL) const
{
	StreamL << "  rtl_spin_lock_irqsave(&" << LockIdentifier() << ", flags);" << endl;
	StreamL << "  " << Identifier() << " = " << GlobalIdentifier() << ";" << endl;
	StreamL << "  rtl_spin_unlock_irqrestore(&" << LockIdentifier() << ", flags);" << endl;
}

void CSystemVariable::GenerateGlobalWriting(ostream& StreamL,
	void* ExtraParameterL = NULL) const
{
	StreamL << "  rtl_spin_lock_irqsave(&" << LockIdentifier() << ", flags);" << endl;
	StreamL << "  " << GlobalIdentifier() << " = " << Identifier() << ";" << endl;
	StreamL << "  rtl_spin_unlock_irqrestore(&" << LockIdentifier() << ", flags);" << endl;
}

void CSystemVariable::GenerateIncludes(CStringsSet& StringsSetL,
	void* ExtraParameterL= NULL) const
{
	StringsSetL.insert("#include <rtl_sync.h>");
}

void CSystemVariable::GenerateLocalDeclaration(ostream& Stream,
	void* pExtraParameter = NULL) const
{
	/* This is necesary because otherwise in the call of
	GenerateLocalDeclarations(Stream, ...) in GenerateCode(ostream&, ...)
	the compiler reports an error, because it thinks it is a call of
	GenerateLocalDeclarations(CStringsSet&, ...) with wrong argument
	type. */
	CVariable::GenerateLocalDeclaration(Stream, pExtraParameter);
}

const string& CSystemVariable::LockIdentifier() const
{
	return m_LockIdentifier;
}

const string& CSystemVariable::Name() const
{
	return Identifier();
}

bool CSystemVariable::SetInitialization(const string& InitializationL)
{
	CInitializableVariable::SetInitialization(InitializationL);

	return true;
}

void CSystemVariable::SetLockIdentifier(const string& LockIdentifier)
{
	m_LockIdentifier = LockIdentifier;
}

bool CSystemVariable::SetName(const string& NameL)
{
	SetIdentifier(NameL);

	if (IllegalCIdentifier(NameL))
	{
		AddError(600);
		return false;
	}	
	else
	{
		SetGlobalIdentifier("systemvar_" + Identifier());
		SetLockIdentifier("lock_" + Identifier());
		return true;
	}
}

bool CSystemVariable::SetType(const string& TypeL)
{
	CVariable::SetType(TypeL);

	if (IllegalCIdentifier(TypeL))
	{
		AddError(602);
		return false;
	}
	else
		return true;
}
