#define MODULE
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/errno.h>

#include <linux/rt_sched.h>

#include "../../rt_ipc/rt_ipc.h"

#define DBG_PRINT_SETUP		rt_sem_t print_sem;
#define DBG_PRINT_INIT		rt_sem_init(&print_sem, RT_SEM_BINARY, 1);
#define TAKE_PRINT		rt_sem_wait(&print_sem, RT_WAIT_FOREVER);
#define GIVE_PRINT		rt_sem_post(&print_sem);

DBG_PRINT_SETUP

RT_TASK_IPC init_task;

#define RT_TO_LINUX	1
#define LINUX_TO_RT	2

#define CHUNKSIZE	10

/* Waits for data from the Linux process, then echoes it back */
void init_task_code(int notused)
{
  char buf[CHUNKSIZE+1];
  DBG_PRINT_INIT;
  if (rtf_ipc_create(RT_TO_LINUX, 200, 1) < 0 ||
      rtf_ipc_create(LINUX_TO_RT, 200, 0) < 0)
  {
    TAKE_PRINT; printk("rt_test: rtf_ipc_create() failed\n"); GIVE_PRINT;
    return;
  }
  for ( ; ; )
  {
    int ret;
    ret = rtf_receive(LINUX_TO_RT, buf, CHUNKSIZE, RT_WAIT_FOREVER);
    if (ret < 0)
      { TAKE_PRINT; printk("rt_test: rtf_receive() failed\n"); GIVE_PRINT; }
    buf[ret] = '\0';
    ret = rtf_send(RT_TO_LINUX, buf, ret, RT_WAIT_FOREVER);
    if (ret < 0)
      { TAKE_PRINT; printk("rt_test: rtf_send() failed\n"); GIVE_PRINT; }
  }

  /* nothing more for this task to do */
  rt_task_suspend(MAKE_RT_TASK(&init_task));
}

int init_module(void)
{
  if (rt_task_ipc_init(&init_task, init_task_code, 0, 3000, 10) != 0)
    printk("rt_test: Could not start init task\n");
  rt_task_wakeup(MAKE_RT_TASK(&init_task));
  return 0;
}

void cleanup_module(void)
{
  rtf_ipc_destroy(RT_TO_LINUX);
  rtf_ipc_destroy(LINUX_TO_RT);

  rt_task_ipc_delete(&init_task);
}
