//
// Real Time Linux Example Widget: Logic (logic.h)
//
// Definition of Logic widget.
//
// The Logic widget is a stand-alone widget that you may
// freely use in your Qt application (at your own risk!)
//

#ifndef LOGIC_H
#define LOGIC_H

#include "das1600.h"

#include <stdio.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>



#include <qdialog.h>

#include <qpixmap.h>
#include <qapp.h>


#include <qtimer.h>
#include <qevent.h>

#include <qbttngrp.h>
#include <qchkbox.h>
#include <qcombo.h>
#include <qframe.h>
#include <qgrpbox.h>
#include <qlabel.h>
#include <qlcdnum.h>
#include <qlined.h>
#include <qlistbox.h>
#include <qpushbt.h>
#include <qradiobt.h>
#include <qscrbar.h>




class LogicTimer : public QTimer
{
    Q_OBJECT
public:
    LogicTimer( QWidget *parent );
    enum { MAXSPEED = 1000 };

public slots:

private:
    int         interval;
};





class LogicCanvas : public QFrame
{
    Q_OBJECT
public:
    LogicCanvas( QWidget *parent = 0, const char *name = 0 );

    void        setPoint( int i, int j );

    int         maxCol() { return maxi; }
    int         maxRow() { return maxj; }
    int         holdDisplay;
    void        voltage(int);
    void        timediv(int);
    int         trigger(int);
    int         voltage_scale;
    int         time_scale;
    int         wait_for_trigger;
public slots:
    void	drawSignals();
    void        set_voltage_scale(int);
    void        set_time_scale(int);
    void        hold();
    void        set_trigger();
private slots:
    void        clear();

protected:

private:
    int         dio;
    enum { SCALE = 10, MAXSIZE = 50, MINSIZE = 10, BORDER = 5 };

    bool        cells[2][MAXSIZE + 2][MAXSIZE + 2];
    LogicTimer  *timer;
    int		normalTimer;
    int         current;
    int         maxi, maxj;
    int         oldx, oldz;
    int         x, y, z, zz, i, oldy;

    unsigned char inbyte;
    static int pos2index( int x )
    {
        return ( x - BORDER ) / SCALE + 1;
    }
    static int index2pos( int i )
    {
        return  ( i - 1 ) * SCALE + BORDER;
    }

};




class Logic : public QFrame		// digital clock widget
{
    Q_OBJECT
public:
    Logic( QWidget *parent=0, const char *name=0 );
    int time_scale;
protected:					// event handlers

private slots:					// internal slots
    void	checkBoxClicked( int );
private:					// internal data
    QPushButton  *pb;
    QPushButton  *pb1;
    LogicCanvas *logic;
    bool	showingColon;
    int         x,y,oldx,oldy;
    unsigned int inbyte;
    int		showDateTimer;
    QLabel      *msg;
    QCheckBox   *cb[32];
};




#endif // LOGIC_H


