%{
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include "../include/rt_fix.h"
#include "../include/rt_fix.h"
#include "./rt_fix_common.h"

extern int yylineno;
int        fd0;
int        fd1;
rt_fix_msg rt_msg;
rt_fix     minusone;

#define prompt  printf("\n%5d : ", yylineno);

static rt_fix read_result();
static int send_operation(int cmd, rt_fix left_operand, rt_fix right_operand);
%}

%union {
    rt_fix number;
    char   string[100];
}

%start lines
%token <string> NEWLINE     ADD     SUB     MUL     DIV     SQRT
%token <string> SIN         COS     TAN     ASIN    ACOS    FRAC
%token <string> LDEXP       ABS     FLOOR   CEIL    EXP     ROUND
%token <string> NOTHING     OPEN    CLOSE 
%token <number> NUMBER
%type  <number> expression term factor 


%%
lines       :   /* empty => no line */
            |   expression
                    { printf("%f", rt_fix_tod($1)); prompt; }
                NEWLINE lines


expression  :   expression ADD term
                    { send_operation(RT_FIX_ADD, $1, $3);
                      $$ = read_result(); }
            |   expression SUB term
                    { send_operation(RT_FIX_SUB, $1, $3);
                      $$ = read_result(); }
            |   term
                    { $$ = $1; }
            ;

term        :   term MUL factor
                    { send_operation(RT_FIX_MUL, $1, $3);
                      $$ = read_result(); }
            |   term DIV factor
                    { send_operation(RT_FIX_DIV, $1, $3);
                      $$ = read_result(); }
            |   factor
                    { $$ = $1; }
            ;

factor      :   OPEN expression CLOSE
                    { $$ = $2; }
            |   NUMBER
                    { $$ = yylval.number; }
            |   SUB NUMBER
                    { send_operation(RT_FIX_MUL, minusone, $2);
                      $$ = read_result(); }
            |   SQRT OPEN expression CLOSE
                    { /* third argument of send_operation is don't care */
                      send_operation(RT_FIX_SQRT, $3, minusone);
                      $$ = read_result(); }
            |   EXP OPEN expression CLOSE
                    { /* third argument of send_operation is don't care */
                      send_operation(RT_FIX_EXP, $3, minusone);
                      $$ = read_result(); }
            |   SIN OPEN expression CLOSE
                    { /* third argument of send_operation is don't care */
                      send_operation(RT_FIX_SIN, $3, minusone);
                      $$ = read_result(); }
            |   ASIN OPEN expression CLOSE
                    { /* third argument of send_operation is don't care */
                      send_operation(RT_FIX_ASIN, $3, minusone);
                      $$ = read_result(); }
            |   COS OPEN expression CLOSE
                    { /* third argument of send_operation is don't care */
                      send_operation(RT_FIX_COS, $3, minusone);
                      $$ = read_result(); }
            |   ACOS OPEN expression CLOSE
                    { /* third argument of send_operation is don't care */
                      send_operation(RT_FIX_ACOS, $3, minusone);
                      $$ = read_result(); }
            |   TAN OPEN expression CLOSE
                    { /* third argument of send_operation is don't care */
                      send_operation(RT_FIX_TAN, $3, minusone);
                      $$ = read_result(); }
            |   ROUND OPEN expression CLOSE
                    { /* third argument of send_operation is don't care */
                      send_operation(RT_FIX_ROUND, $3, minusone);
                      $$ = read_result(); }
            |   FLOOR OPEN expression CLOSE
                    { /* third argument of send_operation is don't care */
                      send_operation(RT_FIX_FLOOR, $3, minusone);
                      $$ = read_result(); }
            |   CEIL OPEN expression CLOSE
                    { /* third argument of send_operation is don't care */
                      send_operation(RT_FIX_CEIL, $3, minusone);
                      $$ = read_result(); }
            |   FRAC OPEN expression CLOSE
                    { /* third argument of send_operation is don't care */
                      send_operation(RT_FIX_FRAC, $3, minusone);
                      $$ = read_result(); }
            |   LDEXP OPEN expression CLOSE
                    { /* third argument of send_operation is don't care
                      send_operation(RT_FIX_LDEXP, $3, minusone);
                      $$ = read_result()*/ ; }
            |   ABS OPEN expression CLOSE
                    { /* third argument of send_operation is don't care */
                      send_operation(RT_FIX_ABS, $3, minusone);
                      $$ = read_result(); }
            ;
%%

int
yyerror(char *msg)
{
    extern int yytok;

    fprintf(stderr,"%s near %s in line %d\n", msg,
                                              yylval.string,
                                              yylineno);
}

static rt_fix
read_result()
{
    int n = 0;

    if((n = read(fd1, (void *)&rt_msg, sizeof(rt_msg))) != sizeof(rt_msg)) {
        fprintf(stderr, "Read unknown message with %d bytes\n", n);
        perror(strerror(errno));
        exit(1);
    }
    return rt_msg.result;

}

static int
send_operation(int command, rt_fix left_operand, rt_fix right_operand)
{
    int err = 0;

    rt_msg.command = command;
    rt_msg.left_operand = left_operand;
    rt_msg.right_operand = right_operand;
    rt_fix_init_i(&rt_msg.result, 0);

    if(write(fd0, (void *)&rt_msg, sizeof(rt_msg)) < 0) {
        fprintf(stderr, "Can't send a command to RT-Linux\n");
        perror(strerror(errno));
        exit(1);
    }
    return err;
}

int
main(int argc, char **argv)
{
    int err = 0;

    if ((fd0 = open("/dev/rtf0", O_WRONLY)) < 0) {
        fprintf(stderr, "Error opening /dev/rtf0\n");
        perror(strerror(errno));
        exit(1);
    }
    if ((fd1 = open("/dev/rtf1", O_RDONLY)) < 0) {
        fprintf(stderr, "Error opening /dev/rtf1\n");
        perror(strerror(errno));
        exit(1);
    }
    rt_fix_init_i(&minusone, -1);
    prompt;
    yyparse();
    close(fd0);
    close(fd1);
}
