#!/usr/bin/X11/wish

#
# maximum value for low and high byte of rt-time
#
set MAX_RT_TICKS_LOW  4294967295
set MAX_RT_TICKS_HIGH 2147483647

proc rtTicks2Seconds {rtTicks} {
    return [expr $rtTicks / 1193180.0]
}

proc seconds2rtTicks {seconds} {
    return [expr $seconds * 1193180]
}

proc double2long {double} {
    set result ""
    regexp {[0-9]+} $double result
    return $result
}

proc setFrequency {frequency} {
    #
    # Toggle D0-Pin with twice the Frequency of the requested tone.
    #
    set ticks [double2long [seconds2rtTicks [expr 1.0 / (2.0 * $frequency.0)]]]

    # 
    # Set the period for the rt-task.
    #
    set file_id [open /proc/sys/rt_linux/0/period w]
    puts -nonewline $file_id "0 $ticks"
    close $file_id
}

set minFreq [double2long [expr 1 / (2 * 0.025)]]
set maxFreq [double2long [expr 1 / (2 * 0.0005)]]
scale .frequency -label Frequency            \
    -from $minFreq -to $maxFreq              \
    -orient horizontal                       \
    -length 5c                               \
    -tickinterval [expr $maxFreq - $minFreq] \
    -command setFrequency

button .quit -text Quit \
    -command {
        #
        # stop the rt-task
        #
        set file_id [open /proc/sys/rt_linux/0/start_time w]
        puts -nonewline $file_id "$MAX_RT_TICKS_HIGH $MAX_RT_TICKS_LOW"
        close $file_id
        exit
    }

pack .frequency .quit
