#define MODULE

#include <linux/module.h>   /* Imports Module Definitions */
#include <linux/kernel.h>   /* Imports some often used functions */
#include <linux/version.h>  /* Imports version number */
#include <linux/config.h>   /* Imports some often used functions */
#include <asm/io.h>         /* Imports outpb() */
#include "rt_registry.h"    /* Imports the RT-Task-Registry */

/*
 * The address of the parallel port
 */
#define LPT 0x378

/*
 * The Data-Bits of the LPT-Port.
 */
static unsigned char lpt_register = 0x00;

#define LPT_D0_TOGGLE { lpt_register = (unsigned char) inb(LPT); \
                        lpt_register = lpt_register ^ 0x01; \
                        outb(lpt_register, LPT); }

/*
 * RT-Task info.
 */
struct rt_task_info {
    RT_TASK rt_task;
    int id;
    char name[RT_REG_STRING_SIZE];
    char info[RT_REG_STRING_SIZE];
};
struct rt_task_info rt_info;

static void
d0_toggle(int t) {
    while (1) {
        LPT_D0_TOGGLE;
        rt_task_wait();
    }
}

static int
on_start_time_change(RTIME old_start_time,
                     RTIME *new_start_time)
{
    RTIME period = rt_reg_get_period(rt_info.id);

    if(NULL != new_start_time) {
        rt_task_delete(&rt_info.rt_task);
        rt_task_init(&rt_info.rt_task,
                d0_toggle, 0, 3000, 4);
        rt_task_make_periodic(&rt_info.rt_task,
                *new_start_time, period);
        if(RT_TIME_END == *new_start_time) {
            sprintf(rt_info.info,"%s","Ready");
            rt_reg_set_info(rt_info.id, rt_info.info);
        }
        return 0;
    }
    return 1;
}

static int
on_period_change(RTIME old_period,
                 RTIME *new_period)
{
    RTIME start_time = rt_get_time() + 100;

    if(NULL != new_period) {
        rt_task_delete(&rt_info.rt_task);
        rt_task_init(&rt_info.rt_task,
                d0_toggle, 0, 3000, 4);
        rt_task_make_periodic(&rt_info.rt_task,
                start_time, *new_period);
        sprintf(rt_info.info,"%s","Running");
        rt_reg_set_info(rt_info.id, rt_info.info);
        rt_reg_set_start_time(rt_info.id, start_time);
        return 0;
    }
    return 1;
}

int
init_module() {
    sprintf(rt_info.name,"%s","Sound");
    rt_task_init(&rt_info.rt_task, d0_toggle, 0, 3000, 4);
    rt_task_make_periodic(&rt_info.rt_task, RT_TIME_END, 29830);
    sprintf(rt_info.info,"%s","Ready");

    /*
     * register the rt-task and the event handler, which gets
     * triggered when the period is changed from user-space.
     */
    rt_info.id = rt_reg_register_task(&rt_info.rt_task,
                                      rt_info.name,
                                      on_start_time_change,
                                      on_period_change,
                                      NULL,
                                      NULL);
    rt_reg_set_info(rt_info.id, rt_info.info);

    return 0;

} /* End of init_module */

void
cleanup_module(void) {
    rt_reg_un_register_task(rt_info.id);
    rt_task_delete(&rt_info.rt_task);
} /* End of cleanup_module */
