/*
 * RT-LINUX Based Network Driver. Source File
 *
 * Written by Axel E. Bernal  	E-mail : abernal@nmt.edu
 * Released under the terms of the GNU Library General Public License
 * Some of the code has been stolen from skbuff.c eth.c and tulip.c
 */


#ifndef MODULE
#define MODULE
#endif

#define SMP_CHECK



#include "rt_tulip.h"

/* Kernel compatibility defines, common to David Hind's PCMCIA package.
   This is only in the support-all-kernels source code. */
#include <linux/version.h>		/* Evil, but neccessary */


/* 
 * 
 *
 * RTN Definitions and Functions 
 *
 *
 */

extern struct device *root_tulip_dev;

static struct rtn_global rtn_global;

extern void rtl_hard_enable_irq(int);

static void refill_skb_out_ring(struct device *);
static void refill_skb_pool_ring(struct device *);
void rtn_free_queue(void);
int rtn_cp_from_temp_rx(struct device *);
static void rtn_wake_sleepers(void);
inline void rtn_clear_out_queue(void);
inline void rtn_process_in_queue(void);

inline void rtn_skb_out_xmit(void);
int rtn_recv(int, char *, int);
inline void rtn_skb_out_xmit_in_irq(void);
static void rtn_tulip_tx_timeout(struct device *dev);
static int rtn_tulip_start_xmit(struct sk_buff *skb, struct device *dev);
static void rtn_tulip_interrupt(unsigned int irq, struct pt_regs *regs);
unsigned int rtn_interrupt(unsigned int irq, struct pt_regs *regs);
/* 
 * Sinchronization variables
 */
extern int volatile rtn_init_flag;
extern int volatile rtn_interrupt_flag;
int volatile rtn_change_dest_flag = 0;
/* 
 * Hardware Addresses 
 */
/* Destination address */
char daddr[ETH_ALEN];
/* = {0x00, 0x40, 0x05, 0xA1, 0xEB,0x71}; */
/* Source Address */
char saddr[ETH_ALEN];


/* Handler to perform copy to swap space...defined in tulip.c */
extern int (*copy_handler)(struct device *);

/* Handler to free transmition buffer memory... defined in tulip.c */
extern void (*freemem_handler)(void);

/* wake up/sleep variables, used to free memory used for the tx buffers */

static int volatile rtn_queue_irq = 0;
static int volatile rtn_start_tx_irq = 0;
static int volatile rtn_queue_to_wakeup = 0;


/* 
 * 
 *
 *TULIP Stuff : Functions and definitions
 *
 *
 */

static void rtn_tulip_timer(unsigned long data);
static void rtn_t21142_timer(unsigned long data);
static void rtn_t21142_lnk_change( struct device *dev);
static void rtn_mxic_timer(unsigned long data);
static void rtn_pnic_timer(unsigned long data);


static void mdio_write(struct device *, int, int , int);
static void select_media(struct device *, int);
static int mdio_read(struct device *, int, int);

/* 
 * Hardware dependent definitions -- tulip card 
 */
 
/* Maximum events (Rx packets, etc.) to handle at each interrupt. */
static int rtn_max_interrupt_work = 25;

/*  The possible media types that can be set in options[] are: */
static const char * const medianame[] = {
	"10baseT", "10base2", "AUI", "100baseTx",
	"10baseT-FD", "100baseTx-FD", "100baseT4", "100baseFx",
	"100baseFx-FD", "MII 10baseT", "MII 10baseT-FD", "MII",
	"10baseT(forced)", "MII 100baseTx", "MII 100baseTx-FD", "MII 100baseT4",
};


#if defined (LINUX_VERSION_CODE) && LINUX_VERSION_CODE < 0x10300
#define RUN_AT(x) (x)			/* What to put in timer->expires.  */
#define DEV_ALLOC_SKB(len) alloc_skb(len, GFP_ATOMIC)
#define virt_to_bus(addr)  ((unsigned long)addr)
#define bus_to_virt(addr) ((void*)addr)

#else  /* 1.3.0 and later */
#define RUN_AT(x) (jiffies + (x))
#define DEV_ALLOC_SKB(len) dev_alloc_skb(len + 2)
#endif

/* More hardware dependent definitions */

/* 21041 transceiver register settings: 10-T, 10-2, AUI, 10-T, 10T-FD*/
static u16 t21041_csr13[] = { 0xEF05, 0xEF09, 0xEF09, 0xEF01, 0xEF09, };
static u16 t21041_csr14[] = { 0x7F3F, 0xF7FD, 0xF7FD, 0x7F3F, 0x7F3D, };
static u16 t21041_csr15[] = { 0x0008, 0x0006, 0x000E, 0x0008, 0x0008, };

static u16 t21142_csr13[] = { 0x0001, 0x0009, 0x0009, 0x0000, 0x0001, };
static u16 t21142_csr14[] = { 0xFFFF, 0x0705, 0x0705, 0x0000, 0x7F3D, };
static u16 t21142_csr15[] = { 0x0008, 0x0006, 0x000E, 0x0008, 0x0008, };


#define EEPROM_SIZE 128
#if defined(__i386__)
#define get_u16(ptr) (*(u16 *)(ptr))
#else
#define get_u16(ptr) (((u8*)(ptr))[0] + (((u8*)(ptr))[1]<<8))
#endif


#ifndef PCI_VENDOR_ID_DEC		/* Now defined in linux/pci.h */
#define PCI_VENDOR_ID_DEC			0x1011
#define PCI_DEVICE_ID_TULIP			0x0002		/* 21040. */
#define PCI_DEVICE_ID_TULIP_FAST	0x0009		/* 21140. */
#endif

#ifndef PCI_DEVICE_ID_DEC_TULIP_PLUS
#define PCI_DEVICE_ID_DEC_TULIP_PLUS	0x0014		/* 21041. */
#endif
#ifndef PCI_DEVICE_ID_DEC_TULIP_21142
#define PCI_DEVICE_ID_DEC_TULIP_21142	0x0019
#endif

#ifndef PCI_VENDOR_ID_LITEON
#define PCI_VENDOR_ID_LITEON	0x11AD
#endif

#ifndef PCI_VENDOR_ID_MXIC
#define	PCI_VENDOR_ID_MXIC		0x10d9
#define PCI_DEVICE_ID_MX98713	0x0512
#define PCI_DEVICE_ID_MX98715	0x0531
#define PCI_DEVICE_ID_MX98725	0x0531
#endif


/* A table describing the chip types. */
enum tbl_flag { HAS_MII=1, HAS_MEDIA_TABLE = 2, CSR12_IN_SROM = 4,};
struct tulip_chip_table {
	int vendor_id, device_id;
	char *chip_name;
	int io_size;
	int valid_intrs;			/* CSR7 interrupt enable settings */
	int flags;
	void (*media_timer)(unsigned long data);
} tulip_tbl[] = {
  { PCI_VENDOR_ID_DEC, PCI_DEVICE_ID_DEC_TULIP,
	"Digital DC21040 Tulip", 128, 0x0001ebef, 0, rtn_tulip_timer },
  { PCI_VENDOR_ID_DEC, PCI_DEVICE_ID_DEC_TULIP_PLUS,
	"Digital DC21041 Tulip", 128, 0x0001ebef, HAS_MEDIA_TABLE, rtn_tulip_timer },
  { PCI_VENDOR_ID_DEC, PCI_DEVICE_ID_DEC_TULIP_FAST,
	"Digital DS21140 Tulip", 128, 0x0001ebef,
	HAS_MII | HAS_MEDIA_TABLE | CSR12_IN_SROM,
	rtn_tulip_timer },
  { PCI_VENDOR_ID_DEC, PCI_DEVICE_ID_DEC_TULIP_21142,
	"Digital DS21142/3 Tulip", 128, 0x0801fbff,
	HAS_MII | HAS_MEDIA_TABLE, rtn_t21142_timer },
  { PCI_VENDOR_ID_LITEON, 0x0002,
	"Lite-On 82c168 PNIC", 256, 0x0001ebef, HAS_MII, rtn_pnic_timer },
  { PCI_VENDOR_ID_MXIC, PCI_DEVICE_ID_MX98713,
	"Macronix 98713 PMAC", 128, 0x0001ebef,
	HAS_MII | HAS_MEDIA_TABLE | CSR12_IN_SROM, rtn_tulip_timer /* Tulip-like! */ },
  { PCI_VENDOR_ID_MXIC, PCI_DEVICE_ID_MX98715,
	"Macronix 98715 PMAC", 256, 0x0001ebef, HAS_MEDIA_TABLE, rtn_mxic_timer },
  { PCI_VENDOR_ID_MXIC, PCI_DEVICE_ID_MX98725,
	"Macronix 98725 PMAC", 256, 0x0001ebef, HAS_MEDIA_TABLE, rtn_mxic_timer },
  { 0x125B, 0x1400, "ASIX AX88140", 128, 0x0001fbff,
	HAS_MII | HAS_MEDIA_TABLE | CSR12_IN_SROM, rtn_tulip_timer },
  {0, 0, 0, 0},
};


/* This matches the table above. */
enum chips { DC21040=0, DC21041=1, DC21140=2, DC21142=3, DC21143=3,
			 LC82C168, MX98713, MX98715, MX98725};


#ifdef TULIP_DEBUG
int tulip_debug = TULIP_DEBUG;
#else
int tulip_debug = 1;
#endif

/* A full-duplex map for media types. */
enum MediaIs {MediaIsFD = 1, MediaAlwaysFD=2, MediaIsMII=4, MediaIsFx=8,
		  MediaIs100=16};
static const char media_cap[] =
{0,0,0,16,  3,19,16,24,  27,4,7,5, 0,20,23,20 };


/* Offsets to the Command and Status Registers, "CSRs".  All accesses
   must be longword instructions and quadword aligned. */
enum tulip_offsets {
	CSR0=0,    CSR1=0x08, CSR2=0x10, CSR3=0x18, CSR4=0x20, CSR5=0x28,
	CSR6=0x30, CSR7=0x38, CSR8=0x40, CSR9=0x48, CSR10=0x50, CSR11=0x58,
	CSR12=0x60, CSR13=0x68, CSR14=0x70, CSR15=0x78 };

/* The bits in the CSR5 status registers, mostly interrupt sources. */
enum status_bits {
	TimerInt=0x800, TPLnkFail=0x1000, TPLnkPass=0x10,
	NormalIntr=0x10000, AbnormalIntr=0x8000,
	RxJabber=0x200, RxDied=0x100, RxNoBuf=0x80, RxIntr=0x40,
	TxFIFOUnderflow=0x20, TxJabber=0x08, TxNoBuf=0x04, TxDied=0x02, TxIntr=0x01,
};


/* The Tulip Rx and Tx buffer descriptors. */
struct tulip_rx_desc {
	s32 status;
	s32 length;
	u32 buffer1, buffer2;
};

struct tulip_tx_desc {
	s32 status;
	s32 length;
	u32 buffer1, buffer2;				/* We use only buffer 1.  */
};

struct medialeaf {
	u8 type;
	u8 media;
	unsigned char *leafdata;
};

struct mediatable {
	u16 defaultmedia;
	u8 leafcount, csr12dir;				/* General purpose pin directions. */
	unsigned has_mii:1, has_nonmii:1;
	struct medialeaf mleaf[0];
};

struct mediainfo {
	struct mediainfo *next;
	int info_type;
	int index;
	unsigned char *info;
};

struct tulip_private {
	char devname[8];			/* Used only for kernel debugging. */
	const char *product_name;
	struct device *next_module;
	struct tulip_rx_desc rx_ring[RX_RING_SIZE];
	struct tulip_tx_desc tx_ring[TX_RING_SIZE];
	/* The saved address of a sent-in-place packet/buffer, for skfree(). */
	struct sk_buff* tx_skbuff[TX_RING_SIZE];
	/* The addresses of receive-in-place skbuffs. */
	struct sk_buff* rx_skbuff[RX_RING_SIZE];
	char *rx_buffs;				/* Address of temporary Rx buffers. */
	u32 setup_frame[48];		/* Pseudo-Tx frame to init address table. */
	int chip_id;
	int revision;
#if LINUX_VERSION_CODE > 0x20139
	struct net_device_stats stats;
#else
	struct enet_statistics stats;
#endif
	struct timer_list timer;	/* Media selection timer. */
	int interrupt;				/* In-interrupt flag. */
#ifdef SMP_CHECK
	int smp_proc_id;			/* Which processor in IRQ handler. */
#endif
	unsigned int cur_rx, cur_tx;		/* The next free ring entry */
	unsigned int dirty_rx, dirty_tx;	/* The ring entries to be free()ed. */
	unsigned int tx_full:1;				/* The Tx queue is full. */
	unsigned int full_duplex:1;			/* Full-duplex operation requested. */
	unsigned int full_duplex_lock:1;
	unsigned int fake_addr:1;			/* Multiport board faked address. */
	unsigned int default_port:4;		/* Last dev->if_port value. */
	unsigned int media2:4;				/* Secondary monitored media port. */
	unsigned int medialock:1;			/* Don't sense media type. */
	unsigned int mediasense:1;			/* Media sensing in progress. */
	unsigned int csr6;					/* Current CSR6 control settings. */
	unsigned char eeprom[128];			/* Serial EEPROM contents. */
	u16 to_advertise;					/* NWay capabilities advertised.  */
	u16 advertising[4];
	signed char phys[4], mii_cnt;		/* MII device addresses. */
	struct mediatable *mtable;
	int cur_index;						/* Current media index. */
	unsigned char pci_bus, pci_dev_fn;
	int pad0, pad1;						/* Used for 8-byte alignment */
/*RTN CHANGE */
     s32 tx_ring_status[RX_RING_SIZE];   /* just for not to free rt skbuffers */
};



/* Functions */

/* 
 *
 * Routines for wake up/sleep mechanisms 
 *
 */


/* software interrupt pended to send a packet */
static void 
rtn_start_tx_irq_handler (int irq, void *dev_id, struct pt_regs *p)
{
     	/* sending packets */
	rtn_skb_out_xmit_in_irq();
     	/* freeing memory */
     	rtn_free_queue();  
}


/* software interrupt pended to wake up an eventually reading process,
waiting for availability of memory */
static void 
rtn_queue_irq_handler (int irq, void *dev_id, struct pt_regs *p)
{
	if(test_and_clear_bit(0,(void *)&rtn_queue_to_wakeup)) {
		wake_up_interruptible(&rtn_global.wait);
	}
}

static inline int check_blocked(sigset_t signal, sigset_t blocked)
{
	int i=0;
	while (i<_NSIG_WORDS) {
		if ( signal.sig[i] & ~blocked.sig[i] ) {
			return 1;
		}
		i++;
	}
	return 0;
}

static void rtn_wake_sleepers(void){
	if (test_bit(1, (void *)&rtn_global.sleeps)) {
		rtn_queue_to_wakeup = 1;
		if(rtn_queue_irq > 0)
			rtl_global_pend_irq(rtn_queue_irq); 
	}
}

/* 
 * Routines for ethernet network and real time protocol encoding 
 */

/* Build the rtn header of the allocated skb. */ 
int
rtn_eth_header(struct sk_buff *skb, void *daddr, void *saddr, unsigned len)
{
     	struct ethhdr *eth;
     	unsigned char *rtnchar;
     	struct device * dev = root_tulip_dev;

     	/*   
      	* We put the distinction between normal ethernet packages and 
      	* real time packages. At receiving time, we have to recognize this 
	* before calling ether_trans_type
      	*/
     	rtnchar = skb_push(skb,sizeof(unsigned short));
     	*(unsigned short *)rtnchar = MASK_RTN;

     	eth = (struct ethhdr *)skb_push(skb,dev->hard_header_len);
	/* 
	 *	For a packet of type ETH_P_802_3 (which is our case) we put the length
	 *	in here instead. It is up to the 802.2 layer to carry protocol information.
	 */
	eth->h_proto = htons(len);

	/*
 	 * Set the source hardware address. 
	 */
	memcpy(eth->h_source,dev->dev_addr,dev->addr_len);

     	/*
	 * copy the destination address. 
      	 */    
	memcpy(eth->h_dest,daddr,dev->addr_len);
	return dev->hard_header_len;
}


static unsigned short 
__eth_type_trans(struct sk_buff *skb, struct device *dev)
{
	struct ethhdr *eth;
	unsigned char *rawp;
//        int i;
	skb->mac.raw=skb->data;
	skb_pull(skb,dev->hard_header_len);
	eth= skb->mac.ethernet;
	
/*	for(i=0;i < dev->addr_len;i++)
		printk("%X ", *(unsigned char *)(eth->h_dest+i));
        conpr("\n");
	for(i=0;i < dev->addr_len;i++)
		printk("%X ", *(unsigned char *)(eth->h_source+i));
        conpr("\n"); */
	if(*eth->h_dest&1)
	{
		if(memcmp(eth->h_dest,dev->broadcast, ETH_ALEN)==0)
			skb->pkt_type=PACKET_BROADCAST;
		else
			skb->pkt_type=PACKET_MULTICAST;
	}
	
	/*
	 *	This ALLMULTI check should be redundant by 1.4
	 *	so don't forget to remove it.
	 *
	 *	Seems, you forgot to remove it. All silly devices
	 *	seems to set IFF_PROMISC.
	 */
	 
	else if(dev->flags&(IFF_PROMISC/*|IFF_ALLMULTI*/))
	{
		if(memcmp(eth->h_dest,dev->dev_addr, ETH_ALEN))
			skb->pkt_type=PACKET_OTHERHOST;
	}
	
	if (ntohs(eth->h_proto) >= 1536)
		return eth->h_proto;
		
	rawp = skb->data;
	
	/*
	 *	This is a magic hack to spot IPX packets. Older Novell breaks
	 *	the protocol design and runs IPX over 802.3 without an 802.2 LLC
	 *	layer. We look for FFFF which isn't a used 802.2 SSAP/DSAP. This
	 *	won't work for fault tolerant netware but does for the rest.
	 */
	if (*(unsigned short *)rawp == 0xFFFF)
		return htons(ETH_P_802_3);
		
	/*
	 *	Real 802.2 LLC
	 */
	return htons(ETH_P_802_2);
}

static unsigned short 
rtn_eth_type_trans(struct sk_buff *skb, struct device *dev) 
{
   	unsigned short protocol;
   	unsigned char *rtnchar;
   	/* 
    	 * rtn packet spotted  
    	 */
	protocol =  __eth_type_trans(skb, dev);
   	rtnchar = skb->data;
   	if(*(unsigned short *)rtnchar == MASK_RTN) {
		skb_pull(skb,sizeof(unsigned short));
        	return ETH_P_RTN;
   	}
   	else return protocol;
}


/* 
 * Routines for the tx-rx packet queueing. most of it has been taken from
 * skbuff.c
 */

static inline struct sk_buff *__skb_delink(struct sk_buff_head *list) 
{
	struct sk_buff *next, *prev, *result;
	prev = (struct sk_buff *) list;
	next = prev->next;
	result = NULL;
	if (next != prev) {
		result = next;
		next = next->next;
		list->qlen--;
		next->prev = prev;
		prev->next = next;
		result->next = NULL;
		result->prev = NULL;
		result->list = NULL;
	}

	return result;
}

static inline 
struct sk_buff *rtl_skb_delink(struct sk_buff_head *list, spinlock_t
*lock)
{
	struct sk_buff *result;
   	unsigned long flags;
	rtl_spin_lock_irqsave(lock,flags);
        result = __skb_delink(list);
	rtl_spin_unlock_irqrestore(lock, flags);    
	return result;
}

static inline 
struct sk_buff *skb_delink(struct sk_buff_head *list, spinlock_t
*lock)
{
	struct sk_buff *result;
   	unsigned long flags;
	spin_lock_irqsave(lock,flags);
        result = __skb_delink(list);
	spin_unlock_irqrestore(lock, flags);    
	return result;
}

static inline void 
__skb_relink(struct sk_buff_head *list, struct sk_buff *newsk)
{
	struct sk_buff *prev, *next;
	newsk->list = list;
	list->qlen++;
	prev = (struct sk_buff *)list;
	next = prev->next;
	newsk->next = next;
	newsk->prev = prev;
	next->prev = newsk;
	prev->next = newsk;

}

static inline void 
skb_relink(struct sk_buff_head *list, struct sk_buff *newsk, spinlock_t *lock)
{
   	unsigned long flags;
	spin_lock_irqsave(lock,flags);
        __skb_relink(list, newsk);
	spin_unlock_irqrestore(lock, flags);    
}

static inline void 
rtl_skb_relink(struct sk_buff_head *list, struct sk_buff *newsk,
spinlock_t *lock)
{
   	unsigned long flags;
	rtl_spin_lock_irqsave(lock,flags);
        __skb_relink(list, newsk);
	rtl_spin_unlock_irqrestore(lock, flags);    
}

static inline void 
__skb_link(struct sk_buff_head *list, struct sk_buff * newsk) 
{
	struct sk_buff *prev, *next;
	newsk->list = list;
	list->qlen++;
	next = (struct sk_buff *)list;
	prev = next->prev;
	newsk->next = next;
        newsk->prev = prev;
  	next->prev = newsk;
        prev->next = newsk;

}

static inline void 
skb_link(struct sk_buff_head *list, struct sk_buff * newsk, spinlock_t *lock) 
{
   	unsigned long flags;
	spin_lock_irqsave(lock,flags);
        __skb_link(list, newsk);
	spin_unlock_irqrestore(lock, flags);    
}

static inline void 
rtl_skb_link(struct sk_buff_head *list, struct sk_buff * newsk, spinlock_t *lock) 
{
   	unsigned long flags;
	rtl_spin_lock_irqsave(lock,flags);
        __skb_link(list, newsk);
	rtl_spin_unlock_irqrestore(lock, flags);    
}

static void inline __skb_reinitialize(struct sk_buff *skb) {
	u8 *data = skb->head;
	skb->data = data;
	skb->tail = data;
	skb->len = 0;
	skb->is_clone = 0;
	skb->cloned = 0;
#if defined (LINUX_VERSION_CODE) && LINUX_VERSION_CODE < 0x10300

#else  /* 1.3.0 and later */
	skb_reserve(skb,16);
#endif

}

/* Application Interface : with user space through rt-fifos and with other real time
applications through RTL_POSIX definitions */

int rtn_usr_send(int minor, char * buffer, int count) {
     	unsigned char *buf;
     	struct sk_buff *skb;
	unsigned long flags;
     	int entry_out;

	rtl_spin_lock_irqsave(&rtn_global.skb_out_lock, flags);
    	entry_out = (rtn_global.cur_out++) % NUM_RTN_SKB_OUT;
	if(rtn_global.skb_out_ring[entry_out]) {
		skb = rtn_global.skb_out_ring[entry_out];
		rtn_global.skb_out_ring[entry_out] = 0;
		rtl_spin_unlock_irqrestore(&rtn_global.skb_out_lock, flags);
	}
	else {
		rtl_spin_unlock_irqrestore(&rtn_global.skb_out_lock, flags);    
		while(rtn_global.skb_out_ring[entry_out] == NULL) {
        	/* 	why not just:	if (signal_pending(current)) */
	        	if (check_blocked(current->signal, current->blocked) )
	       	  		return -ERESTARTSYS;
        	  	rtn_queue_to_wakeup = 0;
	               	set_bit(1,(void *)&rtn_global.sleeps);
        	       	interruptible_sleep_on(&rtn_global.wait);
              		clear_bit(0,(void *)&rtn_global.sleeps);
              	}
        	rtl_spin_lock_irqsave(&rtn_global.skb_out_lock, flags);
              	skb = rtn_global.skb_out_ring[entry_out];
               	rtn_global.skb_out_ring[entry_out] = 0;
              	rtl_spin_unlock_irqrestore(&rtn_global.skb_out_lock, flags);    
    	}
       	if(!skb)  {
         	printk(KERN_ERR"Error NULL skb obtained from Linux.\n");
            	return -ENOMEM;
       	}
          /*   
           *  Fill the information of the sk buffer with the contains of the 
           *  received data.
           *  We put the header of the rtn ethernet address.
           */

      	skb->dev = root_tulip_dev;
       	skb->protocol = htons(ETH_P_RTN);
       	skb_reserve(skb,ETH_HLEN+sizeof(unsigned short));
       	buf = skb_put(skb,count);
       	if(!buffer) 
       		rtf_get(minor,buf, count);
     	else
 		memcpy(buf, buffer, count);
	rtn_eth_header(skb, daddr, saddr, skb->len);
    	rtl_skb_link(&rtn_global.skb_out_queue, skb,&rtn_global.out_queue_lock);
       	start_bh_atomic(); // to synchronize with bh
       	/* sending packets */
       	rtn_skb_out_xmit();
       	end_bh_atomic();
       	return count;
}

int rtn_send(int minor, char * buffer, int count) {
     	unsigned char *buf;
     	struct sk_buff *skb;
	unsigned long flags;
     	int entry_out;

	rtl_spin_lock_irqsave(&rtn_global.skb_out_lock, flags);
	entry_out = (rtn_global.cur_out++) % NUM_RTN_SKB_OUT;
	if(rtn_global.skb_out_ring[entry_out]) {
		skb = rtn_global.skb_out_ring[entry_out];
		rtn_global.skb_out_ring[entry_out] = 0;
		rtl_spin_unlock_irqrestore(&rtn_global.skb_out_lock, flags);
	}
	else {
		return -ENOMEM;
        }
          /*   
           *  Fill the information of the sk buffer with the contains of the 
           *  received data.
           *  We put the header of the rtn ethernet address.
           */

      	skb->dev = root_tulip_dev;
       	skb->protocol = htons(ETH_P_RTN);
       	skb_reserve(skb,ETH_HLEN+sizeof(unsigned short));
       	buf = skb_put(skb,count);
       	if(!buffer) 
       		rtf_get(minor,buf, count);
     	else
 		memcpy(buf, buffer, count);
	rtn_eth_header(skb, daddr, saddr, skb->len);
    	rtl_skb_link(&rtn_global.skb_out_queue, skb,&rtn_global.out_queue_lock);
       	/* sending packets */
       	start_bh_atomic();
       	rtn_skb_out_xmit();
       	end_bh_atomic();
       	return count;
}


int rtn_recv(int minor, char *buffer, int count) {
	struct sk_buff *skb;
        struct sk_buff_head *q = &rtn_global.skb_in_queue;
        unsigned long flags;

        rtl_spin_lock_irqsave(&rtn_global.in_queue_lock,flags);
	if ((skb = __skb_delink(q)) != NULL) {
        	int count = (*((unsigned int *)skb->data))&0x0000ffff;
             if(!buffer)
                rtf_put(minor, skb->data,count); 
             else
                memcpy(buffer, skb->data, count);
		rtl_spin_unlock_irqrestore(&rtn_global.in_queue_lock, flags);
                rtl_skb_link(&rtn_global.skb_free_queue, skb,&rtn_global.free_queue_lock);
                return count;
        }
	else {
		rtl_spin_unlock_irqrestore(&rtn_global.in_queue_lock,
flags);
          return 0;
     }
}
	

int rtn_change_address(int minor, char *arg) {

  	if (test_and_set_bit(0, (void*)&rtn_change_dest_flag)) {
		printk(KERN_DEBUG"%s: Trying to modify again destination address.\n", root_tulip_dev->name);
		clear_bit(0, (void*)&rtn_change_dest_flag);
		return -1;
     	} 
     	if(arg) 
           	memcpy(daddr, arg, 6);
     	else
		rtf_get(minor,daddr, 6);        	
	clear_bit(0, (void*)&rtn_change_dest_flag);
	return 1;
}

int rtn_tulip_handler(unsigned int fifo)
{
	int err;
	struct my_msg_struct msg;
	while ((err = rtf_get(CTL_PIPE, &msg, sizeof(msg))) == sizeof(msg)) {
		switch (msg.command) {
			case SEND_FRM:
                    	// wait if some process is changing daddr
                  		while(test_bit(0,(void *)&rtn_change_dest_flag));                     
                    	/* 
			 * For the time being, it will be like this, in
 			 * this situation, the device will read from the
			 * SND_FIFO,afterwards we could use copy_from_user,
			 * if we implement the file_operations 
			 */
                    		rtn_usr_send(SND_PIPE, 0, msg.length);
				break;
			case RCV_FRM:
                  		rtn_recv(RCV_PIPE, 0, 0); /* Same here... */
				break;
			case CHG_DADDR:
				rtn_change_address(SND_PIPE, 0);
				break;
			default:
				return -EINVAL;
		}
	}
	if (err != 0) {
		return err;
	}
     	return 0;
}


static int rtn_tulip_start_xmit(struct sk_buff *skb, struct device *dev)
{
	struct tulip_private *tp = (struct tulip_private *)dev->priv;
	int entry;
	u32 flag;

	/* Block a timer-based transmit from overlapping.  This could better be
	   done with atomic_swap(1, dev->tbusy), but set_bit() works as well. */
	if (test_and_set_bit(0, (void*)&dev->tbusy) != 0) {
		if (jiffies - dev->trans_start < TX_TIMEOUT)
			return 1;
		rtn_tulip_tx_timeout(dev);
		return 1;
	}
	/* Caution: the write order is important here, set the base address
	   with the "ownership" bits last. */

	/* Calculate the next Tx descriptor entry. */
/*	conpr("t");
	printk("/t%d",rtn_global.dirty_out); */
	entry = tp->cur_tx % TX_RING_SIZE;

	tp->tx_skbuff[entry] = skb;
	tp->tx_ring[entry].buffer1 = virt_to_bus(skb->data);

	if (tp->cur_tx - tp->dirty_tx < TX_RING_SIZE/2) {/* Typical path */
	  flag = 0x60000000; /* No interrupt */
	  dev->tbusy = 0;
	} else if (tp->cur_tx - tp->dirty_tx == TX_RING_SIZE/2) {
	  flag = 0xe0000000; /* Tx-done intr. */
	  dev->tbusy = 0;
	} else if (tp->cur_tx - tp->dirty_tx < TX_RING_SIZE - 2) {
	  flag = 0x60000000; /* No Tx-done intr. */
	  dev->tbusy = 0;
	} else {
	  /* Leave room for set_rx_mode() to fill entries. */
	  flag = 0xe0000000; /* Tx-done intr. */
	  tp->tx_full = 1;
	}
	if (entry == TX_RING_SIZE-1)
		flag |= 0xe2000000;

	tp->tx_ring[entry].length = skb->len | flag;
     	tp->tx_ring_status[entry] = 0x00000000;
	tp->tx_ring[entry].status = 0x80000000;	/* Pass ownership to the chip. */
	tp->cur_tx++;
	/* Trigger an immediate transmit demand. */
	outl(0, dev->base_addr + CSR1);
	dev->trans_start = jiffies;
	return 0;
}


inline void rtn_process_in_queue() {
	struct sk_buff *skb;
        struct sk_buff_head *q = &rtn_global.skb_in_queue;
        unsigned long flags;
	/*
	 *  Let now clear the in queue. -AS
	 *  Use locks and no interrupts to protect
	 *  data
	 */ 
	spin_lock_irqsave(&rtn_global.in_queue_lock,flags);
	while((skb = __skb_delink(q)) != NULL) {
        	rtf_put(RCV_PIPE, skb->data, (*((unsigned int *)skb->data))&0x0000ffff);
		spin_unlock_irqrestore(&rtn_global.in_queue_lock, flags);

#if (LINUX_VERSION_CODE > 0x20155)
   		dev_kfree_skb(skb);
#else
                dev_kfree_skb(skb, FREE_WRITE);
#endif
         	spin_lock_irqsave(&rtn_global.in_queue_lock,flags);
        }        
  	spin_unlock_irqrestore(&rtn_global.in_queue_lock, flags);
}

int rtn_skb_restart_xmit_in_irq(struct device *dev)
{
	struct sk_buff *skb;
        struct sk_buff_head *q = &rtn_global.skb_out_queue;
	unsigned long flags;
	/*
	 *  Let now clear the out queue. -AS
	 *
	 *  We are competing here both with rtn_handler() and
	 *  rtn_interrupt(). The same as above
	 *
	 */ 
	if ((skb = skb_delink(q,&rtn_global.out_queue_lock)) != NULL)
	{
		spin_lock_irqsave(&rtn_global.txmit_lock, flags);
		if (rtn_tulip_start_xmit(skb, dev) == 0) {
			spin_unlock_irqrestore(&rtn_global.txmit_lock, flags);
			return -1;
		}

		/* Device kicked us out.The device is buggy (ppp)
		 */
		spin_unlock_irqrestore(&rtn_global.txmit_lock, flags);
		skb_relink(q, skb, &rtn_global.out_queue_lock );
		return -1;
	}
	return q->qlen;
}

int rtn_skb_restart_xmit(struct device *dev)
{
	struct sk_buff *skb;
        struct sk_buff_head *q = &rtn_global.skb_out_queue;
	unsigned long flags;
	/*
	 *  Let now clear the out queue. -AS
	 *
	 *  We are competing here both with rtn_handler() and
	 *  rtn_interrupt(). The same as above
	 *
	 */ 
	if ((skb = rtl_skb_delink(q,&rtn_global.out_queue_lock)) != NULL)
	{
		rtl_spin_lock_irqsave(&rtn_global.txmit_lock, flags);
		if (rtn_tulip_start_xmit(skb, dev) == 0) {
			rtl_spin_unlock_irqrestore(&rtn_global.txmit_lock, flags);
			return -1;
		}

		/* Device kicked us out.The device is buggy (ppp)
		 */
		rtl_spin_unlock_irqrestore(&rtn_global.txmit_lock, flags);
		rtl_skb_relink(q, skb, &rtn_global.out_queue_lock );
		return -1;
	}
	return q->qlen;
}




inline void rtn_skb_out_xmit_in_irq(void) {
           	while (!root_tulip_dev->tbusy &&
			rtn_skb_restart_xmit_in_irq(root_tulip_dev)< 0)
			/* NOTHING */;

}

inline void rtn_skb_out_xmit() {
           	while (!root_tulip_dev->tbusy &&
			rtn_skb_restart_xmit(root_tulip_dev)< 0)
			/* NOTHING */;

}


void rtn_clear_out_queue() {
	struct sk_buff *skb;
        struct sk_buff_head *q = &rtn_global.skb_out_queue;
	/*
	 *  Let now clear the in queue. -AS
	 *
	 *  We are competing here both with rtn_handler() and
	 *  rtn_tulip_interrupt().
	 *  We don't want either of those to mess with skb ptrs
	 *  while we work on them, thus lock use.
	 *
	 */ 
	while((skb = rtl_skb_delink(q, &rtn_global.out_queue_lock)) !=
NULL)
	{	
#if (LINUX_VERSION_CODE > 0x20155)
   		dev_kfree_skb(skb);
#else
                dev_kfree_skb(skb, FREE_WRITE);
#endif

        }        
}

void rtn_free_queue() {
	struct sk_buff *skb;
        struct sk_buff_head *q = &rtn_global.skb_free_queue;
	while((skb = skb_delink(q, &rtn_global.free_queue_lock)) != NULL)
{
       // Free the original skb. 
#if (LINUX_VERSION_CODE > 0x20155)
   			dev_kfree_skb(skb);
#else
                	dev_kfree_skb(skb, FREE_WRITE);
#endif
	} 
}

static void refill_skb_out_ring(struct device *dev) {
	unsigned int cur_ptr;
	unsigned long flags;
	spin_lock_irqsave(&rtn_global.skb_out_lock, flags);
	cur_ptr = rtn_global.cur_out; 
	for ( ; cur_ptr - rtn_global.dirty_out > 0 ; ) {	
	  	struct sk_buff *new_skb;
		int entry = rtn_global.dirty_out % NUM_RTN_SKB_OUT;
//	        spin_unlock_irqrestore(&rtn_global.skb_out_lock, flags);           
		new_skb = DEV_ALLOC_SKB(PKT_BUF_SZ);
//		spin_lock_irqsave(&rtn_global.skb_out_lock, flags);
           	if(!new_skb) {
	      		break; // This is bad... 
                }
	    	new_skb->dev = dev; /* Mark as being used by this device. */
		rtn_global.skb_out_ring[entry] = new_skb;
		entry = (++rtn_global.dirty_out) % NUM_RTN_SKB_OUT;
	}
        spin_unlock_irqrestore(&rtn_global.skb_out_lock, flags); 
}

static void refill_skb_pool_ring(struct device *dev) {
	unsigned int cur_ptr;
	unsigned long flags;
	spin_lock_irqsave(&rtn_global.pool_lock, flags);	
	cur_ptr = rtn_global.cur_pool;
	for ( ; cur_ptr - rtn_global.dirty_pool > 0 ;) {	
		struct sk_buff *new_skb;
		int entry = rtn_global.dirty_pool % POOL_RING_SIZE;
//	        spin_unlock_irqrestore(&rtn_global.pool_lock, flags);    
		new_skb = DEV_ALLOC_SKB(PKT_BUF_SZ);
//		spin_lock_irqsave(&rtn_global.pool_lock, flags);	
           	if(!new_skb) {
	      		break; // This is bad....
                }
	    	new_skb->dev = dev; /* Mark as being used by this device. */
		rtn_global.skb_pool_ring[entry] = new_skb;
		entry = (++rtn_global.dirty_pool) %  POOL_RING_SIZE;
	}
        spin_unlock_irqrestore(&rtn_global.pool_lock, flags);    
}

int rtn_cp_from_temp_rx(struct device *dev)  {
	struct sk_buff *skb;
	int work_done = 0;
        struct sk_buff_head *q = &rtn_global.rx_skbuff_queue;
        while((skb = skb_delink(q, &rtn_global.rx_lock)) != NULL) {
      		/* process reception */
           	netif_rx(skb);
          	work_done++;
     	}

        /* refill the tx ring  rtn_global.skb_out_ring */  
	refill_skb_out_ring(dev);
	rtn_wake_sleepers();
        /* refill the rx ring  rtn_global.skb_pool_ring */
	refill_skb_pool_ring(dev);
/*	if(rtn_start_tx_irq > 0)
		rtl_global_pend_irq(rtn_start_tx_irq); // free memory  */
        return work_done;
}

static int
rtn_cp_to_temp_rx(struct device *dev)
{
	struct tulip_private *tp = (struct tulip_private *)dev->priv;
	int rx_tulip_entry = tp->cur_rx % RX_RING_SIZE;
     	int rx_temp_entry;
	int rx_work_limit = tp->dirty_rx + RX_RING_SIZE - tp->cur_rx;
     	int entry;
	unsigned long flags;
	unsigned int work_done = 0;

	if (tulip_debug > 4)
		printk(KERN_DEBUG " In tulip_rx(), entry %d %8.8x.\n", rx_tulip_entry,
			   tp->rx_ring[rx_tulip_entry].status);
	/* If we own the next entry, it's a new packet. Send it up. */
	
	rx_temp_entry = rtn_global.cur_pool % POOL_RING_SIZE;
	
	while (tp->rx_ring[rx_tulip_entry].status >= 0) {
		s32 status = tp->rx_ring[rx_tulip_entry].status;
		if (--rx_work_limit < 0)
			break;
		
		if ((status & 0x0300) != 0x0300) {
			if ((status & 0xffff) != 0x7fff) { /* Ignore earlier buffers. */
				if (tulip_debug > 1)
					printk(KERN_WARNING "%s: Oversized Ethernet frame spanned "
						   "multiple buffers, status %8.8x!\n",
						   dev->name, status);
			  tp->stats.rx_length_errors++;
			}
		} else if (status & 0x8000) {
			/* There was a fatal error. */
			if (tulip_debug > 2)
				printk(KERN_DEBUG "%s: Receive error, Rx status %8.8x.\n",
					   dev->name, status);
			tp->stats.rx_errors++; /* end of a packet.*/
			if (status & 0x0890) tp->stats.rx_length_errors++;
			if (status & 0x0004) tp->stats.rx_frame_errors++;
			if (status & 0x0002) tp->stats.rx_crc_errors++;
			if (status & 0x0001) tp->stats.rx_fifo_errors++;
		} else {
			/* Omit the four octet CRC from the length. */
			short pkt_len = (status >> 16) - 4;
               		struct sk_buff *skb;
	          	rtl_spin_lock_irqsave(&rtn_global.pool_lock, flags);
			skb = rtn_global.skb_pool_ring[rx_temp_entry];
			if(!skb) {
                  printk(KERN_DEBUG"Null skbuffer requested.\n");	
				/* we cant do anything else than wait for the next cycle. */
	         		rtl_spin_unlock_irqrestore(&rtn_global.pool_lock, flags);    
				break; 
			}
		  	rtn_global.skb_pool_ring[rx_temp_entry] = 0;
        		rx_temp_entry = (++rtn_global.cur_pool) % POOL_RING_SIZE;
         		rtl_spin_unlock_irqrestore(&rtn_global.pool_lock, flags);    
               		skb->dev = dev;
               		skb_reserve(skb, 2);	/* 16 byte align the IP header */

#if LINUX_VERSION_CODE < 0x10300
   			memcpy(skb->data,(unsigned char *)tp->rx_ring[rx_tulip_entry].buffer1, pkt_len);
#elif LINUX_VERSBION_CODE < 0x20200  || defined(__alpha__)
               		memcpy(skb_put(skb, pkt_len),(unsigned char *)bus_to_virt(tp->rx_ring[rx_tulip_entry].buffer1), pkt_len);
#else	     
               		eth_copy_and_sum(skb, (unsigned char *)bus_to_virt(tp->rx_ring[rx_tulip_entry].buffer1), pkt_len, 0);
               		skb_put(skb, pkt_len);

#endif
	         	rtl_spin_unlock_irqrestore(&rtn_global.pool_lock, flags);    
	               	work_done++;

#if LINUX_VERSION_CODE > 0x10300
          /* try to spot a rtn packet */
	              	if((skb->protocol = rtn_eth_type_trans(skb, dev)) == ETH_P_RTN) { /* rtn protocol packet */
                                rtl_skb_link(&rtn_global.skb_in_queue, skb,&rtn_global.in_queue_lock);
                  		rtn_global.rx_packets++;
				rtn_global.rx_bytes += pkt_len;
              	 	}
              		else { // normal packet
				rtl_skb_link(&rtn_global.rx_skbuff_queue,skb,&rtn_global.rx_lock);
	      		}	

#else
                  	skb->len = pkt_len;
#endif
                  	dev->last_rx = jiffies;
                  	tp->stats.rx_packets++;
#if LINUX_VERSION_CODE > 0x20127
                  	tp->stats.rx_bytes += pkt_len;
#endif
          	}

	  	rx_tulip_entry = (++tp->cur_rx) % RX_RING_SIZE;
	} /* end while */
	/* Change the status of the  Rx ring  skbuffers. */
	for (; tp->cur_rx - tp->dirty_rx > 0; tp->dirty_rx++) {
		entry = tp->dirty_rx % RX_RING_SIZE;
		tp->rx_ring[entry].status = 0x80000000; 
	}
	return (work_done);
}


static void rtn_tulip_interrupt(unsigned int irq, struct pt_regs *regs) {

   #ifdef SA_SHIRQ		/* Use the now-standard shared IRQ implementation. */
	struct device *dev = root_tulip_dev;
#else
	struct device *dev = (struct device *)(irq2dev_map[irq]);
#endif

	struct tulip_private *tp;
	long ioaddr;
	int csr5, work_budget = rtn_max_interrupt_work;

	if (dev == NULL) {
		printk (KERN_ERR" tulip_interrupt(): irq %d for unknown device.\n",
				irq);
		return;
	}
	ioaddr = dev->base_addr;
	tp = (struct tulip_private *)dev->priv;
#ifdef SMP_CHECK
	tp->smp_proc_id = hard_smp_processor_id();
#endif
//	dev->interrupt = 1;
	do {
		csr5 = inl(ioaddr + CSR5);
		/* Acknowledge all of the current interrupt sources ASAP. */
		outl(csr5 & 0x0001ffff, ioaddr + CSR5);

          	if (tulip_debug > 4)
			printk(KERN_DEBUG "%s: interrupt  csr5=%#8.8x new csr5=%#8.8x.\n",
				   dev->name, csr5, inl(dev->base_addr + CSR5));

		if ((csr5 & (NormalIntr|AbnormalIntr)) == 0) 
			break;		

		if (csr5 & (RxIntr | RxNoBuf))
			work_budget -= rtn_cp_to_temp_rx(dev);

          	if (csr5 & (TxNoBuf | TxDied | TxIntr))  {
			unsigned int dirty_tx;
			for (dirty_tx = tp->dirty_tx; tp->cur_tx - dirty_tx > 0; dirty_tx++) {
				int entry = dirty_tx % TX_RING_SIZE;
				int status = tp->tx_ring[entry].status;

				if (status < 0) {
					break;			/* It still hasn't been Txed */
				}
				/* Check for Rx filter setup frames. */
				if (tp->tx_skbuff[entry] == NULL) {
				  continue;
				}

				if (status & 0x8000) {
					/* There was an major error, log it. */
#ifndef final_version
					if (tulip_debug > 1)
						printk(KERN_DEBUG "%s: Transmit error, Tx status %8.8x.\n",
							   dev->name, status);
#endif
					tp->stats.tx_errors++;
					if (status & 0x4104) tp->stats.tx_aborted_errors++;
					if (status & 0x0C00) tp->stats.tx_carrier_errors++;
					if (status & 0x0200) tp->stats.tx_window_errors++;
					if (status & 0x0002) tp->stats.tx_fifo_errors++;
					if ((status & 0x0080) && tp->full_duplex == 0)
						tp->stats.tx_heartbeat_errors++;
#ifdef ETHER_STATS
					if (status & 0x0100) tp->stats.collisions16++;
#endif
				} else {
#ifdef ETHER_STATS
					if (status & 0x0001) tp->stats.tx_deferred++;
#endif
#if LINUX_VERSION_CODE > 0x20127
					tp->stats.tx_bytes += tp->tx_ring[entry].length & 0x7ff;
#endif
					tp->stats.collisions += (status >> 3) & 15;
					tp->stats.tx_packets++;
				}

				/* Put the skb to be freed in queue. */
				rtl_skb_link(&rtn_global.skb_free_queue,
tp->tx_skbuff[entry],&rtn_global.free_queue_lock);
               		} /* end for */

#ifndef final_version
			if (tp->cur_tx - dirty_tx > TX_RING_SIZE) {
				printk(KERN_ERR "%s: Out-of-sync dirty pointer, %d vs. %d, full=%d.\n",
					   dev->name, dirty_tx, tp->cur_tx, tp->tx_full);
				dirty_tx += TX_RING_SIZE;
			}
#endif

			if (tp->tx_full && dev->tbusy
				&& tp->cur_tx - dirty_tx  < TX_RING_SIZE - 2) {
				/* The ring is no longer full, clear tbusy. */
				tp->tx_full = 0;
				dev->tbusy = 0;
				if(rtn_start_tx_irq > 0)
					rtl_global_pend_irq(rtn_start_tx_irq); // free memory 
				mark_bh(NET_BH);
			}

			tp->dirty_tx = dirty_tx;
			if (csr5 & TxDied) {
				if (tulip_debug > 1)
					printk(KERN_WARNING "%s: The transmitter stopped!"
						   "  CSR5 is %x, CSR6 %x.\n",
						   dev->name, csr5, inl(ioaddr + CSR6));
				outl(tp->csr6 | 0x0002, ioaddr + CSR6);
				outl(tp->csr6 | 0x2002, ioaddr + CSR6);
			}
          } /* end if */

             /* Log errors. */
          if (csr5 & AbnormalIntr) {	/* Abnormal error summary bit. */
			if (csr5 & TxJabber) tp->stats.tx_errors++;
			if (csr5 & TxFIFOUnderflow) {
				if ((tp->csr6 & 0xC000) != 0xC000)
					tp->csr6 += 0x4000;	/* Bump up the Tx threshold */
				else
					tp->csr6 |= 0x00200000;  /* Store-n-forward. */
				/* Restart the transmit process. */
				outl(tp->csr6 | 0x0002, ioaddr + CSR6);
				outl(tp->csr6 | 0x2002, ioaddr + CSR6);
			}
			if (csr5 & RxDied) {		/* Missed a Rx frame. */
				tp->stats.rx_errors++;
				tp->stats.rx_missed_errors += inl(ioaddr + CSR8) & 0xffff;
			}
			if (csr5 & TimerInt) {
				printk(KERN_ERR "%s: Something Wicked happened! %8.8x.\n",
					   dev->name, csr5);
				/* Hmmmmm, it's not clear what to do here. */
			}
			if (csr5 & (TPLnkPass | TPLnkFail | 0x08000000)
				&& tp->chip_id == DC21142) {
				if (tulip_debug > 1)
					printk(KERN_INFO"%s: 21142 link change, CSR5 = %8.8x.\n",
						   dev->name, csr5);
				rtn_t21142_lnk_change(dev);
			}
			/* Clear all error sources, included undocumented ones! */
			outl(0x0800f7ba, ioaddr + CSR5);        
                   
          }  /* end if */


	  if (--work_budget < 0) {
	       if (tulip_debug > 1)
			printk(KERN_WARNING "%s: Too much work at interrupt, "
					   "csr5=0x%8.8x.\n", dev->name, csr5);
		/* Acknowledge all interrupt sources. */
		outl(0x8001ffff, ioaddr + CSR5);
#ifdef notdef
		/* Clear all but standard interrupt sources. */
		outl((~csr5) & 0x0001ebef, ioaddr + CSR7);
#endif
			break;
	  }
       } while (1);

       if (tulip_debug > 3)
		printk(KERN_DEBUG "%s: exiting interrupt, csr5=%#4.4x.\n",
			   dev->name, inl(ioaddr + CSR5));

//	dev->interrupt = 0;
}

/* 
 * Hw independent rtn routine to handle the network interrupts 
 */

unsigned int rtn_interrupt(unsigned int irq, struct pt_regs *regs) {
	struct device *dev = root_tulip_dev;
//     	static int debug = 0;
//	unsigned long flags;
     	/* multiprocessor lock */
     	if (test_and_set_bit(0, (void*)&rtn_interrupt_flag)) {
		printk(KERN_DEBUG"%s: Re-entering the rtn interrupt handler.\n", dev->name);
        	rtl_hard_enable_irq(root_tulip_dev->irq);
		return 1;
     	}
     	/* entering into the hw-dependent interrupt*/
     	rtn_tulip_interrupt(irq, regs);

	/* sending packets */
	rtn_skb_out_xmit();
     	rtl_hard_enable_irq(root_tulip_dev->irq);
     	rtl_global_pend_irq (irq);
     	clear_bit(0, (void*)&rtn_interrupt_flag);
	return 0;
}

void init_queues(void) {
	skb_queue_head_init(&rtn_global.skb_free_queue);
	skb_queue_head_init(&rtn_global.rx_skbuff_queue);
	skb_queue_head_init(&rtn_global.skb_out_queue);
	skb_queue_head_init(&rtn_global.skb_in_queue);
}

void init_spin_locks(void) {
	spin_lock_init(&rtn_global.rx_lock);
	spin_lock_init(&rtn_global.free_queue_lock);
	spin_lock_init(&rtn_global.in_queue_lock);
	spin_lock_init(&rtn_global.out_queue_lock);
	spin_lock_init(&rtn_global.pool_lock);
	spin_lock_init(&rtn_global.skb_out_lock);

}


/*
 * In the initialization of the module, we have to
 * pre-allocate space for the incoming messages
 * This space will be retrieved with DEV_ALLOC_SKB
 */
 
int rtn_init(void)
{
   	int i,out=0,rx=0;
	int old_irq_state;
     	int irq = -1;
	struct tulip_private *tp = (struct tulip_private *)root_tulip_dev->priv;
     	if(test_bit(0,(void *)&rtn_init_flag))  // re-entrance not allowed
        	return -EINVAL;
     	/* Synchronizing with normal interrupt */
    	while(test_bit(0,(void *)&tp->interrupt));
    	if((!tp)||(tp->next_module)) // device has not been installed
      		return -EINVAL;
    	irq = rtl_get_soft_irq (rtn_queue_irq_handler, "rtn_q");
    	if (irq > 0) {
       		rtn_queue_irq = irq;
    	} 
	else return -1;
    	irq = -1;
    	irq = rtl_get_soft_irq (rtn_start_tx_irq_handler, "rtn_t");
    	if (irq > 0) {
       		rtn_start_tx_irq = irq;
    	} else return -1;

	rtf_create(SND_PIPE, SIZE_SND_PIPE);         
	rtf_create(RCV_PIPE, SIZE_RCV_PIPE);
	rtf_create(CTL_PIPE, SIZE_CTL_PIPE); /* input control channel */

     	rtl_no_interrupts(old_irq_state);
	rtf_create_handler(CTL_PIPE, &rtn_tulip_handler); 
  
	/* 
 	 * Pre-allocation of the pool of sk buffers
 	 */
     
	for (rx = 0; rx < POOL_RING_SIZE; rx++) {

        /* Note the receive buffer must be longword aligned.
         * dev_alloc_skb() provides 16 byte alignment.  But do *not*
         * use skb_reserve() to align the IP header! 
	 */
        	struct sk_buff *skb;
        	skb = DEV_ALLOC_SKB(PKT_BUF_SZ);
        	if (skb == NULL) 
           		goto no_mem;
        	rtn_global.skb_pool_ring[rx] = skb;
        	skb->dev = root_tulip_dev;	/* Mark as being used by this
device. */
	}

	for (out = 0; out < NUM_RTN_SKB_OUT; out++) {

        /* Note the receive buffer must be longword aligned.
         * dev_alloc_skb() provides 16 byte alignment.  But do *not*
         * use skb_reserve() to align the IP header! 
	 */
        	struct sk_buff *skb;
        	skb = DEV_ALLOC_SKB(PKT_BUF_SZ);
        	if (skb == NULL) 
           		goto no_mem;
        	rtn_global.skb_out_ring[out] = skb;
        	skb->dev = root_tulip_dev;	/* Mark as being used by this device. */
	}

	/* initializing locks */
        init_spin_locks();
     /* queues initialization */
     /* initializing queues */
        init_queues();

        /* copying local hardware address */
        memcpy(saddr,root_tulip_dev->dev_addr,6);
     /* Initialization of the interrupt handling */

     	copy_handler = rtn_cp_from_temp_rx;
     	freemem_handler = rtn_free_queue;

     	rtn_global.rx_packets = 0;
     	rtn_global.rx_bytes= 0;

   /* ring pointers */
    
     	rtn_global.cur_out = 0;	
     	rtn_global.dirty_out = 0;	
     	rtn_global.cur_pool = 0;	
     	rtn_global.dirty_pool = 0;	

   /* wake up/sleep variables */
      	rtn_global.sleeps =0;
 	rtn_global.wait =0;

     	rtl_request_global_irq(root_tulip_dev->irq, rtn_interrupt);
     	rtl_hard_enable_irq(root_tulip_dev->irq);
     	set_bit(0,(void *)&rtn_init_flag);  // all is fine, we can begin to work
     	rtl_restore_interrupts(old_irq_state);
     	printk("RTN Module loaded successfully!, with irq=%d!\n",root_tulip_dev->irq);
	return 0;

no_mem: 
     	rtl_restore_interrupts(old_irq_state);
     	if(rx < POOL_RING_SIZE) {
        	for(i=0;i<rx;i++) 
           		dev_kfree_skb(rtn_global.skb_pool_ring[i]);
        	return -EINVAL;
     	}

     	if(out < NUM_RTN_SKB_OUT) {
        	for(i=0;i<out;i++) 
           		dev_kfree_skb(rtn_global.skb_out_ring[i]);
        	for(i=0;i<POOL_RING_SIZE;i++) 
           		dev_kfree_skb(rtn_global.skb_pool_ring[i]);
        	return -EINVAL;

     	}
	return 0;
}

void rtn_uninit(void)
{
     int i;
	int old_irq_state;
	struct tulip_private *tp = (struct tulip_private *)root_tulip_dev->priv;
	rtf_destroy(1);
	rtf_destroy(2);
	rtf_destroy(3);

     while (test_bit(0, (void *)&rtn_interrupt_flag)); /* wait for interrupts to finish */
/* we dont want the interrupts get messed with free pointers, so lock */
     rtl_no_interrupts (old_irq_state);
     rtl_free_global_irq(root_tulip_dev->irq);
     rtl_restore_interrupts(old_irq_state);
     while (test_bit(0,(void *)&rtn_interrupt_flag) && test_bit(0, (void*)&tp->interrupt)); /* wait for interrupts to finish */
     rtl_no_interrupts (old_irq_state);
	if (rtn_queue_irq) 
		free_irq(rtn_queue_irq, 0);
     	if (rtn_start_tx_irq) 
		free_irq(rtn_start_tx_irq, 0);
     copy_handler = NULL;
     freemem_handler=  NULL;
     rtl_restore_interrupts(old_irq_state);
     rtn_process_in_queue(); /* cleaning the remain packets haven't
being
received */

     rtn_clear_out_queue();/* cleaning the remain packets haven't being
sent */
     rtn_free_queue(); /* cleaning the swap space */
 
     for(i=0;i<POOL_RING_SIZE;i++)
	if(rtn_global.skb_pool_ring[i])
        	dev_kfree_skb(rtn_global.skb_pool_ring[i]);

     for(i=0;i<NUM_RTN_SKB_OUT;i++)
	if(rtn_global.skb_out_ring[i])
        	dev_kfree_skb(rtn_global.skb_out_ring[i]);

     clear_bit(0, (void *)&rtn_init_flag);
     printk("\nRTN module unloaded successfully!\n");
}


#ifdef CONFIG_RTL_POSIX_IO
#include <rtl_posixio.h>

static int rtn_open (struct rtl_file *filp)
{
	if (!(filp->f_flags & O_NONBLOCK)) {
		return -EACCES; /* TODO: implement blocking IO */
	}
	return 0;
}

static int rtn_release (struct rtl_file *filp)
{
	return 0;
}

static ssize_t rtn_write(struct rtl_file *filp, const char *buf, size_t count, loff_t* ppos)
{
	return rtn_send(RTL_MINOR_FROM_FILEPTR(filp), (char *) buf, count);
	
}

static ssize_t rtn_read(struct rtl_file *filp, char *buf, size_t count, loff_t* ppos)
{
	return rtn_recv(RTL_MINOR_FROM_FILEPTR(filp), buf, count);
	
}

static ssize_t rtn_ioctl(struct rtl_file *filp, unsigned int cmd, unsigned long arg)
{
   	switch(cmd) {
      		case CHANGE_DEST_ADDR: 
		return rtn_change_address(RTL_MINOR_FROM_FILEPTR(filp),
(char *)arg);
	      	default : return -1;
   	}
}



static struct rtl_file_operations rtn_fops = {
       	NULL,
	rtn_read,
	rtn_write,
	NULL,
     rtn_ioctl,
	NULL,
	rtn_open,
	rtn_release
};
#endif


#ifdef MODULE


int init_module(void)
{
	int ret;
	ret = rtn_init();
	if (ret < 0) {
		return ret;
	}
#ifdef CONFIG_RTL_POSIX_IO
	if (rtl_register_chrdev (RTT_MAJOR, "rtt", &rtn_fops)) {
		printk ("RTT : unable to get RTT major %d\n", RTT_MAJOR);
		rtn_uninit();
		return -EIO;
	}
#endif
	return 0;

}


void cleanup_module(void)
{
#ifdef CONFIG_RTL_POSIX_IO
	rtl_unregister_chrdev(RTT_MAJOR, "rtt");
#endif
	rtn_uninit();
}


#endif				/* MODULE */




/* timer routines for the tulip card */


/* MII transceiver control section.
   Read and write the MII registers using software-generated serial
   MDIO protocol.  See the MII specifications or DP83840A data sheet
   for details. */

/* The maximum data clock rate is 2.5 Mhz.  The minimum timing is usually
   met by back-to-back PCI I/O cycles, but we insert a delay to avoid
   "overclocking" issues or future 66Mhz PCI. */
#define mdio_delay() inl(mdio_addr)

/* Read and write the MII registers using software-generated serial
   MDIO protocol.  It is just different enough from the EEPROM protocol
   to not share code.  The maxium data clock rate is 2.5 Mhz. */
#define MDIO_SHIFT_CLK	0x10000
#define MDIO_DATA_WRITE0 0x00000
#define MDIO_DATA_WRITE1 0x20000
#define MDIO_ENB		0x00000		/* Ignore the 0x02000 databook setting. */
#define MDIO_ENB_IN		0x40000
#define MDIO_DATA_READ	0x80000


static int mdio_read(struct device *dev, int phy_id, int location)
{
	struct tulip_private *tp = (struct tulip_private *)dev->priv;
	int i;
	int read_cmd = (0xf6 << 10) | (phy_id << 5) | location;
	int retval = 0;
	long mdio_addr = dev->base_addr + CSR9;

	if (tp->chip_id == LC82C168) {
		long ioaddr = dev->base_addr;
		int i = 1000;
		outl(0x60020000 + (phy_id<<23) + (location<<18), ioaddr + 0xA0);
		while (--i > 0)
			if ( ! ((retval = inl(ioaddr + 0xA0)) & 0x80000000))
				return retval & 0xffff;
		return 0xffff;
	}

	/* Establish sync by sending at least 32 logic ones. */ 
	for (i = 32; i >= 0; i--) {
		outl(MDIO_ENB | MDIO_DATA_WRITE1, mdio_addr);
		mdio_delay();
		outl(MDIO_ENB | MDIO_DATA_WRITE1 | MDIO_SHIFT_CLK, mdio_addr);
		mdio_delay();
	}
	/* Shift the read command bits out. */
	for (i = 15; i >= 0; i--) {
		int dataval = (read_cmd & (1 << i)) ? MDIO_DATA_WRITE1 : 0;

		outl(MDIO_ENB | dataval, mdio_addr);
		mdio_delay();
		outl(MDIO_ENB | dataval | MDIO_SHIFT_CLK, mdio_addr);
		mdio_delay();
	}
	/* Read the two transition, 16 data, and wire-idle bits. */
	for (i = 19; i > 0; i--) {
		outl(MDIO_ENB_IN, mdio_addr);
		mdio_delay();
		retval = (retval << 1) | ((inl(mdio_addr) & MDIO_DATA_READ) ? 1 : 0);
		outl(MDIO_ENB_IN | MDIO_SHIFT_CLK, mdio_addr);
		mdio_delay();
	}
	return (retval>>1) & 0xffff;
}

static void mdio_write(struct device *dev, int phy_id, int location, int value)
{
	struct tulip_private *tp = (struct tulip_private *)dev->priv;
	int i;
	int cmd = (0x5002 << 16) | (phy_id << 23) | (location<<18) | value;
	long mdio_addr = dev->base_addr + CSR9;

	if (tp->chip_id == LC82C168) {
		long ioaddr = dev->base_addr;
		int i = 1000;
		outl(cmd, ioaddr + 0xA0);
		do
			if ( ! (inl(ioaddr + 0xA0) & 0x80000000))
				break;
		while (--i > 0);
		return;
	}

	/* Establish sync by sending 32 logic ones. */ 
	for (i = 32; i >= 0; i--) {
		outl(MDIO_ENB | MDIO_DATA_WRITE1, mdio_addr);
		mdio_delay();
		outl(MDIO_ENB | MDIO_DATA_WRITE1 | MDIO_SHIFT_CLK, mdio_addr);
		mdio_delay();
	}
	/* Shift the command bits out. */
	for (i = 31; i >= 0; i--) {
		int dataval = (cmd & (1 << i)) ? MDIO_DATA_WRITE1 : 0;
		outl(MDIO_ENB | dataval, mdio_addr);
		mdio_delay();
		outl(MDIO_ENB | dataval | MDIO_SHIFT_CLK, mdio_addr);
		mdio_delay();
	}
	/* Clear out extra bits. */
	for (i = 2; i > 0; i--) {
		outl(MDIO_ENB_IN, mdio_addr);
		mdio_delay();
		outl(MDIO_ENB_IN | MDIO_SHIFT_CLK, mdio_addr);
		mdio_delay();
	}
	return;
}


/* Set up the transceiver control registers for the selected media type. */
static void select_media(struct device *dev, int startup)
{
	long ioaddr = dev->base_addr;
	struct tulip_private *tp = (struct tulip_private *)dev->priv;
	struct mediatable *mtable = tp->mtable;
	u32 new_csr6;
	int check_mii =0, i;

	if (mtable) {
		struct medialeaf *mleaf = &mtable->mleaf[tp->cur_index];
		unsigned char *p = mleaf->leafdata;
		switch (mleaf->type) {
		case 0:					/* 21140 non-MII xcvr. */
			if (tulip_debug > 1)
				printk(KERN_DEBUG "%s: Using a 21140 non-MII transceiver"
					   " with control setting %2.2x.\n",
					   dev->name, p[1]);
			dev->if_port = p[0];
			if (startup)
				outl(mtable->csr12dir | 0x100, ioaddr + CSR12);
			outl(p[1], ioaddr + CSR12);
			new_csr6 = 0x02000000 | ((p[2] & 0x71) << 18);
			break;
		case 2: case 4: {
			u16 setup[3];
			for (i = 0; i < 3; i++)
				setup[i] = get_u16(&p[i*2 + 1]);

			dev->if_port = p[0] & 15;
			if (tulip_debug > 1)
				printk(KERN_DEBUG "%s: 21142 non-MII %s transceiver control %4.4x/%4.4x.\n",
					   dev->name, medianame[dev->if_port], setup[0], setup[1]);
			if (p[0] & 0x40) {	/* SIA (CSR13-15) setup values are provided. */
				outl(0, ioaddr + CSR13);
				outl(setup[1], ioaddr + CSR14);
				outl(setup[2], ioaddr + CSR15);
				outl(setup[0], ioaddr + CSR13);
				for (i = 0; i < 3; i++)			/* Re-fill setup[]  */
					setup[i] = get_u16(&p[i*2 + 7]);
			} else if (dev->if_port <= 4) {
				outl(0, ioaddr + CSR13);
				outl(t21142_csr14[dev->if_port], ioaddr + CSR14);
				outl(t21142_csr15[dev->if_port], ioaddr + CSR15);
				outl(t21142_csr13[dev->if_port], ioaddr + CSR13);
			} else {
				outl(0, ioaddr + CSR14);
				outl(8, ioaddr + CSR15);
				outl(0, ioaddr + CSR13);
			}
			outl(setup[0]<<16, ioaddr + CSR15);	/* Direction */
			outl(setup[1]<<16, ioaddr + CSR15);	/* Data */
			if (mleaf->type == 4)
				new_csr6 = 0x82020000 | ((setup[2] & 0x71) << 18);
			else
				new_csr6 = 0x82420000;
			break;
		}
		case 1: case 3: {
			int phy_num = p[0];
			int init_length = p[1];
			u16 *misc_info;
			u16 to_advertise;

			dev->if_port = 11;
			check_mii = 1;
			new_csr6 = 0x020E0000;
			if (mleaf->type == 3) {	/* 21142 */
				u16 *init_sequence = (u16*)(p+2);
				u16 *reset_sequence = &((u16*)(p+3))[init_length];
				int reset_length = p[2 + init_length*2];
				misc_info = reset_sequence + reset_length;
				if (startup)
					for (i = 0; i < reset_length; i++)
						outl(get_u16(&reset_sequence[i]) << 16, ioaddr + CSR15);
				for (i = 0; i < init_length; i++)
					outl(get_u16(&init_sequence[i]) << 16, ioaddr + CSR15);
			} else {
				u8 *init_sequence = p + 2;
				u8 *reset_sequence = p + 3 + init_length;
				int reset_length = p[2 + init_length];
				misc_info = (u16*)(reset_sequence + reset_length);
				if (startup) {
					outl(mtable->csr12dir | 0x100, ioaddr + CSR12);
					for (i = 0; i < reset_length; i++)
						outl(reset_sequence[i], ioaddr + CSR12);
				}
				for (i = 0; i < init_length; i++)
					outl(init_sequence[i], ioaddr + CSR12);
			}
			to_advertise = (get_u16(&misc_info[1]) & tp->to_advertise) | 1;
			tp->advertising[phy_num] = to_advertise;
			if (tulip_debug > 1 || 1)
				printk(KERN_DEBUG "%s:  Advertising %4.4x on PHY %d (%d).\n",
					   dev->name, to_advertise, phy_num, tp->phys[phy_num]);
			/* Bogus: put in by a committee?  */
			mdio_write(dev, tp->phys[phy_num], 4, to_advertise);
			break;
		}
		default:
		  new_csr6 = 0x020E0000;
		}
		if (tulip_debug > 1)
			printk(KERN_DEBUG "%s: Using media type %s, CSR12 is %2.2x.\n",
				   dev->name, medianame[dev->if_port],
				   inl(ioaddr + CSR12) & 0xff);
	} else if (tp->chip_id == DC21041) {
		if (tulip_debug > 1)
			printk(KERN_DEBUG "%s: 21041 using media %s, CSR12 is %4.4x.\n",
				   dev->name, medianame[dev->if_port & 15],
				   inl(ioaddr + CSR12) & 0xffff);
		outl(0x00000000, ioaddr + CSR13); /* Reset the serial interface */
		outl(t21041_csr14[dev->if_port], ioaddr + CSR14);
		outl(t21041_csr15[dev->if_port], ioaddr + CSR15);
		outl(t21041_csr13[dev->if_port], ioaddr + CSR13);
		new_csr6 = 0x80020000;
	} else if (tp->chip_id == LC82C168) {
		if (startup && ! tp->medialock)
			dev->if_port = tp->mii_cnt ? 11 : 0;
		if (tulip_debug > 1)
			printk(KERN_DEBUG "%s: PNIC PHY status is %3.3x, CSR12 %4.4x,"
				   " media %s.\n",
				   dev->name, inl(ioaddr + 0xB8), inl(ioaddr + CSR12),
				   medianame[dev->if_port]);
		if (tp->mii_cnt) {
			new_csr6 = 0x812C0000;
			outl(0x0001, ioaddr + CSR15);
			outl(0x0201B07A, ioaddr + 0xB8);
		} else if (startup) {
			/* Start with 10mbps to do autonegotiation. */
			outl(0x32, ioaddr + CSR12);
			new_csr6 = 0x00420000;
			outl(0x0001B078, ioaddr + 0xB8);
			outl(0x0201B078, ioaddr + 0xB8);
		} else if (dev->if_port == 3  ||  dev->if_port == 5) {
			outl(0x33, ioaddr + CSR12);
			new_csr6 = 0x01860000;
			if (startup)
				outl(0x0201F868, ioaddr + 0xB8); /* Trigger autonegotiation. */
			else
				outl(0x1F868, ioaddr + 0xB8);
		} else {
			outl(0x32, ioaddr + CSR12);
			new_csr6 = 0x00420000;
			outl(0x1F078, ioaddr + 0xB8);
		}
	} else if (tp->chip_id == DC21040) {					/* 21040 */
		/* Turn on the xcvr interface. */
		int csr12 = inl(ioaddr + CSR12);
		if (tulip_debug > 1)
			printk(KERN_DEBUG "%s: 21040 media type is %s, CSR12 is %2.2x.\n",
				   dev->name, dev->if_port ? "AUI" : "10baseT", csr12);
		new_csr6 = (dev->if_port ? 0x01860000 : 0x00420000);
		/* Set the full duplux match frame. */
		outl(FULL_DUPLEX_MAGIC, ioaddr + CSR11);
		outl(0x00000000, ioaddr + CSR13); /* Reset the serial interface */
		outl(dev->if_port ? 0x0000000C : 0x00000004, ioaddr + CSR13);
	} else {					/* Unknown chip type with no media table. */
		if (tp->default_port == 0)
			if (tp->mii_cnt) {
				dev->if_port = 11;
			} else
				dev->if_port = 3;
		if (media_cap[dev->if_port] & MediaIsMII) {
			new_csr6 = 0x020E0000;
		} else if (media_cap[dev->if_port] & MediaIsFx) {
			new_csr6 = 0x028600000;
		} else
			new_csr6 = 0x038600000;
		if (tulip_debug > 1)
			printk(KERN_DEBUG "%s: No media description table, assuming "
				   "%s transceiver, CSR12 %2.2x.\n",
				   dev->name, medianame[dev->if_port],
				   inl(ioaddr + CSR12));
	}

	tp->csr6 = new_csr6 | (tp->csr6 & 0xfdff) | (tp->full_duplex ? 0x0200 : 0);
	return;
}


static void rtn_tulip_timer(unsigned long data)
{
	struct device *dev = (struct device *)data;
	struct tulip_private *tp = (struct tulip_private *)dev->priv;
	long ioaddr = dev->base_addr;
	u32 csr12 = inl(ioaddr + CSR12);
	int next_tick = 0;

	if (tulip_debug > 3) {
		printk(KERN_DEBUG "%s: Media selection tick, status %8.8x mode %8.8x "
			   "SIA %8.8x %8.8x %8.8x %8.8x.\n",
			   dev->name, inl(ioaddr + CSR5), inl(ioaddr + CSR6),
			   csr12, inl(ioaddr + CSR13),
			   inl(ioaddr + CSR14), inl(ioaddr + CSR15));
	}
	switch (tp->chip_id) {
	case DC21040:
		if (csr12 & 0x0002) { /* Network error */
			printk(KERN_INFO "%s: No 10baseT link beat found, switching to %s media.\n",
				   dev->name, dev->if_port ? "10baseT" : "AUI");
			dev->if_port ^= 1;
			outl(dev->if_port ? 0x0000000C : 0x00000004, ioaddr + CSR13);
			dev->trans_start = jiffies;
		}
		break;
	case DC21041:
		if (tulip_debug > 2)
			printk(KERN_DEBUG "%s: 21041 media tick  CSR12 %8.8x.\n",
				   dev->name, csr12);
		switch (dev->if_port) {
		case 0: case 3: case 4:
		  if (csr12 & 0x0004) { /*LnkFail */
			/* 10baseT is dead.  Check for activity on alternate port. */
			tp->mediasense = 1;
			if (csr12 & 0x0200)
				dev->if_port = 2;
			else
				dev->if_port = 1;
			printk(KERN_INFO "%s: No 21041 10baseT link beat, Media switched to %s.\n",
				   dev->name, medianame[dev->if_port]);
			outl(0, ioaddr + CSR13); /* Reset */
			outl(t21041_csr14[dev->if_port], ioaddr + CSR14);
			outl(t21041_csr15[dev->if_port], ioaddr + CSR15);
			outl(t21041_csr13[dev->if_port], ioaddr + CSR13);
			next_tick = 10*HZ;			/* 2.4 sec. */
		  } else
			next_tick = 30*HZ;
		  break;
		case 1:					/* 10base2 */
		case 2:					/* AUI */
		  if (csr12 & 0x0100) {
			next_tick = (30*HZ);			/* 30 sec. */
			tp->mediasense = 0;
		  } else if ((csr12 & 0x0004) == 0) {
			printk(KERN_INFO "%s: 21041 media switched to 10baseT.\n", dev->name);
			dev->if_port = 0;
			select_media(dev, 0);
			next_tick = (24*HZ)/10;				/* 2.4 sec. */
		  } else if (tp->mediasense || (csr12 & 0x0002)) {
			dev->if_port = 3 - dev->if_port; /* Swap ports. */
			select_media(dev, 0);
			next_tick = 20*HZ;
		  } else {
			next_tick = 20*HZ;
		  }
		  break;
		}
		break;
	case DC21140:  case DC21142: case MX98713: default: {
		struct medialeaf *mleaf;
		unsigned char *p;
		if (tp->mtable == NULL) {	/* No EEPROM info, use generic code. */
			/* Not much that can be done.
			   Assume this a generic MII or SYM transceiver. */
			next_tick = 60*HZ;
			if (tulip_debug > 2)
				printk(KERN_DEBUG "%s: network media monitor CSR6 %8.8x "
					   "CSR12 0x%2.2x.\n",
					   dev->name, inl(ioaddr + CSR6), csr12 & 0xff);
			break;
		}
		mleaf = &tp->mtable->mleaf[tp->cur_index];
		p = mleaf->leafdata;
		switch (mleaf->type) {
		case 0: case 4: {
			/* Type 0 serial or 4 SYM transceiver.  Check the link beat bit. */
			int offset = mleaf->type == 4 ? 5 : 2;
			s8 bitnum = p[offset];
			if (p[offset+1] & 0x80) {
				if (tulip_debug > 1)
					printk(KERN_DEBUG"%s: Transceiver monitor tick "
						   "CSR12=%#2.2x, no media sense.\n",
						   dev->name, csr12);
				if (mleaf->type == 4) {
					if (mleaf->media == 3 && (csr12 & 0x02))
						goto select_next_media;
				}
				break;
			}
			if (tulip_debug > 2)
				printk(KERN_DEBUG "%s: Transceiver monitor tick: CSR12=%#2.2x"
					   " bit %d is %d, expecting %d.\n",
					   dev->name, csr12, (bitnum >> 1) & 7,
					   (csr12 & (1 << ((bitnum >> 1) & 7))) != 0,
					   (bitnum >= 0));
			/* Check that the specified bit has the proper value. */
			if ((bitnum < 0) !=
				((csr12 & (1 << ((bitnum >> 1) & 7))) != 0)) {
				if (tulip_debug > 1)
					printk(KERN_DEBUG "%s: Link beat detected for %s.\n", dev->name,
						   medianame[mleaf->media]);
				if ((p[2] & 0x61) == 0x01)	/* Bogus Znyx board. */
					goto actually_mii;
				break;
			}
			if (tp->medialock)
				break;
	  select_next_media:
			if (--tp->cur_index < 0) {
				/* We start again, but should instead look for default. */
				tp->cur_index = tp->mtable->leafcount - 1;
			}
			dev->if_port = tp->mtable->mleaf[tp->cur_index].media;
			if (media_cap[dev->if_port] & MediaIsFD)
				goto select_next_media; /* Skip FD entries. */
			if (tulip_debug > 1)
				printk(KERN_DEBUG "%s: No link beat on media %s,"
					   " trying transceiver type %s.\n",
					   dev->name, medianame[mleaf->media & 15],
					   medianame[tp->mtable->mleaf[tp->cur_index].media]);
			select_media(dev, 0);
			/* Restart the transmit process. */
			outl(tp->csr6 | 0x0002, ioaddr + CSR6);
			outl(tp->csr6 | 0x2002, ioaddr + CSR6);
			next_tick = (24*HZ)/10;
			break;
		}
		case 1:  case 3: {		/* 21140, 21142 MII */
			int mii_reg1, mii_reg5;
		actually_mii:
			mii_reg1 = mdio_read(dev, tp->phys[0], 1);
			mii_reg5 = mdio_read(dev, tp->phys[0], 5);
			if (tulip_debug > 1)
				printk(KERN_INFO "%s: MII status %4.4x, Link partner report "
					   "%4.4x, CSR12 %2.2x, %cD.\n",
					   dev->name, mii_reg1, mii_reg5, csr12,
					   tp->full_duplex ? 'F' : 'H');
			if (mii_reg1 != 0xffff  &&  (mii_reg1 & 0x0004) == 0) {
				int new_reg1 = mdio_read(dev, tp->phys[0], 1);
				if ((new_reg1 & 0x0004) == 0) {
					printk(KERN_INFO "%s: No link beat on the MII interface,"
						   " status then %4.4x now %4.4x.\n",
						   dev->name, mii_reg1, new_reg1);
					if (tp->mtable  &&  tp->mtable->has_nonmii)
						goto select_next_media;
				}
			}
			if (mii_reg5 == 0xffff  ||  mii_reg5 == 0x0000)
				;				/* No MII device or no link partner report */
			else if (tp->full_duplex_lock)
				;
			else {
				int negotiated = mii_reg5 & tp->advertising[0];
				int duplex = ((negotiated & 0x0100) != 0
							  || (negotiated & 0x00C0) == 0x0040);
				/* 100baseTx-FD  or  10T-FD, but not 100-HD */
				if (tp->full_duplex != duplex) {
					tp->full_duplex = duplex;
					if (tp->full_duplex)
						tp->csr6 |= 0x0200;
					else
						tp->csr6 &= ~0x0200;
					outl(tp->csr6 | 0x0002, ioaddr + CSR6);
					outl(tp->csr6 | 0x2002, ioaddr + CSR6);
					if (tulip_debug > 0) /* Gurppp, should be >1 */
						printk(KERN_INFO "%s: Setting %s-duplex based on MII"
							   " Xcvr #%d parter capability of %4.4x.\n",
							   dev->name, tp->full_duplex ? "full" : "half",
							   tp->phys[0], mii_reg5);
				}
			}
			next_tick = 60*HZ;
			break;
		}
		case 2:					/* 21142 serial block has no link beat. */
		default:
			break;
		}
	}
	break;
	}
	if (next_tick) {
		tp->timer.expires = RUN_AT(next_tick);
		add_timer(&tp->timer);
	}
}

/* Handle the 21143 uniquely: do autoselect with NWay, not the EEPROM list
   of available transceivers.  */
/* Handle the 21143 uniquely: do autoselect with NWay, not the EEPROM list
   of available transceivers.  */
static void rtn_t21142_timer(unsigned long data)
{
	struct device *dev = (struct device *)data;
	struct tulip_private *tp = (struct tulip_private *)dev->priv;
	long ioaddr = dev->base_addr;
	int csr12 = inl(ioaddr + CSR12);
	int next_tick = 60*HZ;
	int new_csr6 = 0;

	if (tulip_debug > 1)
		printk(KERN_INFO"%s: 21142 negotiation status %8.8x, %s.\n",
			   dev->name, csr12, medianame[dev->if_port]);
	if (dev->if_port == 3) {
		if (csr12 & 2) {		/* No 100mbps link beat, revert to 10mbps. */
			new_csr6 = 0x82420200;
			outl(new_csr6, ioaddr + CSR6);
			outl(0x0000, ioaddr + CSR13);
			outl(0x0003FFFF, ioaddr + CSR14);
			outl(0x0008, ioaddr + CSR15);
			outl(0x0001, ioaddr + CSR13);
			outl(0x1301, ioaddr + CSR12); /* Start NWay. */
		}
	} else if ((csr12 & 0x7000) != 0x5000) {
		/* Negotiation failed.  Search media types. */
		if (tulip_debug > 1)
			printk(KERN_INFO"%s: 21142 negotiation failed, status %8.8x.\n",
				   dev->name, csr12);
		if (!(csr12 & 4)) {		/* 10mbps link beat good. */
			new_csr6 = 0x82420000;
			dev->if_port = 0;
			outl(0, ioaddr + CSR13);
			outl(0x0003FFFF, ioaddr + CSR14);
			outl(t21142_csr15[dev->if_port], ioaddr + CSR15);
			outl(t21142_csr13[dev->if_port], ioaddr + CSR13);
		} else if (csr12 & 0x100) {
			new_csr6 = 0x82420200;
			dev->if_port = 2;
			outl(0, ioaddr + CSR13);
			outl(0x0003FFFF, ioaddr + CSR14);
			outl(0x0008, ioaddr + CSR15);
			outl(0x0001, ioaddr + CSR13);
		} else {
			/* Select 100mbps port to check for link beat. */
			new_csr6 = 0x83860000;
			dev->if_port = 3;
			outl(0, ioaddr + CSR13);
			outl(0x0003FF7F, ioaddr + CSR14);
			outl(8, ioaddr + CSR15);
			outl(1, ioaddr + CSR13);
		}
		if (tulip_debug > 1)
			printk(KERN_INFO"%s: Testing new 21142 media %s.\n",
				   dev->name, medianame[dev->if_port]);
		if (new_csr6 != (tp->csr6 & ~0x00D5)) {
			tp->csr6 &= 0x00D5;
			tp->csr6 |= new_csr6;
			outl(0x0301, ioaddr + CSR12);
			outl(tp->csr6 | 0x0002, ioaddr + CSR6);
			outl(tp->csr6 | 0x2002, ioaddr + CSR6);
		}
	}
	tp->timer.expires = RUN_AT(next_tick);
	add_timer(&tp->timer);
}


static void rtn_t21142_lnk_change( struct device *dev)
{
	struct tulip_private *tp = (struct tulip_private *)dev->priv;
	long ioaddr = dev->base_addr;
	int csr12 = inl(ioaddr + CSR12);

	if (tulip_debug > 1)
		printk(KERN_INFO"%s: 21142 link status interrupt %8.8x, CSR5 %x.\n",
			   dev->name, csr12, inl(ioaddr + CSR5));

	if ((csr12 & 0x7000) == 0x5000) {
		if (csr12 & 0x01800000) {
			/* Switch to 100mbps mode. */
			outl(tp->csr6 | 0x0002, ioaddr + CSR6);
			if (csr12 & 0x01000000) {
				dev->if_port = 5;
				tp->csr6 = 0x83860200;
			} else {
				dev->if_port = 3;
				tp->csr6 = 0x83860000;
			}
			outl(tp->csr6 | 0x2002, ioaddr + CSR6);
		} /* Else 10baseT-FD is handled automatically. */
	} else if (dev->if_port == 3) {
		if (!(csr12 & 2))
			printk(KERN_INFO"%s: 21142 100baseTx link beat good.\n",
				   dev->name);
		else
			dev->if_port = 0;
	} else if (dev->if_port == 0) {
		if (!(csr12 & 4))
			printk(KERN_INFO"%s: 21142 10baseT link beat good.\n",
				   dev->name);
	} else if (!(csr12 & 4)) {		/* 10mbps link beat good. */
			printk(KERN_INFO"%s: 21142 10mpbs sensed media.\n",
				   dev->name);
			dev->if_port = 0;
	} else  {		/* 100mbps link beat good. */
		printk(KERN_INFO"%s: 21142 100baseTx sensed media.\n",
			   dev->name);
		dev->if_port = 3;
		tp->csr6 = 0x83860000;
		outl(0x0003FF7F, ioaddr + CSR14);
		outl(0x0301, ioaddr + CSR12);
		outl(tp->csr6 | 0x0002, ioaddr + CSR6);
		outl(tp->csr6 | 0x2002, ioaddr + CSR6);
	}
}

static void rtn_mxic_timer(unsigned long data)
{
	struct device *dev = (struct device *)data;
	struct tulip_private *tp = (struct tulip_private *)dev->priv;
	long ioaddr = dev->base_addr;
	int next_tick = 60*HZ;

	if (tulip_debug > 3) {
		printk(KERN_INFO"%s: MXIC negotiation status %8.8x.\n", dev->name,
			   inl(ioaddr + CSR12));
	}
	if (next_tick) {
		tp->timer.expires = RUN_AT(next_tick);
		add_timer(&tp->timer);
	}
}

static void rtn_pnic_timer(unsigned long data)
{
	struct device *dev = (struct device *)data;
	struct tulip_private *tp = (struct tulip_private *)dev->priv;
	long ioaddr = dev->base_addr;
	int csr12 = inl(ioaddr + CSR12);
	int next_tick = 60*HZ;
	int new_csr6 = tp->csr6 & ~0x40C40200;

	if (media_cap[dev->if_port] & MediaIsMII) {
		int negotiated = mdio_read(dev, tp->phys[0], 5) & tp->advertising[0];

		if (tulip_debug > 1)
			printk(KERN_DEBUG "%s: LC82C168 negotiated capability %8.8x, "
				   "CSR5 %8.8x.\n",
				   dev->name, negotiated, inl(ioaddr + CSR5));

		if (negotiated & 0x0380) 				/* 10 vs 100mbps */
			new_csr6 |= 0x812E0000;
		else
			new_csr6 |= 0x816E0000;
		if (((negotiated & 0x0300) == 0x0100)			/* Duplex */
			|| (negotiated & 0x00C0) == 0x0040
			|| tp->full_duplex_lock) {
			tp->full_duplex = 1;
			new_csr6 |= 0x0200;
		}
		if (tulip_debug > 1)
			printk(KERN_DEBUG "%s: LC82C168 MII PHY status %4.4x, Link "
				   "partner report %4.4x, csr6 %8.8x/%8.8x.\n",
			   dev->name, mdio_read(dev, tp->phys[0], 1), negotiated,
				   tp->csr6, inl(ioaddr + CSR6));
	} else {
		int phy_reg = inl(ioaddr + 0xB8);
		int csr5 = inl(ioaddr + CSR5);

		if (tulip_debug > 1)
			printk(KERN_DEBUG "%s: LC82C168 phy status %8.8x, CSR5 %8.8x.\n",
				   dev->name, phy_reg, csr5);

		if (phy_reg & 0x04000000) {	/* Remote link fault */
			/*outl(0x0201F078, ioaddr + 0xB8);*/
			next_tick = 3*HZ;
		}
		if (inl(ioaddr + CSR5) & TPLnkFail) { /* 100baseTx link beat */
			if (tulip_debug > 1)
				printk(KERN_DEBUG "%s: %s link beat failed, CSR12 %4.4x, "
					   "CSR5 %8.8x, PHY %3.3x.\n",
					   dev->name, medianame[dev->if_port], csr12,
					   inl(ioaddr + CSR5), inl(ioaddr + 0xB8));
			if (tp->medialock) {
			} else if (dev->if_port == 0) {
				dev->if_port = 3;
				outl(0x33, ioaddr + CSR12);
				new_csr6 = 0x01860000;
				outl(0x1F868, ioaddr + 0xB8);
			} else {
				dev->if_port = 0;
				outl(0x32, ioaddr + CSR12);
				new_csr6 = 0x00420000;
				outl(0x1F078, ioaddr + 0xB8);
			}
			new_csr6 |= (tp->csr6 & 0xfdff);
			next_tick = 3*HZ;
		} else
			new_csr6 = tp->csr6;
		if (tp->full_duplex_lock  ||  (phy_reg & 0x30000000) != 0) {
			tp->full_duplex = 1;
			new_csr6 |= 0x00000200;
		}
	}
	if (tp->csr6 != new_csr6) {
		tp->csr6 = new_csr6;
		outl(tp->csr6 | 0x0002, ioaddr + CSR6);	/* Restart Tx */
		outl(tp->csr6 | 0x2002, ioaddr + CSR6);
		dev->trans_start = jiffies;
		if (tulip_debug > 0) /* Gurppp, should be >1 */
			printk(KERN_INFO "%s: Changing PNIC configuration to %s-duplex, "
				   "CSR6 %8.8x.\n",
				   dev->name, tp->full_duplex ? "full" : "half", new_csr6);
	}
	tp->timer.expires = RUN_AT(next_tick);
	add_timer(&tp->timer);
}


static void rtn_tulip_tx_timeout(struct device *dev)
{
  struct tulip_private *tp = (struct tulip_private *)dev->priv;
  long ioaddr = dev->base_addr;

  if (media_cap[dev->if_port] & MediaIsMII) {
	  /* Do nothing -- the media monitor should handle this. */
	  if (tulip_debug > 1)
		  printk(KERN_WARNING "%s: Transmit timeout using MII device.\n",
				 dev->name);
	  dev->trans_start = jiffies;
	  return;
  } else if (tp->chip_id == DC21040) {
	  if (inl(ioaddr + CSR12) & 0x0002) {
		  printk(KERN_INFO "%s: transmit timed out, switching to %s media.\n",
				 dev->name, dev->if_port ? "10baseT" : "AUI");
		  dev->if_port ^= 1;
		  outl(dev->if_port ? 0x0000000C : 0x00000004, ioaddr + CSR13);
	  }
	  dev->trans_start = jiffies;
	  return;
  } else if (tp->chip_id == DC21041) {
	u32 csr12 = inl(ioaddr + CSR12);

	printk(KERN_WARNING "%s: 21041 transmit timed out, status %8.8x, CSR12 %8.8x,"
		   " CSR13 %8.8x, CSR14 %8.8x, resetting...\n",
		   dev->name, inl(ioaddr + CSR5), csr12,
		   inl(ioaddr + CSR13), inl(ioaddr + CSR14));
	tp->mediasense = 1;
	if (dev->if_port == 1 || dev->if_port == 2)
		if (csr12 & 0x0004) {
			dev->if_port = 2 - dev->if_port;
		} else
			dev->if_port = 0;
	else
		dev->if_port = 1;
	select_media(dev, 0);
	tp->stats.tx_errors++;
	dev->trans_start = jiffies;
	return;
  } else if (tp->chip_id == DC21140 || tp->chip_id == DC21142
			 || tp->chip_id == MX98713) {
	  /* Stop the transmit process. */
	  outl(tp->csr6 | 0x0002, ioaddr + CSR6);
	  printk(KERN_WARNING "%s: 21140 transmit timed out, status %8.8x, "
			 "SIA %8.8x %8.8x %8.8x %8.8x, resetting...\n",
			 dev->name, inl(ioaddr + CSR5), inl(ioaddr + CSR12),
			 inl(ioaddr + CSR13), inl(ioaddr + CSR14), inl(ioaddr + CSR15));
	  if (tp->mtable) {
		  if (--tp->cur_index < 0) {
			  /* We start again, but should instead look for default. */
			  tp->cur_index = tp->mtable->leafcount - 1;
		  }
		  select_media(dev, 0);
		  printk(KERN_WARNING "%s: transmit timed out, switching to %s media.\n",
				 dev->name, dev->if_port ? "100baseTx" : "10baseT");
	  }
	  outl(tp->csr6 | 0x2002, ioaddr + CSR6);
	  tp->stats.tx_errors++;
	  dev->trans_start = jiffies;
	  return;
  } else
	printk(KERN_WARNING "%s: transmit timed out, status %8.8x, CSR12 %8.8x,"
		   " resetting...\n",
		   dev->name, inl(ioaddr + CSR5), inl(ioaddr + CSR12));
#ifdef way_too_many_messages
  printk("  Rx ring %8.8x: ", (int)tp->rx_ring);
  for (i = 0; i < RX_RING_SIZE; i++)
	printk(" %8.8x", (unsigned int)tp->rx_ring[i].status);
  printk("\n  Tx ring %8.8x: ", (int)tp->tx_ring);
  for (i = 0; i < TX_RING_SIZE; i++)
	printk(" %8.8x", (unsigned int)tp->tx_ring[i].status);
  printk("\n");
#endif

  /* Perhaps we should reinitialize the hardware here. */
  dev->if_port = 0;
  /* Stop and restart the chip's Tx processes . */
  outl(tp->csr6 | 0x0002, ioaddr + CSR6);
  outl(tp->csr6 | 0x2002, ioaddr + CSR6);
  /* Trigger an immediate transmit demand. */
  outl(0, ioaddr + CSR1);

  dev->trans_start = jiffies;
  tp->stats.tx_errors++;
  return;
}

/*
 * RTL=rtl directory
 * Compile-command : "gcc -I$(RTL)/include -DMODULE -D__KERNEL__ -O6 -c rtn_module.c -Wall -D__RTL__
 */
