/*
 * RT-LINUX Based Network Driver. Header File
 *
 * Written by Axel Bernal	E-mail: abernal@nmt.edu
 * Released under the terms of the GNU Library General Public License
 *
 */

#ifndef __RTN_H__
#define __RTN_H__

#define SEND_FRM	1
#define RCV_FRM		2
#define RCV_FILE	3
#define SND_FILE	4
#define CHG_DADDR	5
#define NUM_RTN_SKB_OUT	64
#define MASK_RTN 	0x0000
#define MAX_RCV_WORK	4
#define MAX_SND_WORK	4
#define ALL_PACKETS	0
#define PKT_BUF_SZ 	1536  /* Size of each temporary rx buffer */

/* IOCTL Stuff */
#define CHANGE_DEST_ADDR 0x101

/* Definitions for the pipes */
#define MAX_SKB_SND_PIPE 16
#define MAX_SKB_RCV_PIPE 16
#define SND_PIPE 1
#define RCV_PIPE 2
#define CTL_PIPE 3
#define SIZE_SND_PIPE 16*1536
#define SIZE_RCV_PIPE 16*1536
#define SIZE_CTL_PIPE 4092 

/* RTL POSIX DEFINITIONS */
#define RTT_MAJOR 160 

/*
 * This is the constant for the RTN protocol, but we have 
 * to be careful if it is used in next versions.
 */
#define ETH_P_RTN 	0x0018


#include <linux/module.h>
#include <linux/types.h>
#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/string.h>
#include <linux/timer.h>
#include <linux/ptrace.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <linux/malloc.h>
#include <linux/interrupt.h>
#include <linux/pci.h>

#include <asm/processor.h>		/* Processor type for cache alignment. */
#include <asm/bitops.h>
#include <asm/io.h>
#include <asm/dma.h>

#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/skbuff.h>



#include <linux/delay.h>	/* for udelay() */
#include <linux/skbuff.h>
#include <linux/if_ether.h>
#include <linux/version.h>
#include <linux/errno.h>

#include <asm/uaccess.h>
#include <asm/system.h>
#include <linux/in6.h>
#include <asm/checksum.h>
#include <rtl_sync.h>
#include <linux/rtl.h>

#include <linux/cons.h>
#include <asm/spinlock.h>

#include <rtl_sched.h>
#include <rtl_fifo.h>


/* Keep the ring sizes a power of two for efficiency.
   Making the Tx ring too large decreases the effectiveness of channel
   bonding and packet priority.
   There are no ill effects from too-large receive rings. */



#define TX_RING_SIZE	16
#define RX_RING_SIZE	32
#define POOL_RING_SIZE	32 /* size of the swapping space */


/* Time in jiffies before concluding the transmitter is hung. */

#define TX_TIMEOUT  (4*HZ)

/* This is a mysterious value that can be written to CSR11 in the 21040 (only)
   to support a pre-NWay full-duplex signaling mechanism using short frames.
   No one knows what it should be, but if left at its default value some
   10base2(!) packets trigger a full-duplex-request interrupt. */

#define FULL_DUPLEX_MAGIC	0x6969

/* 
 * Structures used by the rtn module 
 */

#define FRM_HEADER	0x0001
#define FILE_HEADER 0x0002
#define FILE_TAIL 	0x0003

struct my_msg_struct {
	int command;
	int length;
};


struct rtn_global {
   
/* Queues */
   struct sk_buff_head skb_in_queue; /* rx queue */
   struct sk_buff_head skb_out_queue; /* tx queue */
   struct sk_buff_head skb_free_queue; /* to be freed */
/* The pre-allocated number of sk buffers for the copy of the Tulip Ring */
   struct sk_buff_head rx_skbuff_queue;

/* Pool of sk buffers */
   struct sk_buff * skb_pool_ring[RX_RING_SIZE];
   struct sk_buff * skb_out_ring[NUM_RTN_SKB_OUT];

/* pointers for the skb pool */
   volatile unsigned int dirty_out, cur_out;	/* pool of skbuffs for out */ 
   volatile unsigned int dirty_pool, cur_pool;	/* The ring entries to be allocated again. */ 
/* spinlocks */
   spinlock_t pool_lock;
   spinlock_t skb_out_lock;
   spinlock_t in_queue_lock; /* lock for the in queue */
   spinlock_t out_queue_lock;  /* lock for the out queue */
   spinlock_t free_queue_lock;   
   spinlock_t rx_lock;
   spinlock_t txmit_lock;
/* statistical */
   unsigned long rx_packets;
   unsigned long rx_bytes;
   int sleeps;
   struct wait_queue *wait;

};
   
#endif



