/************************************************************************
 *                                                                      *
 *  bmc1000.h  ---  daq inputs header file for bmc1000.c.               *
 *                                                                      *
 *  This file is open source software; you can redistribute it and/or   *
 *  modify it under the terms of the version 2 of GNU General Public    *
 *  License as published by the Free Software Foundation.               *
 *                                                                      *
 *  This program is distributed in the hope that it will be useful,     *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       *
 *  GNU General Public License for more details.                        *
 *                                                                      *
 *  You should have received a copy of the GNU General Public License   *
 *  along with this package; if not, write to the Free Software         *
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.           *
 *  or look at http://www.gnu.org/fsf/fsf.html                          *
 *                                                                      *
 *  Copyleft, Peter Wurmsdobler, peterw@thinkingnerds.com               *
 *                                                                      *
 ************************************************************************/


/************************************************************************
 *  io stuff                                                            * 
 ************************************************************************/

/* AI range
 */
#define BMC1000_AI_PM10     (0x00) /* [ -10,  10   ] Volt */
#define BMC1000_AI_PM5      (0x20) /* [  -5,   5   ] Volt */
#define BMC1000_AI_PM2      (0x40) /* [  -2,   2   ] Volt */
#define BMC1000_AI_PM1      (0x60) /* [  -1,   1   ] Volt */

/* DIO port type
 */
#define BMC1000_DIO_INPUT    0x00 
#define BMC1000_DIO_OUTPUT   0x01 


/***********************************************************************
 * Analog input basic functions ...                                    *
 ***********************************************************************/

/* select channel for successive conversion
 */
extern void bmc1000_ai_adc_select( unsigned short int channel );

/* program ADC for successive conversion, with both channel 
 * and range as defined in header file.
 */
extern void bmc1000_ai_adc_program( unsigned short int channel,
    unsigned short int range );

/* get a single analog value given that channel is already
 * selected and its range set, mode is software.
 */
extern int bmc1000_ai_get_value0( unsigned short int *value );

/* get a single analog value( channel ) given that its range set.
 */
extern int bmc1000_ai_get_value1( unsigned short int *value,
    unsigned short int channel );

/* get a single analog value( channel, range ).
 */
extern int bmc1000_ai_get_value2( unsigned short int *value,
    unsigned short int channel, unsigned short int range );


/***********************************************************************
 * Analog output basic functions ...                                   *
 ***********************************************************************/

/* select channel for successive DA conversion
 */
extern void bmc1000_ao_dac_select( unsigned short int channel );

/* set a single analog value given that channel is already
 * selected and ints range set value mapped to 16bit.
 */
extern void bmc1000_ao_set_value0( unsigned short int value );

/* set a single analog value (channel) given that its range set.
 * value is mapped to 16bit independently of DAC.
 */
extern void bmc1000_ao_set_value1( unsigned short int value, 
    unsigned short int channel );


/***********************************************************************
 * Digital input/output basic functions ...                            *
 ***********************************************************************/

/*bmc1000_dio_set_byte  ---  set byte at port 0-3 by "byte".
 */ 
extern void bmc1000_dio_set_byte( unsigned char byte,
    unsigned short int nport );

/*set bit at port, channel 0-7, by "bit" 0 or 1.
 */ 
extern void bmc1000_dio_set_bit( unsigned short int bit,
    unsigned short int nport, unsigned short int nbit );
    
/* bmc1000_dio_get_byte  ---  get byte at port 0-3 into "byte"
 */ 
extern unsigned char bmc1000_dio_get_byte( unsigned short int nport );

/* bmc1000_dio_get_bit  ---  get bit at port 0-5, channel 0-7,
 * into "bit".
 */ 
extern unsigned short int bmc1000_dio_get_bit( unsigned short int nport,
    unsigned short int nbit );

/* config ports as INPUT or OUTPUT, if output, set to zero.
 */ 
extern void bmc1000_dio_conf_port( unsigned short int nport,
    unsigned char type );
