/***********************************************************************
 *                                                                      *
 *  rt_demo.c  ---   real time demo and test module.                    *
 *                                                                      *
 *  This file is open source software; you can redistribute it and/or   *
 *  modify it under the terms of the version 2 of GNU General Public    *
 *  License as published by the Free Software Foundation.               *
 *                                                                      *
 *  This program is distributed in the hope that it will be useful,     *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       *
 *  GNU General Public License for more details.                        *
 *                                                                      *
 *  You should have received a copy of the GNU General Public License   *
 *  along with the control/rt_control/xcontrol package; if not, write   *
 *  to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,     *
 *  MA 02139, USA.                                                      *
 *                                                                      *
 *  Copyleft, Peter Wurmsdobler, peterw@thinkingnerds.com               *
 *                                                                      *
 ************************************************************************/

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/string.h>
#include <linux/errno.h>
#include <linux/time.h>
#include <linux/mm.h>
#include <rtl_fifo.h>
#include <rtl_time.h>
#include <rtl_sched.h>
#include "acl6126.h"
#include "bmc1000.h"
#include "pci180x.h"
#include "pciad16da4.h"
#include "pcl818.h"
#include "piod48.h"
#include "piodax.h"
#include "spp.h"

//#define DEBUG
#ifdef DEBUG
    #define DPRINTK(args...)    rtl_printf(## args)
#else
    #define DPRINTK(args...)
#endif

#define DEMO_NAME       ("rt_demo:")      

#define TS 1000000000  /* in ns */

pthread_t demo;
void *rt_demo_thread( void *t )
    {
    int channel = 0;
    unsigned short int value = 0x0;
 
/*
    piod48_dio_conf_port( 0x0, PIOD48_DIO_OUTPUT );  
    piod48_dio_conf_port( 0x1, PIOD48_DIO_OUTPUT );   
    piod48_dio_conf_port( 0x2, PIOD48_DIO_INPUT ); 
*/

/*
    for ( channel=0; channel<4; channel++ )
        piodax_ao_dac_program( channel, PIODAX_AO_PM5 );
*/

    for ( channel=0; channel<16; channel++ )
        pci180x_ai_adc_program( channel, PCI180X_AI_PM5 );

    pthread_make_periodic_np( demo, gethrtime(), TS );
    DPRINTK( "%s Starting periodic thread\n", DEMO_NAME ); 

    while(1)
        {
        pthread_wait_np(); 
        DPRINTK( "%s resume thread\n", DEMO_NAME );
       
/*
        value = piod48_dio_get_bit( 0x2, 3 );
        rtl_printf( "value = 0x%x\n", value );
*/

/*
        for ( channel=0; channel<4; channel++ )
            piodax_ao_set_value1( value, channel );
        rtl_printf( "value = 0x%x\n", value );
        value += 0x100;
*/

        for ( channel=0; channel<16; channel++ )
            {
            pci180x_ai_get_value1( &value, channel );
            rtl_printf( "value[%d] = 0x%x\n", channel,value );
            }
        }
    } 

int init_module(void)
    {
    pthread_attr_t attr;           
    struct sched_param sched_param;
        
    pthread_attr_init( &attr );
    pthread_attr_setcpu_np( &attr, 0 );
    sched_param.sched_priority = 1;
    pthread_attr_setschedparam( &attr, &sched_param );
    if ( pthread_create( &demo,  &attr, rt_demo_thread, (void *)1 ) )
       return -EAGAIN;

    return 0;
    }
        
void cleanup_module(void)
    {
    pthread_delete_np( demo );                    
    }
