/************************************************************************
 *                                                                      *
 *  pciad16da4.h  ---  daq inputs header file for pciad16da4.c.         *
 *                                                                      *
 *  This file is open source software; you can redistribute it and/or   *
 *  modify it under the terms of the version 2 of GNU General Public    *
 *  License as published by the Free Software Foundation.               *
 *                                                                      *
 *  This program is distributed in the hope that it will be useful,     *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       *
 *  GNU General Public License for more details.                        *
 *                                                                      *
 *  You should have received a copy of the GNU General Public License   *
 *  along with this package; if not, write to the Free Software         *
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.           *
 *  or look at http://www.gnu.org/fsf/fsf.html                          *
 *                                                                      *
 *  Copyleft, Peter Wurmsdobler, peterw@thinkingnerds.com               *
 *                                                                      *
 ************************************************************************/

/************************************************************************
 *  io stuff                                                            * 
 ************************************************************************/

/* AI range
 */
#define PCIAD16DA4_AI_PM1    (0x0140)    /* [  -1.0,    1.0   ] Volt */
#define PCIAD16DA4_AI_PM5    (0x0000)    /* [  -5.0,    5.0   ] Volt */
#define PCIAD16DA4_AI_PM10   (0x0100)    /* [ -10.0,   10.0   ] Volt */
#define PCIAD16DA4_AI_0P1    (0x0240)    /* [   0.0,    1.0   ] Volt */
#define PCIAD16DA4_AI_0P10   (0x0200)    /* [   0.0,   10.0   ] Volt */


/***********************************************************************
 * Analog input basic functions ...                                    *
 ***********************************************************************/

/* select channel for successive conversion
 */
extern void pciad16da4_ai_adc_select( unsigned short int channel );

/* get a single analog value given that channel is already
 * selected and ints range set, mode is software.
 */
extern int pciad16da4_ai_get_value0( unsigned short int *value );

/* get a single analog value( channel ) given that its range set.
 */
extern int pciad16da4_ai_get_value1( unsigned short int *value, 
    unsigned short int channel );


/***********************************************************************
 * Analog output basic functions ...                                   *
 ***********************************************************************/

/* select channel for successive DA conversion
 */
extern void pciad16da4_ao_dac_select( unsigned short int channel );

/* set a single analog value given that channel is already
 * selected and ints range set value mapped to 16bit.
 */
extern void pciad16da4_ao_set_value0( unsigned short int value );

/* set a single analog value (channel) given that its range set.
 * value is mapped to 16bit independently of DAC.
 */
extern void pciad16da4_ao_set_value1( unsigned short int value, 
    unsigned short int channel );


/***********************************************************************
 * Digital input/output basic functions ...                            *
 ***********************************************************************/

/* get byte from port "nport".
 */ 
extern unsigned char pciad16da4_di_get_byte( unsigned short int nport );

/* get bit #"nbit" from port "nport".
 */ 
extern unsigned short int pciad16da4_di_get_bit( unsigned short int nport, 
    unsigned short int nbit );

/* set byte "byte" on port "nport"
 */ 
extern void pciad16da4_do_set_byte( unsigned char byte,
    unsigned short int nport );

/* set bit "bit" with #"nbit" on port "nport".
 */ 
extern void pciad16da4_do_set_bit( unsigned short int bit,
    unsigned short int nport, unsigned short int nbit );
