/************************************************************************
 *                                                                      *
 *  spp.c  ---   real time standard parallel port module.               *
 *                                                                      *
 *  This file is open source software; you can redistribute it and/or   *
 *  modify it under the terms of the version 2 of GNU General Public    *
 *  License as published by the Free Software Foundation.               *
 *                                                                      *
 *  This program is distributed in the hope that it will be useful,     *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       *
 *  GNU General Public License for more details.                        *
 *                                                                      *
 *  You should have received a copy of the GNU General Public License   *
 *  along with the control/rt_control/xcontrol package; if not, write   *
 *  to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,     *
 *  MA 02139, USA.                                                      *
 *                                                                      *
 *  Copyleft, Peter Wurmsdobler, peterw@thinkingnerds.com               *
 *                                                                      *
 ************************************************************************/

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/mc146818rtc.h>
#include <linux/ioport.h>
#include "spp.h"

//#define DEBUG
#ifdef DEBUG
    #define DPRINTK(args...)    printk(## args)
#else
    #define DPRINTK(args...)
#endif

/* some SPP defines
 */
#define SPP_NAME    ("spp:")
#define SPP_BASE    (0x278)
#define SPP_RANGE   (3)
#define SPP_IRQ     (5)
#define SPP_DAT     (0x00)
#define SPP_STA     (0x01)
#define SPP_CTR     (0x02)


/************************************************************************
 * global values                                                        * 
 ************************************************************************/

struct
    {
    unsigned short int base_io;
    unsigned short int irq;
    } 
    spp;

/* dis/enable parallel port interrupts
 */
void spp_set_status( char status )
    {
    /* enable interrupt on receiver line status
     * nAcck on Pin10 is inverted!
     */
    outb( status, SPP_BASE+SPP_CTR );
    }

/* get number of interrupt line, irq handler can be defined outside
 */
int spp_irq_get( void )
    {
    return spp.irq;
    }

/*irq handler for spp 
 */
unsigned int spp_irq_handler( unsigned int irq_number, 
    struct pt_regs *p )
    {
    /* nothing done her, use this template in your real time application !
     */
    
    return 0;
    }

/* init_module  ---  the compulsory init function.
 */
int init_module(void)
    {
    /* request the SPP region, interrupt can be requested outside.
     */
    spp.irq = SPP_IRQ;
    printk("%s IRQ is selected to be %d (0 = no interrupt)\n", 
        SPP_NAME, spp.irq );

    spp.base_io = SPP_BASE;
    if ( check_region( spp.base_io, SPP_RANGE ) < 0 ) 
        {
     	printk( "%s could not request base io addresss\n", SPP_NAME );
        return -EIO;
        }
    request_region( spp.base_io, SPP_RANGE, SPP_NAME );
    printk( "%s requested base io address 0x%x\n", SPP_NAME, 
        spp.base_io );

    spp_set_status( SPP_DISABLE );

    return 0;
    }
        
/* cleanup_module  ---  the compulsory cleanup function
 */
void cleanup_module(void)
    {
    /* Release the SPP
     */
    spp_set_status( SPP_DISABLE );
    release_region( spp.base_io, SPP_RANGE );
    }
