#ifndef __RTN_LIB__
#define __RTN_LIB__
/*
 * Some useful Library Routines for the use of RTN module in Linux 
 * Header File
 * Written by Axel E. Bernal  	E-mail : abernal@nmt.edu
 * Released under the terms of the GNU Library General Public License
 */


#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
 #include <sys/types.h>

/* ioctl definitions */

#define SND_FRM		1
#define RCV_FRM		2
#define RCV_FILE	3
#define SND_FILE 	4
#define CHG_DADDR	5	
#define WAIT_ACK 	6
#define SEND_ACK 	7

#define PKT_BUF_SZ 1492


#define FILE_HEADER 	0x10000000
#define FRM_HEADER	0x20000000
#define FILE_TAIL	0x30000000
#define ACK_FILE 	0x40000000

/* 
 * Structures used by the rtn module 
 */


#define MILLION 1000000


struct my_msg_struct {
	int command;
	int length;
};

#define BUFSIZE 3072;

#define WRITE_RTN_PACKET(PIPE_CTL,PIPE_OUT, BUFFER, COUNT, MSG)	\
	if(((COUNT) = write((PIPE_OUT),(BUFFER),(MSG).length)) < 0) { \
		fprintf(stderr, "Can't send a command to RT-task\n"); \
		return -1; \
	} \
	if(write((PIPE_CTL), &(MSG), sizeof(MSG)) < 0) { \
		fprintf(stderr, "Can't send a command to RT-task\n"); \
		return -1; \
	}           							

#define READ_RTN_PACKET(PIPE_CTL,PIPE_IN,BUFFER,COUNT,MSG) \
	if (write((PIPE_CTL), &(MSG), sizeof(MSG)) < 0) { \
		fprintf(stderr, "Can't send a command to RT-task\n"); \
		return -1; \
	} \
   	if(((COUNT) = read((PIPE_IN),(BUFFER),PKT_BUF_SZ))<0) { \
		fprintf(stderr, "Can't send a command to RT-task\n"); \
		return -1; \
	}           						

#define IS_BEGIN_FILE(BUFFER) \
((*((unsigned int *)(BUFFER)))&0xffff0000) == FILE_HEADER 	

#define IS_END_FILE(BUFFER) \
((*((unsigned int *)(BUFFER)))&0xffff0000) == FILE_TAIL 	

#define IS_ACK(BUFFER) \
((*((unsigned int *)(buffer)))&0xffff0000) == ACK_FILE 		

#define BUILD_ACK(BUFFER, MSG) \
 	*((unsigned int *)(BUFFER)) = ACK_FILE|((MSG).length & 0x0000ffff) 

#define BUILD_FRAME_HEADER(BUFFER,MSG) \
	*((unsigned int *)(BUFFER)) = FRM_HEADER|((MSG).length & 0x0000ffff) 

#define BUILD_FILE_TAIL(BUFFER,MSG) \
	*((unsigned int *)(BUFFER)) = FILE_TAIL|((MSG).length&0x0000ffff) 

#define BUILD_FILE_HEADER(BUFFER,MSG) \
	*((unsigned int *)(BUFFER))= FILE_HEADER|((MSG).length&0x0000ffff)   


int open_rtn_device();
int close_rtn_device();
inline int ioctl_rtn_packet(int, unsigned long);

#endif

