/*
 * An Example to transmit a file using library functions of the RTN module
 * in Linux. 
 * Source File
 * Written by Axel E. Bernal  	E-mail : abernal@nmt.edu
 * Released under the terms of the GNU Library General Public License
 */

#include "rtt_lib.h"




static char buffer[3072];

int fd0, fd1, ctl;

extern int volatile fd0,fd1,ctl;

int open_rtn_device()
{
	int n;
	int fd_flags;
	struct my_msg_struct msg;
	
	if ((fd0 = open("/dev/rtf1", O_WRONLY)) < 0) {
		fprintf(stderr, "Error opening /dev/rtf1\n");
		return -1;
	}

	if ((fd1 = open("/dev/rtf2", O_RDONLY)) < 0) {
		fprintf(stderr, "Error opening /dev/rtf2\n");
		return -1;
	}
	if((fd_flags = fcntl(fd1,F_GETFL,0)) == -1) {
		fprintf(stderr, "Could not get flags\n");
		return -1;
	}
	else {
		fd_flags |= O_NONBLOCK;
		if((fd_flags = fcntl(fd1,F_SETFL,fd_flags)) == -1) {
			fprintf(stderr, "Could not set flags\n");
			return -1;
		}
	}	

	if ((ctl = open("/dev/rtf3", O_WRONLY)) < 0) {
		fprintf(stderr, "Error opening /dev/rtf3\n");
		return -1;
	}
}

int close_rtn_device()
{
	close(fd0);
	close(fd1);
	close(ctl);
}


int chg_address(char *buf) {
	int act_len;
	struct my_msg_struct msg;
	// sending ack
	// changing the remote address
	msg.command = CHG_DADDR;
	msg.length = 6;
     	WRITE_RTN_PACKET(ctl,fd0, buf,act_len,msg);
	return 0;
}
 
static int send_ack () {	
	int accum;
	int act_len, i;
	struct my_msg_struct msg;
	// sending ack
	msg.command = SND_FRM;
	msg.length = sizeof(unsigned int);
     	BUILD_ACK(buffer, msg); 
     	WRITE_RTN_PACKET(ctl,fd0, buffer,act_len,msg);
	return 0;
}


int wait_ack() {
	static int debug = 0;
	int act_len;
	struct my_msg_struct msg;
	msg.command = RCV_FRM;
	msg.length = PKT_BUF_SZ;
	while(1) {
        	READ_RTN_PACKET(ctl,fd1,buffer,act_len,msg);
		if(debug++ == 10000000) { debug = 0; printf("Bad Ack\n"); }
		if(act_len == 0) continue;
          	if(IS_ACK(buffer)) 
			return 1;
		else
			return 0;
	}
}


inline int ioctl_rtn_device(int cmd, unsigned long arg) {
	switch(cmd) {
		case 	WAIT_ACK :              
             			return wait_ack();
		case 	SEND_ACK :
			       // sending ack
                         return send_ack();
		case 	CHG_DADDR :
             			return chg_address((char *)arg);
          default : return -1;
	}
}


static int rtn_rcv_file(int *fds, char **name) {
	int volatile accum;
	int volatile act_len;
   	static int debug;
// This code is for the receiver 
	struct my_msg_struct msg;
	 // the name of the file : we could pass also the permissions
      // and all the information... but for now this is enough 
      // to show the operation of the driver	
	msg.command = RCV_FRM;
	msg.length = PKT_BUF_SZ;
	while(1) {
        	READ_RTN_PACKET(ctl,fd1,buffer,act_len,msg);
		if(act_len > 0) break;
	}
	accum = act_len - sizeof(unsigned int);
	if(IS_BEGIN_FILE(buffer)) { // create file in fs 
		buffer[act_len] = 0;
		*name = strdup(buffer+sizeof(unsigned int));
		if((*fds = open(*name,O_RDWR|O_CREAT)) < 0) {
			fprintf(stderr, "Error opening des_file\n");
			return -1;
		}	
	}
	else {
		fprintf(stderr, "Not a header skbuff\n");
		return -1;
	}
	msg.command = RCV_FRM;
	msg.length = PKT_BUF_SZ;
	while(1) {
        	READ_RTN_PACKET(ctl,fd1,buffer,act_len,msg);
          if(act_len == 0)  continue; // no data
		accum +=act_len - sizeof(unsigned int);
		if(debug++ == 500000000) { debug = 0; printf("S"); }
		write(*fds, buffer+sizeof(unsigned int), act_len - sizeof(unsigned int));
		if(IS_END_FILE(buffer))
		       	break;
	} 
	return accum;

}

static int rtn_snd_file(int fds, char *name) {
	int accum;
	int act_len, i;
	struct my_msg_struct msg;

/* This code is for the SENDER */
	/* Sending the name */
	msg.length = strlen(name) + sizeof(unsigned int);
	msg.command = SND_FRM;
     	BUILD_FILE_HEADER(buffer,msg);
	strcpy(buffer + sizeof(unsigned int), name);
	WRITE_RTN_PACKET(ctl,fd0,buffer, act_len, msg);
	msg.command = SND_FRM;
	accum = msg.length - sizeof(unsigned int);
	while(1) {
		act_len = read(fds,buffer + sizeof(unsigned int), PKT_BUF_SZ - sizeof(unsigned int));
		msg.length = act_len + sizeof(unsigned int);
		if (act_len < (PKT_BUF_SZ - sizeof(unsigned int)))
             		BUILD_FILE_TAIL(buffer,msg);
		else	
             		BUILD_FRAME_HEADER(buffer,msg);
		accum +=act_len;
          	WRITE_RTN_PACKET(ctl,fd0,buffer, act_len, msg);
		if (act_len < (PKT_BUF_SZ - sizeof(unsigned int)))
			break;
	} 
	return accum;
}




main() {
	char * name;
	int fds, fdd;
	int i = 0, bytes_written, bytes_read;
	struct timeval tpstart;
	struct timeval tpend;
	long int To, T;
	char daddr[6] = {0x00,0x40,0x05,0xA1,0xEB,0x71};
//	char daddr[6] = {0x00,0x40,0x05,0x35,0xEC,0x9D};
 	if (open_rtn_device() < 0) return;
	gettimeofday(&tpstart,NULL);
	gettimeofday(&tpend,NULL);
	To = MILLION*(tpend.tv_sec - tpstart.tv_sec) + tpend.tv_usec -
tpstart.tv_usec;
	ioctl_rtn_device(CHG_DADDR,(unsigned long)daddr);
	printf("receiving\n");
	gettimeofday(&tpstart,NULL);
	bytes_read = rtn_rcv_file(&fds,&name);
	gettimeofday(&tpend,NULL);
	T = MILLION*(tpend.tv_sec - tpstart.tv_sec) + tpend.tv_usec - tpstart.tv_usec - To;
	if(ioctl_rtn_device(SEND_ACK,0) < 0) return;
	close(fds);
	printf("%d\t%ld\t%d\n",i,T, bytes_read);
	free(name);
	i++;     
}

