/*
 * An Example to calculate te round trip time of a single packet in Linux
 * using RT_TULIP
 * Source File
 * Written by Axel E. Bernal  	E-mail : abernal@nmt.edu
 * Released under the terms of the GNU Library General Public License
 */

#include "rtt_lib.h"
#include<unistd.h>



static char buffer[3072];

int fd0, fd1, ctl;

extern int volatile fd0,fd1,ctl;

int open_rtn_device()
{
	int n;
	int fd_flags;
	struct my_msg_struct msg;
	
	if ((fd0 = open("/dev/rtf1", O_WRONLY)) < 0) {
		fprintf(stderr, "Error opening /dev/rtf1\n");
		return -1;
	}

	if ((fd1 = open("/dev/rtf2", O_RDONLY)) < 0) {
		fprintf(stderr, "Error opening /dev/rtf2\n");
		return -1;
	}
	if((fd_flags = fcntl(fd1,F_GETFL,0)) == -1) {
		fprintf(stderr, "Could not get flags\n");
		return -1;
	}
	else {
		fd_flags |= O_NONBLOCK;
		if((fd_flags = fcntl(fd1,F_SETFL,fd_flags)) == -1) {
			fprintf(stderr, "Could not set flags\n");
			return -1;
		}
	}	

	if ((ctl = open("/dev/rtf3", O_WRONLY)) < 0) {
		fprintf(stderr, "Error opening /dev/rtf3\n");
		return -1;
	}
}

int close_rtn_device()
{
	close(fd0);
	close(fd1);
	close(ctl);
}


int chg_address(char *buf) {
	int act_len;
	struct my_msg_struct msg;
	// sending ack
	// changing the remote address
	msg.command = CHG_DADDR;
	msg.length = 6;
     	WRITE_RTN_PACKET(ctl,fd0, buf,act_len,msg);
	return 0;
}
 
static int send_ack () {	
	int accum;
	int act_len, i;
	struct my_msg_struct msg;
	// sending ack
	msg.command = SND_FRM;
	msg.length = sizeof(unsigned int);
     	BUILD_ACK(buffer, msg); 
     	WRITE_RTN_PACKET(ctl,fd0, buffer,act_len,msg);
	return 0;
}


int wait_ack() {
	static int debug = 0;
	int act_len;
	struct my_msg_struct msg;
	msg.command = RCV_FRM;
	msg.length = PKT_BUF_SZ;
	while(1) {
        	READ_RTN_PACKET(ctl,fd1,buffer,act_len,msg);
		if(debug++ == 10000000) { debug = 0; printf("Bad Ack\n"); }
		if(act_len == 0) continue;
          	if(IS_ACK(buffer)) 
			return 1;
		else
			return 0;
	}
}


inline int ioctl_rtn_device(int cmd, unsigned long arg) {
	switch(cmd) {
		case 	WAIT_ACK :              
             			return wait_ack();
		case 	SEND_ACK :
			       // sending ack
                         return send_ack();
		case 	CHG_DADDR :
             			return chg_address((char *)arg);
          default : return -1;
	}
}


static int rtn_rcv_signal() {
	int volatile act_len;
   	static int debug;
	struct my_msg_struct msg;
	msg.command = RCV_FRM;
	msg.length = PKT_BUF_SZ;
	while(1) {
        	READ_RTN_PACKET(ctl,fd1,buffer,act_len,msg);
		if(act_len > 0) break;
	}
	if(IS_BEGIN_FILE(buffer))  // create file in fs 
		;
	else {
		fprintf(stderr, "Not a header skbuff\n");
		return -1;
	}
	msg.command = RCV_FRM;
	msg.length = PKT_BUF_SZ;
	while(1) {
        	READ_RTN_PACKET(ctl,fd1,buffer,act_len,msg);
          if(act_len == 0)  continue; // no data
		if(debug++ == 500000000) { debug = 0; printf("S"); }
		if(IS_END_FILE(buffer))
		       	break;
	} 
	return act_len - sizeof(unsigned int);

}

static int rtn_snd_signal(int n) {
	int accum;
	int act_len, i;
	struct my_msg_struct msg;

/* This code is for the SENDER */
	/* Sending the name */
	msg.length = sizeof(unsigned int);
	msg.command = SND_FRM;
     	BUILD_FILE_HEADER(buffer,msg);
	WRITE_RTN_PACKET(ctl,fd0,buffer, act_len, msg);
	msg.command = SND_FRM;
	while(1) {
		msg.length = n + sizeof(unsigned int);
      		BUILD_FILE_TAIL(buffer,msg);
          	WRITE_RTN_PACKET(ctl,fd0,buffer, act_len, msg);
		if (act_len < (PKT_BUF_SZ - sizeof(unsigned int)))
			break;
	} 
	return act_len;
}




main() {
	char * name;
	int bytes_written, bytes_read, i;
	struct timeval tpstart, temp;
	struct timeval tpend;
	long int To, T;
//	char daddr[6] = {0x00,0x40,0x05,0x35,0xEC,0x9D};
	char daddr[6] = {0x00,0x40,0x05,0xA1,0xEB,0x71};
 	if (open_rtn_device() < 0) return;
	gettimeofday(&tpstart,NULL);
	gettimeofday(&tpend,NULL);
	To = MILLION*(tpend.tv_sec - tpstart.tv_sec) + tpend.tv_usec -
tpstart.tv_usec;
	ioctl_rtn_device(CHG_DADDR,(unsigned long)daddr);
	while(1) {
		printf("receiving....\n");
		bytes_read = rtn_rcv_signal();
		if(ioctl_rtn_device(SEND_ACK,0) < 0) return;
		gettimeofday(&tpend,NULL);
		sleep(1);
		printf(" and sending back%s\n", buffer);
		temp = tpstart;
		gettimeofday(&tpstart,NULL);
		bytes_written = rtn_snd_signal(bytes_read);
		T = MILLION*(tpend.tv_sec - temp.tv_sec) + tpend.tv_usec - temp.tv_usec - To;
		printf("Aproximate time :", buffer);
		printf("%ld...\n",T);
 		while(!ioctl_rtn_device(WAIT_ACK,0)) ;
		i++;
	}		     
}
