//
// Real Time Linux Example of Digital Oscilloscope
//
// Bill Crum
// New Mexico Tech
// tobor@nmt.edu
// This is only an example. No guarantees are expressed.
//


#include "scope.h"
#include <qdatetm.h>
#include <qpainter.h>

// The scope is check the real-time fifo every 50 ms.
// The scope is not in real-time, it displays data that was collected
// from real-time tasks.

ScopeTimer::ScopeTimer( QWidget *parent ) : QTimer( parent ), interval( 5 )
{
    initMetaObject();
    start( interval );
}

// Pause button to look a signal


void ScopeTimer::pause( bool stopIt )
{
    if ( stopIt )
        stop();
    else
        start( interval );
}


// The scope canvas

ScopeCanvas::ScopeCanvas ( QWidget *parent, const char *name)
  : QFrame( parent, name)
{
  char buffer[10000];
    initMetaObject();

    setMinimumSize( 100 + (2 * BORDER),100 + (2 * BORDER) );
    setMaximumSize( 1000 + (2 * BORDER), 1000 + (2 * BORDER) );

    setSizeIncrement( 1, 1);
  
    timer = new ScopeTimer(this);

    voltage_scale = 1;
    time_scale = 25;
    center_scale = 0;

    x = 1011;

    if ((ad0 = open("/dev/rtf1", O_RDONLY)) < 0) {
      fprintf(stderr, "Error opening /dev/rtf1\n");
      exit(1);
    }

    read(ad0, buffer, 5000);
    clear();
    connect( timer, SIGNAL(timeout()), SLOT(showSignal()) );


}



//
// Creates the Scope widget
//

Scope::Scope( QWidget *parent, const char *name )
    : QFrame( parent, name )
{
    initMetaObject();				// initialize meta object

 // Create a combo box

    QComboBox *combo = new QComboBox( this, "comboBox" );
    combo->insertItem( " 2 V /  Div" );
    combo->insertItem( " 5 V /  Div" );
    combo->insertItem( "10 V /  Div" );
    combo->setGeometry( 0, 0, 120, 25 );
    combo->setAutoResize( FALSE );
    combo->move(5,5 );
    combo->resize(120,25);

 // Create a combo box

    QComboBox *combo2 = new QComboBox( this, "comboBox2" );
    combo2->insertItem( " .1 S  / Div" );
    combo2->insertItem( " 50 mS / Div" );
    combo2->insertItem( " 20 mS / Div" );
    combo2->insertItem( " 10 mS / Div" );
    combo2->setGeometry( 0, 0, 120, 25 );
    combo2->setAutoResize( FALSE );
    combo2->move(140,5 );
    combo2->resize(120,25);

    pb = new QPushButton( "Pause", this );
    pb->setToggleButton( TRUE );
    pb->resize( 120, 25 );
    pb->move( 271,5 );



    scope = new ScopeCanvas(this);
    scope->move( 0,30 );




    connect( combo, SIGNAL(activated(int)), scope, SLOT(set_voltage_scale(int)) );
    connect( combo2, SIGNAL(activated(int)), scope, SLOT(set_time_scale(int)) );
    connect( pb, SIGNAL(toggled(bool)), scope, SLOT(hold()));

    QSize s;
    s = scope->minimumSize();
    setMinimumSize( 400,300);
    s = scope->maximumSize();
    setMaximumSize( 400,300 );
    s = scope->sizeIncrement();
    setSizeIncrement( s.width(), s.height() );

    scope->setFrameStyle( QFrame::Panel | QFrame::Sunken );
    scope->move(5,30 );
    scope->resize(391, 268 );
    resize( 400, 300 );


    showingColon = FALSE;
    setFrameStyle( QFrame::Box | QFrame::Raised );

    setLineWidth( 2 );				// set frame line width
    scope->show();
}


//
// Handles timer events for the scope widget.
//

void ScopeCanvas::timerEvent( )
{
  showSignal();
}


void ScopeCanvas::hold()
{
  if (holdDisplay)
    holdDisplay = 0;
  else
    holdDisplay = 1;
}




void ScopeCanvas::set_voltage_scale(int scale)
{
  switch (scale)
    {
    case 0:
      voltage_scale = 1;
      center_scale = 0;
      break;
    case 1:
      voltage_scale = 2.5;
      center_scale = 3072;
      break;
    case 2:
      voltage_scale = 5;
      center_scale = 8192;
      break;
    case 3:
      voltage_scale = 4;
      break;
    }
}

void ScopeCanvas::set_time_scale(int scale)
{
  switch(scale)
    {
    case 0:
      time_scale = 25;
      break;
    case 1:
      time_scale = 50;
      break;
    case 2:
      time_scale = 100;
      break;
    case 3:
      time_scale = 200;
      break;
    }
}


//
// Displays the data by drawing straight lines between points.
//

void ScopeCanvas::showSignal()
{
  int lloop;
  QPainter paint;

  paint.begin(this);
  paint.setWindow( 0,0,1000,(int)(4096 * voltage_scale) );//1300
  
  paint.setPen(blue);
  /*   while (read(ad0, inbyte, 2) > 0)*/

  for (lloop = 0; lloop < 10; lloop++)
    {
      if(read(ad0, inbyte, 2)< 0)
	printf("error\n");

      x  += time_scale;
      if (x < 1010)
	{
	  y = (unsigned int) inbyte[1];
	  y = y << 8;
	  y &= 0xff00;
	  y += (unsigned int) inbyte[0];
	  y &= 0xffff;
	  y = y >> 4;
	  //      printf("%x %d\n",y,y);
	  y = (y + center_scale);
	  paint.drawLine(oldx,oldy,x,y); 
	  oldx = x;
	  oldy = y;
	}
    }
      //  printf("%x %x %x %x \n", x, y, oldx, oldy);
  paint.end();
  if (x > 1010)
    {
      if (!holdDisplay)
        {
          x = 0;
          oldx = x;
	  clear();
        }
    }
}


void ScopeCanvas::clear()
{
      repaint();

      QPainter paint;
      paint.begin( this ); 
      paint.setWindow( 0,0,400,300 );


  paint.setPen(green);
  paint.drawLine(40,4,40,296);
  paint.drawLine(80,4,80,296);
  paint.drawLine(120,4,120,296);
  paint.drawLine(160,4,160,296);
  paint.drawLine(200,4,200,296);
  paint.drawLine(240,4,240,296);
  paint.drawLine(280,4,280,296);
  paint.drawLine(320,4,320,296);
  paint.drawLine(360,4,360,296);

  paint.drawLine(4,30,396,30);
  paint.drawLine(4,60,396,60);
  paint.drawLine(4,90,396,90);
  paint.drawLine(4,120,396,120);
  paint.drawLine(4,150,396,150);
  paint.drawLine(4,180,396,180);
  paint.drawLine(4,210,396,210);
  paint.drawLine(4,240,396,240);
  paint.drawLine(4,270,396,270);

  
  paint.end();

}






