/*Copyright (C) 1999 by Heinz Haeberle, heinz.haeberle@gmx.net

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.*/
#include <string.h>
extern "C" {
#include "softplc.h"
#include "iodefine.h"
#include "iodrv.h"
}

static sPLCTimer 	PLCTimer[MAXTIMER];
static sPLCCounter 	PLCCounter[MAXCOUNTER];

tDIOBool N(tDIOBool value)
{
	return (value != 0) ? 0 : 1;
}

tDIOBool GetTimer(sPLCTimer *timer)	//flanke schaltet ein, dann ausang true bis timeout
{
	if (timer->endtime == 0)
		return 0;
	if (/*(timer->startsignal)&&*/(timer->endtime >= ActualPLCTime))
		return 1;
//	timer->endtime = 0;
	return 0;
}

void ResetTimer(sPLCTimer *timer)
{
	timer->startsignal = 0;
	timer->endtime = 0;
}

// retriggerbares Monoflop
tDIOBool SetTimer(sPLCTimer *timer, tDIOBool value, tPLCTime ptime)
{
	// flankenerkennung
	if ((value) && (!timer->startsignal))
	{
		timer->startsignal = 1;
		timer->endtime = ActualPLCTime+ptime;
		return 1;
	}
	if (!value)
		timer->startsignal = 0;
	return GetTimer(timer);
}

// reset timer
void RT(sPLCTimer *timer, tDIOBool value)
{
	// nur ruecksetzen wenn value == 1
	if (value == 0)
		return;
	ResetTimer(timer);
}

// Zeitverhalten als Impuls
tDIOBool SI(sPLCTimer *timer, tDIOBool value, tPLCTime ptime)
{
	if (value)
		return SetTimer(timer, value, ptime);
	timer->startsignal = 0;
	return 0;		
}

//  Zeitverhalten verlngerter Impuls
tDIOBool SV(sPLCTimer *timer, tDIOBool value, tPLCTime ptime)
{
	return SetTimer(timer, value, ptime);
}

// Zeitverhalten als Einchaltverzgerung
tDIOBool SE(sPLCTimer *timer, tDIOBool value, tPLCTime ptime)
{
	if (SI(timer, value, ptime) == 0)
	{
		if ((value) && (timer->startsignal))
			return 1;
	}
	return 0;		
}

// Zeitverhalten als speichernde Einchaltverzgerung
tDIOBool SS(sPLCTimer *timer, tDIOBool value, tPLCTime ptime)
{
	if (SetTimer(timer, value, ptime) == 0)
	{
		if ((value) && (timer->startsignal))
			return 1;
	}
	return 0;		
}

// Zeitverhalten als Auschaltverzgerung
tDIOBool SA(sPLCTimer *timer, tDIOBool value, tPLCTime ptime)
{
	return SetTimer(timer, N(value), ptime) || value;
}
#if 0
// nicht retriggerbares Monoflop
tDIOBool SXX(sPLCTimer *timer, tDIOBool value, tPLCTime ptime)
{
//	tDIOBool actvalue;
	if (GetTimer(timer))
		return 1;
	return SetTimer(timer, value, ptime);
}
#endif

// jetzt kommen die Zaehler

tDIOBool GetCounter(sPLCCounter *counter)
{
	return (counter->counter != 0) ? 1 : 0;
}

tDIOBool SetCounter(sPLCCounter *counter, tDIOBool value, int count)
{
	// flankenerkennung zum setzen des wertes
	if ((value) && (!counter->startsignal))
	{
		counter->startsignal = 1;
		counter->counter = count;
		return 1;
	}
	if (!value)
		counter->startsignal = 0;
	return GetCounter(counter);
}

tDIOBool IncCounter(sPLCCounter *counter, tDIOBool value)
{
	// flankenerkennung zum vorwaertszaehler
	if ((value) && (!counter->incflag))
	{
		counter->incflag = 1;
		if (counter->counter < 999)
			counter->counter++;
	}
	if (!value)
		counter->incflag = 0;
	return GetCounter(counter);
}

tDIOBool DecCounter(sPLCCounter *counter, tDIOBool value)
{
	// flankenerkennung zum vorwaertszaehler
	if ((value) && (!counter->decflag))
	{
		counter->decflag = 1;
		if (counter->counter > 0)
			counter->counter--;
	}
	if (!value)
		counter->decflag = 0;
	return GetCounter(counter);
}
typedef unsigned char byte;

byte GetCounterValue(sPLCCounter *counter)
{
	return (byte)counter->counter;
}

#define ZV IncCounter
#define ZR DecCounter
#define Z  GetCounter
#define ZL SetCounter
#define ZX GetCounterValue

class PLCByte{
	sGlobalDigital *data;
public:
	void setdata(sGlobalDigital *_data){data = _data;};
	byte getvalue()
	{
		byte ret ;
		ret = data[0].value +
			  (data[1].value << 1) +
			  (data[2].value << 2) +
			  (data[3].value << 3) +
			  (data[4].value << 4) +
			  (data[5].value << 5) +
			  (data[6].value << 6) +
			  (data[7].value << 7);
		return ret;
	}
	
	tDIOBool operator>=(byte a){
		return (getvalue() >= a) ? 1 : 0;
	}
	tDIOBool operator>=(PLCByte &a){
		return (getvalue() >= a.getvalue()) ? 1 : 0;
	}
	tDIOBool operator<=(byte a){
		return (getvalue() <= a) ? 1 : 0;
	}
	tDIOBool operator<=(PLCByte &a){
		return (getvalue() <= a.getvalue()) ? 1 : 0;
	}
	tDIOBool operator<(byte a){
		return (getvalue() < a) ? 1 : 0;
	}
	tDIOBool operator<(PLCByte &a){
		return (getvalue() < a.getvalue()) ? 1 : 0;
	}
	tDIOBool operator>(byte a){
		return (getvalue() > a) ? 1 : 0;
	}
	tDIOBool operator>(PLCByte &a){
		return (getvalue() > a.getvalue()) ? 1 : 0;
	}
	tDIOBool operator==(byte a){
		return (getvalue() == a) ? 1 : 0;
	}
	tDIOBool operator==(PLCByte &a){
		return (getvalue() == a.getvalue()) ? 1 : 0;
	}
	tDIOBool operator!=(byte a){
		return (getvalue() != a) ? 1 : 0;
	}
	tDIOBool operator!=(PLCByte &a){
		return (getvalue() != a.getvalue()) ? 1 : 0;
	}
	byte operator-(PLCByte &a){
		return getvalue() - a.getvalue();
	}
	byte operator-(byte value){
		return getvalue() - value;
	}
	byte operator+(PLCByte &a){
		return a.getvalue() + getvalue();
	}
	byte operator+(byte value){
		return getvalue() + value;
	}
	void operator=(PLCByte value){
		*this = value.getvalue();
	}
	void operator=(byte value){
//for (i=0;i<8;i++)	// without loop to make it inline and fast
		data[0].value = (value & (1 << 0)) ? 1 : 0;
		data[1].value = (value & (1 << 1)) ? 1 : 0;
		data[2].value = (value & (1 << 2)) ? 1 : 0;
		data[3].value = (value & (1 << 3)) ? 1 : 0;
		data[4].value = (value & (1 << 4)) ? 1 : 0;
		data[5].value = (value & (1 << 5)) ? 1 : 0;
		data[6].value = (value & (1 << 6)) ? 1 : 0;
		data[7].value = (value & (1 << 7)) ? 1 : 0;
	}
};

//PLCByte AB0,AB1,AB2,AB3,AB4,AB5,EB0,EB1,EB2,EB3,EB4,EB5,MB0,MB1,MB2,MB3,MB4,MB5,MB6,MB7,MB8,MB9;
// the function InitPLC1 is defined there
#define IODEFINE_EXECUTE
#include "iodefine.h"
#undef IODEFINE_EXECUTE

extern "C"{
void InitPLC()
{
	InitPLC1();
}
// hier wird der Benutzerbereich eingeklinkt!
#include "plc.cpp"
} /* extern "C" */