/*Copyright (C) 1999 by Heinz Haeberle, heinz.haeberle@gmx.net

This is a RTLinux-module for the generic IO, used by the softplc

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.*/

#include <linux/module.h>
#include <linux/kernel.h>
#include <asm/io.h>
#include <rtl.h>
#include <rtl_fifo.h>
#include <time.h>
#include <rtl_sched.h>
#include <rtl_sync.h>
#include <rtl_time.h>
#include <rtlinux/mbuff.h>
#include "rtl_debug.h"
#include "iodrv.h"

/*
sGlobalDI IODataDI[MAXDICHANNEL];
sGlobalDO IODataDO[MAXDICHANNEL];
sGlobalM IODataM[MAXMERKER];
*/

sGlobalDigital *IODataDI;
sGlobalDigital *IODataDO;
sGlobalDigital *IODataM;

static IOFunction DICBArray[MAXDICHANNEL];
static IOFunction DOCBArray[MAXDOCHANNEL];	
static int DIfirstFreeChannel=0;
static int DOfirstFreeChannel=0;

int IOAddDIFunction(int channels, IOFunction fn)
{
	int i;
	breakpoint();
	if ((DIfirstFreeChannel + channels) > MAXDICHANNEL)
		return -1;
	for (i=DIfirstFreeChannel;i<DIfirstFreeChannel+channels;i++)
		DICBArray[i] = fn;
	DIfirstFreeChannel +=channels;	
	return 0;
}

int IOAddDOFunction(int channels, IOFunction fn)
{
	int i;
	if ((DOfirstFreeChannel + channels) > MAXDOCHANNEL)
		return -1;
	for (i=DOfirstFreeChannel;i<DOfirstFreeChannel+channels;i++)
		DOCBArray[i] = fn;
	DOfirstFreeChannel +=channels;	
	return 0;
}

int IORemDIFunction(IOFunction fn)
{
	int i;
	for (i=0;i<MAXDICHANNEL;i++)
		if(DICBArray[i] == fn)
			DICBArray[i] = NULL;
	return 0;
}

int IORemDOFunction(IOFunction fn)
{
	int i;
	for (i=0;i<MAXDICHANNEL;i++)
		if(DOCBArray[i] == fn)
			DOCBArray[i] = NULL;
	return 0;
}

int IOSyncDI(int fromchannel, int channels)
{
	int i;
	uIOParam iop;
	IOFunction fn=NULL;
	if ((channels >= MAXDICHANNEL) || (channels < 0))
		channels = MAXDICHANNEL - fromchannel;
	for (i=0;i<channels;i++,fromchannel++)
	{
		if ((DICBArray[fromchannel] != fn) && (DICBArray[fromchannel] != NULL))
		{
			iop.diptr = &IODataDI[fromchannel];
			fn = DICBArray[fromchannel];
			(*DICBArray[fromchannel])(IOCMD_SyncDI, iop);
		}
	}
	return 0;
}

int IOSyncDO(int fromchannel, int channels)
{
	int i;
	uIOParam iop;
	IOFunction fn=NULL;
	if ((channels >= MAXDOCHANNEL) || (channels < 0))
		channels = MAXDOCHANNEL - fromchannel;
	for (i=0;i<channels;i++,fromchannel++)
	{
		if ((DOCBArray[fromchannel] != fn) && (DOCBArray[fromchannel] != NULL))
		{
			iop.doptr = &IODataDO[fromchannel];
			fn = DOCBArray[fromchannel];
			(*DOCBArray[fromchannel])(IOCMD_SyncDO, iop);
		}
	}
	return 0;
}

int init_module(void)
{
	IODataDI = (sGlobalDigital*)mbuff_alloc(IODRV_SHM_NAME,IOMBUFFSIZE);
	if (!IODataDI)
	{
		rtl_printf("can't allocate shared mem!\n");
		return -1;
	}
	IODataDO = &IODataDI[MAXDICHANNEL];
	IODataM  = &IODataDO[MAXDOCHANNEL];
	memset(IODataDI, 0, sizeof(IODataDI));
	memset(IODataDO, 0, sizeof(IODataDO));
	memset(DICBArray, 0, sizeof(DICBArray));
	memset(DOCBArray, 0, sizeof(DOCBArray));
	DIfirstFreeChannel=0;
	DOfirstFreeChannel=0;
	return 0;
}

void cleanup_module(void)
{
	if (IODataDI)
		mbuff_free("IOMbuff", IODataDI);
}
