/* real time linux task (dummy task) for RT-Linux version 0.5  */
/* simulates a 12 bit, 8 channel AD-board                      */
/* in a real world application, the functions kanal0 to kanal7 */
/* must be replaced by functions that read the samples from    */
/* the AD-converter card.                                      */

#define MODULE

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/errno.h>
#include <linux/rt_sched.h>
#include <linux/rtf.h>
#include <asm/rt_time.h>
#include "control.h"

RT_TASK mytask;
int     sr;
char    channels;

/* generates periodic output values         */
/* output range is 12 bit (0 to +4096)      */

int kanal0() {
  static float x=1.0;
  static float dx=0.001;
         int vout;
	 
  vout = x*2048.0+2048.0;
  x-=dx;
  if (x>=1.0)    {x=1; dx=0.001;};
  if (x<=-1.0)   {x=-1; dx=-0.001;};

  return(vout);
}

int kanal1() {
  static float x=1.0;
  static float dx=0.001;
         int vout;
	 
  if (dx<=0) vout=-1*x*x*2048.0+4096.0;
  if (dx>0)  vout=   x*x*2048.0;
  x-=dx;
  if (x>=1.0)    {x=1; dx=0.001;};
  if (x<=-1.0)   {x=-1; dx=-0.001;};

  return(vout);
}

int kanal2() {
  static float x=1.0;
  static float dx=0.001;
         int vout;
	 
  vout = x*x*x*2048.0+2048;
  x-=dx;
  if (x>=1.0)    {x=1; dx=0.001;};
  if (x<=-1.0)   {x=-1; dx=-0.001;};

  return(vout);
}

int kanal3() {
  static float x=1.0;
  static float dx=0.003;
         int vout;
	 
  vout = x*2048.0+2048;
  x-=dx;
  if (x>=1.0)    {x=1; dx=0.003;};
  if (x<=-1.0)   {x=-1; dx=-0.003;};

  return(vout);
}

int kanal4() {
  static float x=1.0;
  static float dx=0.003;
         int vout;
	 
  if (dx<=0) vout=-1*x*x*2048.0+4096.0;
  if (dx>0)  vout=   x*x*2048.0;
  x-=dx;
  if (x>=1.0)    {x=1; dx=0.003;};
  if (x<=-1.0)   {x=-1; dx=-0.003;};

  return(vout);
}

int kanal5() {
  static float x=1.0;
  static float dx=0.003;
         int vout;
	 
  vout = x*x*x*2048.0+2048;
  x-=dx;
  if (x>=1.0)    {x=1; dx=0.003;};
  if (x<=-1.0)   {x=-1; dx=-0.003;};

  return(vout);
}

int kanal6() {
  static float x=1.0;
  static float dx=0.005;
         int vout;
	 
  vout = x*2048.0+2048;
  x-=dx;
  if (x>=1.0)    {x=1; dx=0.005;};
  if (x<=-1.0)   {x=-1; dx=-0.005;};

  return(vout);
}

int kanal7() {
  static float x=1.0;
  static float dx=0.005;
         int vout;
	 
  if (dx<=0) vout=-1*x*x*2048.0+4096.0;
  if (dx>0)  vout=   x*x*2048.0;
  x-=dx;
  if (x>=1.0)    {x=1; dx=0.005;};
  if (x<=-1.0)   {x=-1; dx=-0.005;};

  return(vout);
}

/* .................................. */
/* function to write samples to fifos */
/* .................................. */
void fun(int t) {
  int data, rc;

  int kanal0();
  int kanal1();
  int kanal2();
  int kanal3();
  int kanal4();
  int kanal5();
  int kanal6();
  int kanal7();

  while(1) {
    /* FIFO's 0-7 transfer the samples from channel 1 to 8                       */
    /* FIFO 9 is an indicator whether a FIFO is full when trying to write to it  */
    if (channels==0) rt_task_wait();
    if ((channels & 1)==1) {
      data=kanal0();
      if (rtf_put(0, (char *) &data, 2) < 0) {
	rc=0;
	rtf_put(9, (char *) &rc, sizeof(rc));
      }
      rt_task_wait();
    }
    if ((channels & 2)==2) {
      data=kanal1();
      if (rtf_put(1, (char *) &data, 2) < 0) {
	rc=1;
	rtf_put(9, (char *) &rc, sizeof(rc));
      }
      rt_task_wait();
    }
    if ((channels & 4)==4) {
      data=kanal2();
      if (rtf_put(2, (char *) &data, 2) < 0) {
	rc=2;
	rtf_put(9, (char *) &rc, sizeof(rc));
      }
      rt_task_wait();
    }
    if ((channels & 8)==8) {
      data=kanal3();
      if (rtf_put(3, (char *) &data, 2) < 0) {
	rc=3;
	rtf_put(9, (char *) &rc, sizeof(rc));
      }
      rt_task_wait();
    }
    if ((channels & 16)==16) {
      data=kanal4();
      if (rtf_put(4, (char *) &data, 2) < 0) {
	rc=4;
	rtf_put(9, (char *) &rc, sizeof(rc));
      }
      rt_task_wait();
    }
    if ((channels & 32)==32) {
      data=kanal5();
      if (rtf_put(5, (char *) &data, 2) < 0) {
	rc=5;
	rtf_put(9, (char *) &rc, sizeof(rc));
      }
      rt_task_wait();
    }
    if ((channels & 64)==64) {
      data=kanal6();
      if (rtf_put(6, (char *) &data, 2) < 0) {
	rc=6;
	rtf_put(9, (char *) &rc, sizeof(rc));
      }
      rt_task_wait();
    }
    if ((channels & 128)==128) {
      data=kanal7();
      if (rtf_put(7, (char *) &data, 2) < 0) {
	rc=7;
	rtf_put(9, (char *) &rc, sizeof(rc));
      }
      rt_task_wait();
    }
  }
}

int my_handler(unsigned int fifo) {
  struct my_msg_struct msg;
  int err;
  RTIME now;

  /* FIFO 8 transfers commands to control the driver */
  while ((err=rtf_get(8, &msg, sizeof(msg))) == sizeof(msg)) {
    switch(msg.command) {
    case START_TASK:
      now=rt_get_time();
      rt_task_make_periodic(&mytask, now, (RTIME)((unsigned long)RT_TICKS_PER_SEC/(unsigned long)sr));
      break;
    case STOP_TASK:
      rt_task_suspend(&mytask);
      break;
    case SET_CH:
      channels=(unsigned char)msg.value;
      break;
    case SET_POL:
      break;
    case SET_SR:
      sr=msg.value;
      break;
    default:
      return -EINVAL;
    }
  }
  if (err!=0) {
    return -EINVAL;
  }
  return 0;
}

int init_module(void)
{
  sr=1;
  channels=0x00;
  rtf_create(0,10000);
  rtf_create(1,10000);
  rtf_create(2,10000);
  rtf_create(3,10000);
  rtf_create(4,10000);
  rtf_create(5,10000);
  rtf_create(6,10000);
  rtf_create(7,10000);
  rtf_create(8,100);  /* input control channel   */
  rtf_create(9,100);  /* fifo overflow indicator */
  rt_task_init(&mytask, fun, 0, 4000, 4);
  rtf_create_handler(8, &my_handler);
  return 0;
}

void cleanup_module(void)
{
  rtf_destroy(0);
  rtf_destroy(1);
  rtf_destroy(2);
  rtf_destroy(3);
  rtf_destroy(4);
  rtf_destroy(5);
  rtf_destroy(6);
  rtf_destroy(7);
  rtf_destroy(8);
  rtf_destroy(9);
  rt_task_delete(&mytask);
}
