/* real time linux task for reading samples from the MAX186 chip       */
/* MAX186 is a 12 bit, 8 channel multiplexed, serial AD-converter chip */
/* do not use with the MAX192 chip if pin 9 is on ground               */
/* MAX192 does not support single-ended bipolar mode !!!               */
/* Note: if pin 9 of the chip is on V-- the MAX192 does support        */
/*       single ended bipolar mode (other than stated in the           */
/*       official reference manual from MAXIM)                         */

#define MODULE

/* #define PORT  0x3FC for COM1 */
/* #define PORT  0x2FC for COM2 */
/* #define PORT  0x3EC for COM3 */
/* #define PORT  0x2EC for COM4 */
#define PORT    0x2FC  /* COM2 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/errno.h>
#include <linux/rt_sched.h>
#include <linux/rtf.h>
#include <asm/rt_time.h>
#include <asm/io.h> 
#include "control.h"

RT_TASK mytask;
int     sr;
char    channels;
short   bipol[8];

/* read channels from AD-card */
int kanal0(int port, short bipol) {
  int vout=0;
  int inp;
  int  i, addit;

  /* get bits */
  addit=2048;
  for (i=0; i<7; i++) {
    outb((0xFE & inb(port)), port);                /* pull SCLK low             */
    outb((0xFD & inb(port)), port);                /* pull DIN  low, do nothing */
    outb((0x01 | inb(port)), port);                /* pull SCLK high            */
    if (((inb(port+2) & 0x10))==0x10) vout+=addit; /* get bits 11-5             */
    addit=addit/2;
  }
  outb((0xFE & inb(port)), port);                /* pull SCLK low             */
  outb((0x02 | inb(port)), port);                /* set start bit in DIN to 1 */
  outb((0x01 | inb(port)), port);                /* pull SCLK high            */
  if (((inb(port+2) & 0x10))==0x10) vout+=addit; /* get bit 4                 */
  addit=addit/2;
  for (i=0; i<3; i++) {
    outb((0xFE & inb(port)), port); /* pull SCLK low                             */
    outb((0xFD & inb(port)), port); /* set DIN low, select channel bits 2-4 to 0 */
    outb((0x01 | inb(port)), port); /* pull SCLK high                            */
    if (((inb(port+2) & 0x10))==0x10) vout+=addit; /* get bits 3-1               */
    addit=addit/2;
  }
  /* uni- or bipolar */
  outb((0xFE & inb(port)), port);                /* pull SCLK low          */
  if (bipol) outb((0xFD & inb(port)), port);     /* set DIN low, bipolar   */
  else       outb((0x02 | inb(port)), port);     /* set DIN high, unipolar */
  outb((0x01 | inb(port)), port);                /* pull SCLK high         */
  if (((inb(port+2) & 0x10))==0x10) vout+=addit; /* get bit 0              */
  addit=addit/2;

  inp=inb(port); outb(0xFE & inp, port); /* pull SCLK low              */
  inp=inb(port); outb(0x02 | inp, port); /* set DIN high, single ended */
  inp=inb(port); outb(0x01 | inp, port); /* pull SCLK high             */

  inp=inb(port); outb(0xFE & inp, port); /* pull SCLK low                           */
  inp=inb(port); outb(0x02 | inp, port); /* set DIN high, PD1=1 internal clock mode */
  inp=inb(port); outb(0x01 | inp, port); /* pull SCLK high                          */

  inp=inb(port); outb(0xFE & inp, port); /* pull SCLK low                          */
  inp=inb(port); outb(0xFD & inp, port); /* set DIN low, PD0=0 internal clock mode */
  inp=inb(port); outb(0x01 | inp, port); /* pull SCLK high                         */

  inp=inb(port); outb(0xFE & inp, port); /* pull SCLK low                               */
  inp=inb(port); outb(0xFD & inp, port); /* set DIN low, empty bit 16 of shift register */ 
  inp=inb(port); outb(0x01 | inp, port); /* pull SCLK high                              */

  return(vout);
}

int kanal1(int port, short bipol) {
  int vout=0;
  int inp;
  int  i, addit;

  /* get bits */
  addit=2048;
  for (i=0; i<7; i++) {
    outb((0xFE & inb(port)), port);
    outb((0xFD & inb(port)), port);
    outb((0x01 | inb(port)), port);
    if (((inb(port+2) & 0x10))==0x10) vout+=addit;
    addit=addit/2;
  }
  for (i=0; i<2; i++) {
    outb((0xFE & inb(port)), port);
    outb((0x02 | inb(port)), port);
    outb((0x01 | inb(port)), port);
    if (((inb(port+2) & 0x10))==0x10) vout+=addit;
    addit=addit/2;
  }
  for (i=0; i<2; i++) {
    outb((0xFE & inb(port)), port);
    outb((0xFD & inb(port)), port);
    outb((0x01 | inb(port)), port);
    if (((inb(port+2) & 0x10))==0x10) vout+=addit;
    addit=addit/2;
  }
  /* uni- or bipolar */
  outb((0xFE & inb(port)), port);
  if (bipol) outb((0xFD & inb(port)), port);
  else       outb((0x02 | inb(port)), port);
  outb((0x01 | inb(port)), port);
  if (((inb(port+2) & 0x10))==0x10) vout+=addit;
  addit=addit/2;

  inp=inb(port); outb(0xFE & inp, port);
  inp=inb(port); outb(0x02 | inp, port);
  inp=inb(port); outb(0x01 | inp, port);

  inp=inb(port); outb(0xFE & inp, port);
  inp=inb(port); outb(0x02 | inp, port);
  inp=inb(port); outb(0x01 | inp, port);

  inp=inb(port); outb(0xFE & inp, port);
  inp=inb(port); outb(0xFD & inp, port);
  inp=inb(port); outb(0x01 | inp, port);

  inp=inb(port); outb(0xFE & inp, port);
  inp=inb(port); outb(0xFD & inp, port);
  inp=inb(port); outb(0x01 | inp, port);

  return(vout);
}

int kanal2(int port, short bipol) {
  int vout=0;
  int inp;
  int  i, addit;

  /* get bits */
  addit=2048;
  for (i=0; i<7; i++) {
    outb((0xFE & inb(port)), port);
    outb((0xFD & inb(port)), port);
    outb((0x01 | inb(port)), port);
    if (((inb(port+2) & 0x10))==0x10) vout+=addit;
    addit=addit/2;
  }

  outb((0xFE & inb(port)), port);
  outb((0x02 | inb(port)), port);
  outb((0x01 | inb(port)), port);
  if (((inb(port+2) & 0x10))==0x10) vout+=addit;
  addit=addit/2;
  
  outb((0xFE & inb(port)), port);
  outb((0xFD & inb(port)), port);
  outb((0x01 | inb(port)), port);
  if (((inb(port+2) & 0x10))==0x10) vout+=addit;
  addit=addit/2;
  
  outb((0xFE & inb(port)), port);
  outb((0xFD & inb(port)), port);
  outb((0x01 | inb(port)), port);
  if (((inb(port+2) & 0x10))==0x10) vout+=addit;
  addit=addit/2;
  
  outb((0xFE & inb(port)), port);
  outb((0x02 | inb(port)), port);
  outb((0x01 | inb(port)), port);
  if (((inb(port+2) & 0x10))==0x10) vout+=addit;
  addit=addit/2;

  /* uni- or bipolar */
  outb((0xFE & inb(port)), port);
  if (bipol) outb((0xFD & inb(port)), port);
  else       outb((0x02 | inb(port)), port);
  outb((0x01 | inb(port)), port);
  if (((inb(port+2) & 0x10))==0x10) vout+=addit;
  addit=addit/2;
  
  inp=inb(port); outb(0xFE & inp, port);
  inp=inb(port); outb(0x02 | inp, port);
  inp=inb(port); outb(0x01 | inp, port);

  inp=inb(port); outb(0xFE & inp, port);
  inp=inb(port); outb(0x02 | inp, port);
  inp=inb(port); outb(0x01 | inp, port);

  inp=inb(port); outb(0xFE & inp, port);
  inp=inb(port); outb(0xFD & inp, port);
  inp=inb(port); outb(0x01 | inp, port);

  inp=inb(port); outb(0xFE & inp, port);
  inp=inb(port); outb(0xFD & inp, port);
  inp=inb(port); outb(0x01 | inp, port);

  return(vout);
}

int kanal3(int port, short bipol) {
  int vout=0;
  int inp;
  int  i, addit;

  /* get bits */
  addit=2048;
  for (i=0; i<7; i++) {
    outb((0xFE & inb(port)), port);
    outb((0xFD & inb(port)), port);
    outb((0x01 | inb(port)), port);
    if (((inb(port+2) & 0x10))==0x10) vout+=addit;
    addit=addit/2;
  }

  outb((0xFE & inb(port)), port);
  outb((0x02 | inb(port)), port);
  outb((0x01 | inb(port)), port);
  if (((inb(port+2) & 0x10))==0x10) vout+=addit;
  addit=addit/2;
  
  outb((0xFE & inb(port)), port);
  outb((0x02 | inb(port)), port);
  outb((0x01 | inb(port)), port);
  if (((inb(port+2) & 0x10))==0x10) vout+=addit;
  addit=addit/2;
  
  outb((0xFE & inb(port)), port);
  outb((0xFD & inb(port)), port);
  outb((0x01 | inb(port)), port);
  if (((inb(port+2) & 0x10))==0x10) vout+=addit;
  addit=addit/2;
  
  outb((0xFE & inb(port)), port);
  outb((0x02 | inb(port)), port);
  outb((0x01 | inb(port)), port);
  if (((inb(port+2) & 0x10))==0x10) vout+=addit;
  addit=addit/2;
  
  /* uni- or bipolar */
  outb((0xFE & inb(port)), port);
  if (bipol) outb((0xFD & inb(port)), port);
  else       outb((0x02 | inb(port)), port);
  outb((0x01 | inb(port)), port);
  if (((inb(port+2) & 0x10))==0x10) vout+=addit;
  addit=addit/2;
  
  inp=inb(port); outb(0xFE & inp, port);
  inp=inb(port); outb(0x02 | inp, port);
  inp=inb(port); outb(0x01 | inp, port);

  inp=inb(port); outb(0xFE & inp, port);
  inp=inb(port); outb(0x02 | inp, port);
  inp=inb(port); outb(0x01 | inp, port);

  inp=inb(port); outb(0xFE & inp, port);
  inp=inb(port); outb(0xFD & inp, port);
  inp=inb(port); outb(0x01 | inp, port);

  inp=inb(port); outb(0xFE & inp, port);
  inp=inb(port); outb(0xFD & inp, port);
  inp=inb(port); outb(0x01 | inp, port);

  return(vout);
}

int kanal4(int port, short bipol) {
  int vout=0;
  int inp;
  int  i, addit;

  /* get bits */
  addit=2048;
  for (i=0; i<7; i++) {
    outb((0xFE & inb(port)), port);
    outb((0xFD & inb(port)), port);
    outb((0x01 | inb(port)), port);
    if (((inb(port+2) & 0x10))==0x10) vout+=addit;
    addit=addit/2;
  }

  outb((0xFE & inb(port)), port);
  outb((0x02 | inb(port)), port);
  outb((0x01 | inb(port)), port);
  if (((inb(port+2) & 0x10))==0x10) vout+=addit;
  addit=addit/2;
  
  outb((0xFE & inb(port)), port);
  outb((0xFD & inb(port)), port);
  outb((0x01 | inb(port)), port);
  if (((inb(port+2) & 0x10))==0x10) vout+=addit;
  addit=addit/2;
  
  outb((0xFE & inb(port)), port);
  outb((0x02 | inb(port)), port);
  outb((0x01 | inb(port)), port);
  if (((inb(port+2) & 0x10))==0x10) vout+=addit;
  addit=addit/2;
  
  outb((0xFE & inb(port)), port);
  outb((0xFD & inb(port)), port);
  outb((0x01 | inb(port)), port);
  if (((inb(port+2) & 0x10))==0x10) vout+=addit;
  addit=addit/2;
  
  /* uni- or bipolar */
  outb((0xFE & inb(port)), port);
  if (bipol) outb((0xFD & inb(port)), port);
  else       outb((0x02 | inb(port)), port);
  outb((0x01 | inb(port)), port);
  if (((inb(port+2) & 0x10))==0x10) vout+=addit;
  addit=addit/2;
  
  inp=inb(port); outb(0xFE & inp, port);
  inp=inb(port); outb(0x02 | inp, port);
  inp=inb(port); outb(0x01 | inp, port);

  inp=inb(port); outb(0xFE & inp, port);
  inp=inb(port); outb(0x02 | inp, port);
  inp=inb(port); outb(0x01 | inp, port);

  inp=inb(port); outb(0xFE & inp, port);
  inp=inb(port); outb(0xFD & inp, port);
  inp=inb(port); outb(0x01 | inp, port);

  inp=inb(port); outb(0xFE & inp, port);
  inp=inb(port); outb(0xFD & inp, port);
  inp=inb(port); outb(0x01 | inp, port);

  return(vout);
}

int kanal5(int port, short bipol) {
  int vout=0;
  int inp;
  int  i, addit;

  /* get bits */
  addit=2048;
  for (i=0; i<7; i++) {
    outb((0xFE & inb(port)), port);
    outb((0xFD & inb(port)), port);
    outb((0x01 | inb(port)), port);
    if (((inb(port+2) & 0x10))==0x10) vout+=addit;
    addit=addit/2;
  }

  for (i=0; i<3; i++) {
    outb((0xFE & inb(port)), port);
    outb((0x02 | inb(port)), port);
    outb((0x01 | inb(port)), port);
    if (((inb(port+2) & 0x10))==0x10) vout+=addit;
    addit=addit/2;
  }

  outb((0xFE & inb(port)), port);
  outb((0xFD & inb(port)), port);
  outb((0x01 | inb(port)), port);
  if (((inb(port+2) & 0x10))==0x10) vout+=addit;
  addit=addit/2;

  /* uni- or bipolar */
  outb((0xFE & inb(port)), port);
  if (bipol) outb((0xFD & inb(port)), port);
  else       outb((0x02 | inb(port)), port);
  outb((0x01 | inb(port)), port);
  if (((inb(port+2) & 0x10))==0x10) vout+=addit;
  addit=addit/2;

  inp=inb(port); outb(0xFE & inp, port);
  inp=inb(port); outb(0x02 | inp, port);
  inp=inb(port); outb(0x01 | inp, port);

  inp=inb(port); outb(0xFE & inp, port);
  inp=inb(port); outb(0x02 | inp, port);
  inp=inb(port); outb(0x01 | inp, port);

  inp=inb(port); outb(0xFE & inp, port);
  inp=inb(port); outb(0xFD & inp, port);
  inp=inb(port); outb(0x01 | inp, port);

  inp=inb(port); outb(0xFE & inp, port);
  inp=inb(port); outb(0xFD & inp, port);
  inp=inb(port); outb(0x01 | inp, port);

  return(vout);
}

int kanal6(int port, short bipol) {
  int vout=0;
  int inp;
  int  i, addit;

  /* get bits */
  addit=2048;
  for (i=0; i<7; i++) {
    outb((0xFE & inb(port)), port);
    outb((0xFD & inb(port)), port);
    outb((0x01 | inb(port)), port);
    if (((inb(port+2) & 0x10))==0x10) vout+=addit;
    addit=addit/2;
  }

  outb((0xFE & inb(port)), port);
  outb((0x02 | inb(port)), port);
  outb((0x01 | inb(port)), port);
  if (((inb(port+2) & 0x10))==0x10) vout+=addit;
  addit=addit/2;

  outb((0xFE & inb(port)), port);
  outb((0xFD & inb(port)), port);
  outb((0x01 | inb(port)), port);
  if (((inb(port+2) & 0x10))==0x10) vout+=addit;
  addit=addit/2;

  for (i=0; i<2; i++) {
    outb((0xFE & inb(port)), port);
    outb((0x02 | inb(port)), port);
    outb((0x01 | inb(port)), port);
    if (((inb(port+2) & 0x10))==0x10) vout+=addit;
    addit=addit/2;
  }

  /* uni- or bipolar */
  outb((0xFE & inb(port)), port);
  if (bipol) outb((0xFD & inb(port)), port);
  else       outb((0x02 | inb(port)), port);
  outb((0x01 | inb(port)), port);
  if (((inb(port+2) & 0x10))==0x10) vout+=addit;
  addit=addit/2;

  inp=inb(port); outb(0xFE & inp, port);
  inp=inb(port); outb(0x02 | inp, port);
  inp=inb(port); outb(0x01 | inp, port);

  inp=inb(port); outb(0xFE & inp, port);
  inp=inb(port); outb(0x02 | inp, port);
  inp=inb(port); outb(0x01 | inp, port);

  inp=inb(port); outb(0xFE & inp, port);
  inp=inb(port); outb(0xFD & inp, port);
  inp=inb(port); outb(0x01 | inp, port);

  inp=inb(port); outb(0xFE & inp, port);
  inp=inb(port); outb(0xFD & inp, port);
  inp=inb(port); outb(0x01 | inp, port);

  return(vout);
}

int kanal7(int port, short bipol) {
  int vout=0;
  int inp;
  int  i, addit;

  /* get bits */
  addit=2048;
  for (i=0; i<7; i++) {
    outb((0xFE & inb(port)), port);
    outb((0xFD & inb(port)), port);
    outb((0x01 | inb(port)), port);
    if (((inb(port+2) & 0x10))==0x10) vout+=addit;
    addit=addit/2;
  }

  for (i=0; i<4; i++) {
    outb((0xFE & inb(port)), port);
    outb((0x02 | inb(port)), port);
    outb((0x01 | inb(port)), port);
    if (((inb(port+2) & 0x10))==0x10) vout+=addit;
    addit=addit/2;
  }
  
  /* uni- or bipolar */
  outb((0xFE & inb(port)), port);
  if (bipol) outb((0xFD & inb(port)), port);
  else       outb((0x02 | inb(port)), port);
  outb((0x01 | inb(port)), port);
  if (((inb(port+2) & 0x10))==0x10) vout+=addit;
  addit=addit/2;
  
  inp=inb(port); outb(0xFE & inp, port);
  inp=inb(port); outb(0x02 | inp, port);
  inp=inb(port); outb(0x01 | inp, port);

  inp=inb(port); outb(0xFE & inp, port);
  inp=inb(port); outb(0x02 | inp, port);
  inp=inb(port); outb(0x01 | inp, port);

  inp=inb(port); outb(0xFE & inp, port);
  inp=inb(port); outb(0xFD & inp, port);
  inp=inb(port); outb(0x01 | inp, port);

  inp=inb(port); outb(0xFE & inp, port);
  inp=inb(port); outb(0xFD & inp, port);
  inp=inb(port); outb(0x01 | inp, port);

  return(vout);
}

/* .................................. */
/* function to write samples to fifos */
/* .................................. */
void fun(int t) {
  int   data, prech;

  int kanal0(int port, short bipol);
  int kanal1(int port, short bipol);
  int kanal2(int port, short bipol);
  int kanal3(int port, short bipol);
  int kanal4(int port, short bipol);
  int kanal5(int port, short bipol);
  int kanal6(int port, short bipol);
  int kanal7(int port, short bipol);

  prech=-1;
  while(1) {
    /* FIFO's 0-7 transfer the samples from channel 1 to 8                       */
    /* FIFO 9 is an indicator whether a FIFO is full when trying to write to it */

    if (channels==0) rt_task_wait();
    if ((channels & 1)==1) {
      data=kanal0(PORT, bipol[0]);
      if (prech>=0) {
	if (bipol[prech] && data>2047) data-=4095;
	if (bipol[prech]) data+=2047;
	if (rtf_put(prech, (char *) &data, 2) < 0)
	  rtf_put(9, (char *) &prech, sizeof(prech));
      }
      prech=0;
      rt_task_wait();
    }
    if ((channels & 2)==2) {
      data=kanal1(PORT, bipol[1]);
      if (prech>=0) {
	if (bipol[prech] && data>2047) data-=4095;
	if (bipol[prech]) data+=2047;
	if (rtf_put(prech, (char *) &data, 2) < 0)
	  rtf_put(9, (char *) &prech, sizeof(prech));
      }
      prech=1;
      rt_task_wait();
    }
    if ((channels & 4)==4) {
      data=kanal2(PORT, bipol[2]);
      if (prech>=0) {
	if (bipol[prech] && data>2047) data-=4095;
	if (bipol[prech]) data+=2047;
	if (rtf_put(prech, (char *) &data, 2) < 0)
	  rtf_put(9, (char *) &prech, sizeof(prech));
      }
      prech=2;
      rt_task_wait();
    }
    if ((channels & 8)==8) {
      data=kanal3(PORT, bipol[3]);
      if (prech>=0) {
	if (bipol[prech] && data>2047) data-=4095;
	if (bipol[prech]) data+=2047;
	if (rtf_put(prech, (char *) &data, 2) < 0)
	  rtf_put(9, (char *) &prech, sizeof(prech));
      }
      prech=3;
      rt_task_wait();
    }
    if ((channels & 16)==16) {
      data=kanal4(PORT, bipol[4]);
      if (prech>=0) {
	if (bipol[prech] && data>2047) data-=4095;
	if (bipol[prech]) data+=2047;
	if (rtf_put(prech, (char *) &data, 2) < 0)
	  rtf_put(9, (char *) &prech, sizeof(prech));
      }
      prech=4;
      rt_task_wait();
    }
    if ((channels & 32)==32) {
      data=kanal5(PORT, bipol[5]);
      if (prech>=0) {
	if (bipol[prech] && data>2047) data-=4095;
	if (bipol[prech]) data+=2047;
	if (rtf_put(prech, (char *) &data, 2) < 0)
	  rtf_put(9, (char *) &prech, sizeof(prech));
      }
      prech=5;
      rt_task_wait();
    }
    if ((channels & 64)==64) {
      data=kanal6(PORT, bipol[6]);
      if (prech>=0) {
	if (bipol[prech] && data>2047) data-=4095;
	if (bipol[prech]) data+=2047;
	if (rtf_put(prech, (char *) &data, 2) < 0)
	  rtf_put(9, (char *) &prech, sizeof(prech));
      }
      prech=6;
      rt_task_wait();
    }
    if ((channels & 128)==128) {
      data=kanal7(PORT, bipol[7]);
      if (prech>=0) {
	if (bipol[prech] && data>2047) data-=4095;
	if (bipol[prech]) data+=2047;
	if (rtf_put(prech, (char *) &data, 2) < 0)
	  rtf_put(9, (char *) &prech, sizeof(prech));
      }
      prech=7;
      rt_task_wait();
    }
  }
}

int my_handler(unsigned int fifo) {
  struct my_msg_struct msg;
  int err;
  RTIME now;

  /* FIFO 8 transfers commands to control the driver */
  while ((err=rtf_get(8, &msg, sizeof(msg))) == sizeof(msg)) {
    switch(msg.command) {
    case START_TASK:
      now=rt_get_time();
      rt_task_make_periodic(&mytask, now, (RTIME)((unsigned long)RT_TICKS_PER_SEC/(unsigned long)sr));
      break;
    case STOP_TASK:
      rt_task_suspend(&mytask);
      break;
    case SET_CH:
      channels=(unsigned char)msg.value;
      break;
    case SET_POL:
      if ((msg.value & 1)==1) bipol[0]=1; else bipol[0]=0;
      if ((msg.value & 2)==2) bipol[1]=1; else bipol[1]=0;
      if ((msg.value & 4)==4) bipol[2]=1; else bipol[2]=0;
      if ((msg.value & 8)==8) bipol[3]=1; else bipol[3]=0;
      if ((msg.value & 16)==16) bipol[4]=1; else bipol[4]=0;
      if ((msg.value & 32)==32) bipol[5]=1; else bipol[5]=0;
      if ((msg.value & 64)==64) bipol[6]=1; else bipol[6]=0;
      if ((msg.value & 128)==128) bipol[7]=1; else bipol[7]=0;
      break;
    case SET_SR:
      sr=msg.value;
      break;
    default:
      return -EINVAL;
    }
  }
  if (err!=0) {
    return -EINVAL;
  }
  return 0;
}

int init_module(void) {
  int i;
  sr=1;
  channels=0x00;
  for (i=0; i<8; i++) bipol[i]=0;
  rtf_create(0,10000);
  rtf_create(1,10000);
  rtf_create(2,10000);
  rtf_create(3,10000);
  rtf_create(4,10000);
  rtf_create(5,10000);
  rtf_create(6,10000);
  rtf_create(7,10000);
  rtf_create(8,100);
  rtf_create(9,100);
  rt_task_init(&mytask, fun, 0, 4000, 4);
  rtf_create_handler(8, &my_handler);
  return 0;
}


void cleanup_module(void)
{
  rtf_destroy(0);
  rtf_destroy(1);
  rtf_destroy(2);
  rtf_destroy(3);
  rtf_destroy(4);
  rtf_destroy(5);
  rtf_destroy(6);
  rtf_destroy(7);
  rtf_destroy(8);
  rtf_destroy(9);
  rt_task_delete(&mytask);
}


