/* mad.cc  Data Acquisition program for Linux */

#include <Xm/MainW.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeB.h>
#include <Xm/SeparatoG.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>
#include <Xm/DrawingA.h>
#include <iostream.h>
#include <fstream.h>
#include <sys/types.h>
#include <fcntl.h>
#include <stdio.h>
#include <errno.h>
#include "icon.h"

#define DISW 500           // display width
#define DISH 300           // display hight
#define LMAR 50            // left margin for y-axis
#define TICWIDTH 5         // width of the tics of the y-axis
#define UPDATE_TIMER 5     // update of time display in sec.
#define VERSION "2.5"

/* global variables */

typedef struct {
  int   ad_low, ad_high;
  float rw_low, rw_high;
} calib;

typedef struct {
  int   length;      // display length in sec;
  int   xpix;        // number of horizontal pixels in display;
  int   sr;          // sampling rate;
  int   recon;       // program is recording (1) or sleeping (0);
  int   nbuf;        // number of samples read from FIFO and scanned for minmax;
  int   nbuftrue;    // actual size of ch[i].buf, important when window is resized during recording;
  int   nactive;     // number of active channels;
  int   resize;      // window was resized (1);
  int   movavg;      // number of values for movavg;
  int   store;       // save data on file (1) or not (0);
} regist;

typedef struct {
  int   *min;         // array stores min of nbuf samples;
  int   *max;         // array stores max of nbuf samples;
  char  *buf;         // buffer to read from FIFO;
  int   nread;        // number of values in buffer;
  int   first,last;   // first and last element of minmax array;
  short addfirst;     // whether first must be incremented;
  int   xpos;         // x position on that the next sample must be drawn;
  int   active;       // channel is active;
  int   polarity;     // uni (0) or bipolar (1) mode; 
  int   color;        // color of trace (0-7);
  int   height;       // relative height of channel display (0-3);
  calib cal;          // calibration factors;
} channel;

/* Structure for derived channels */
typedef struct {
  int      nch;
  int      threshold;
  int      movavg;
  int      ampli;
  int      *mavg;
  unsigned long masum;	
  channel  ch[7]; // 0:freq, 1:max, 2:mean, 3:min, 4:ampl, 5:dy/dtmin, 6:dy/dtmax;
} der_channel;

/* Structure for menu system */
typedef struct {
  Widget f_menu;
  Widget l_setup;
  Widget s_setup;
  Widget s_data;
  Widget quit;
} f_menu;

typedef struct {
  Widget r_menu;
  Widget s_recording;
  Widget e_recording;
} r_menu;

typedef struct {
  Widget c_menu;
  Widget channel[8];
} c_menu;

typedef struct {
  Widget o_menu;
  Widget srate;
  Widget tracel;
  Widget achannel;
  Widget store;
} o_menu;

typedef struct {
  f_menu file;
  r_menu recording;
  c_menu calibrate;
  o_menu options;
} main_menu;

String colors[] = {"Black", "Blue", "Green", "Red", "Yellow", "Magenta", "Cyan", "White"};

XtAppContext app;
XtWorkProcId work_id;
Widget       drawing_a, mess_label, trace_label, sr_label;
       
ofstream     datei[8], derdat[8];
regist       reg, der;
channel      ch[8];
der_channel  dch[8];
main_menu    mymenu;
int          fd[8], fd8, fd9;

main(int argc, char *argv[])
{
  Pixmap    bitmap;
  Widget    toplevel, main_w, mess_a, rowcol_w;
  Widget    menubar, PullDown;
  XmString  label;
  int       i, j;
  char      s[20];

  fd[0]=open("/dev/rtf0", O_RDONLY | O_NONBLOCK);
  fd[1]=open("/dev/rtf1", O_RDONLY | O_NONBLOCK);
  fd[2]=open("/dev/rtf2", O_RDONLY | O_NONBLOCK);
  fd[3]=open("/dev/rtf3", O_RDONLY | O_NONBLOCK);
  fd[4]=open("/dev/rtf4", O_RDONLY | O_NONBLOCK);
  fd[5]=open("/dev/rtf5", O_RDONLY | O_NONBLOCK);
  fd[6]=open("/dev/rtf6", O_RDONLY | O_NONBLOCK);
  fd[7]=open("/dev/rtf7", O_RDONLY | O_NONBLOCK);
  fd8=open("/dev/rtf8", O_WRONLY);
  fd9=open("/dev/rtf9", O_RDONLY | O_NONBLOCK);

  reg.sr=1000;
  reg.recon=0;
  reg.store=1;
  reg.length=31;
  reg.nactive=0;
  reg.xpix=DISW-LMAR;
  reg.nbuf = reg.length*reg.sr/reg.xpix;
  if (reg.nbuf<1) reg.nbuf=1;
  reg.length=reg.nbuf*reg.xpix/reg.sr;
  reg.nbuftrue=reg.nbuf;
  
  der.length=5;
  der.nactive=0;

  for (i=0; i<8; i++) {
    dch[i].nch=0;
    dch[i].threshold=2048;
    dch[i].movavg=1;
    dch[i].masum=0;
    dch[i].ampli=1;
    dch[i].mavg = new int [dch[i].movavg];
    for (j=0; j<7; j++) {
      dch[i].ch[j].min = new int [1];
      dch[i].ch[j].max = new int [1];
      dch[i].ch[j].nread=0;
      dch[i].ch[j].first=0;
      dch[i].ch[j].last=0;
      dch[i].ch[j].addfirst=0;
      dch[i].ch[j].xpos=0;
      dch[i].ch[j].active=0;
      dch[i].ch[j].polarity=0;
      dch[i].ch[j].color=0;
      dch[i].ch[j].height=0;
      dch[i].ch[j].buf = new char [2];
      dch[i].ch[j].cal.ad_low=0;
      dch[i].ch[j].cal.ad_high=4095;
      dch[i].ch[j].cal.rw_low=0;
      dch[i].ch[j].cal.rw_high=100;
    }
  }

  for (i=0; i<8; i++) {
    ch[i].min = new int [reg.xpix];
    ch[i].max = new int [reg.xpix];
    ch[i].nread=0;
    ch[i].first=0;
    ch[i].last=0;
    ch[i].addfirst=0;
    ch[i].xpos=0;
    ch[i].active=0;
    ch[i].polarity=0;
    ch[i].color=0;
    ch[i].height=0;
    ch[i].buf = new char [2*reg.nbuf];
    ch[i].cal.ad_low=0;
    ch[i].cal.ad_high=4095;
    ch[i].cal.rw_low=0;
    ch[i].cal.rw_high=100;
  }

  extern void f_cb(Widget widget, XtPointer client_data, XtPointer call_data);
  extern void r_cb(Widget widget, XtPointer client_data, XtPointer call_data);
  extern void c_cb(Widget widget, XtPointer client_data, XtPointer call_data);
  extern void o_cb(Widget widget, XtPointer client_data, XtPointer call_data);
  extern void da_cb(Widget widget, XtPointer client_data, XtPointer call_data);
  extern void rs_cb(Widget widget, XtPointer client_data, XtPointer call_data);

  toplevel=XtVaAppInitialize(&app, "Mad", NULL, 0,
			      &argc, argv, NULL, NULL);
  bitmap=XCreatePixmapFromBitmapData(XtDisplay(toplevel),
			  RootWindowOfScreen(XtScreen(toplevel)),
			  (char *)xmad_icon_bits, xmad_icon_width, xmad_icon_height,
			  1,0,1);
  XtVaSetValues(toplevel, 
		XmNiconPixmap, bitmap,
		XmNtitle, "xmad v"VERSION,
		NULL);
  /*   m a i n w i n d o w    */
  main_w=XtVaCreateWidget("main", xmMainWindowWidgetClass, toplevel,
			         NULL);
  /*   m e s s a g e a r e a   */
  mess_a=XtVaCreateWidget("frame", xmFrameWidgetClass, main_w,
				 XmNshadowType, XmSHADOW_OUT,
				 NULL);
  XtVaSetValues(main_w, XmNmessageWindow, mess_a, NULL);
  rowcol_w=XtVaCreateWidget("mess_a", xmRowColumnWidgetClass, mess_a,
			    XmNorientation, XmHORIZONTAL,
			    NULL);
  mess_label=XtVaCreateManagedWidget("mess_l", xmLabelWidgetClass, rowcol_w,
				     XmNlabelString, XmStringCreateLocalized("00:00:00"),
				     XmNrecomputeSize, False,
				     NULL);
  XtVaCreateManagedWidget("trace_t", xmLabelWidgetClass, rowcol_w,
			  XmNlabelString, XmStringCreateLocalized("TL:"),
			  NULL);
  sprintf(s, "%d   ", reg.length);
  trace_label=XtVaCreateManagedWidget("trace_l", xmLabelWidgetClass, rowcol_w,
				      XtVaTypedArg, XmNlabelString, XmRString, s, strlen(s)+1,
				      XmNrecomputeSize, False,
				      NULL);
  XtVaCreateManagedWidget("sr_t", xmLabelWidgetClass, rowcol_w,
			  XmNlabelString, XmStringCreateLocalized("SR:"),
			  NULL);
  sprintf(s, "%d ", reg.sr);
  sr_label=XtVaCreateManagedWidget("sr_l", xmLabelWidgetClass, rowcol_w,
				   XtVaTypedArg, XmNlabelString, XmRString, s, strlen(s)+1,
				   XmNrecomputeSize, False,
				   NULL);
  XtManageChild(rowcol_w);
  XtManageChild(mess_a);
  /*   d r a w i n g a r e a   */
  drawing_a=XtVaCreateManagedWidget("drawing_a",xmDrawingAreaWidgetClass, main_w,
				    XmNwidth, DISW,
				    XmNheight, DISH,
				    NULL);
  XtManageChild(main_w);
  /*   M e n u s y s t e m   */
  menubar = XmCreateMenuBar(main_w, "menubar",NULL,0);
  /*   F i l e m e n u   */;
  PullDown=XmCreatePulldownMenu(menubar, "FilePullDown", NULL,0);
  mymenu.file.f_menu=XtVaCreateManagedWidget("File",
			  xmCascadeButtonWidgetClass, menubar,
			  XmNlabelString, XmStringCreateLocalized("File"),
			  XmNmnemonic, 'F',
			  XmNsubMenuId, PullDown,
			  NULL);
  mymenu.file.l_setup=XtVaCreateManagedWidget("Load Setup",
					      xmPushButtonGadgetClass, PullDown,
					      XmNmnemonic, 'L',
					      NULL);
  XtAddCallback(mymenu.file.l_setup, XmNactivateCallback, f_cb, (XtPointer)0);
  mymenu.file.s_setup=XtVaCreateManagedWidget("Save Setup",
					      xmPushButtonGadgetClass, PullDown,
					      XmNmnemonic, 'S',
					      NULL);
  XtAddCallback(mymenu.file.s_setup, XmNactivateCallback, f_cb, (XtPointer)1);
  XtVaCreateManagedWidget("separator", xmSeparatorGadgetClass, PullDown, NULL);
  mymenu.file.s_data=XtVaCreateManagedWidget("Save Data",
					      xmPushButtonGadgetClass, PullDown,
					      XmNmnemonic, 'D',
					      NULL);
  XtAddCallback(mymenu.file.s_data, XmNactivateCallback, f_cb, (XtPointer)2);
  XtVaCreateManagedWidget("separator", xmSeparatorGadgetClass, PullDown, NULL);
  mymenu.file.quit=XtVaCreateManagedWidget("Quit",
					      xmPushButtonGadgetClass, PullDown,
					      XmNmnemonic, 'Q',
					      NULL);
  XtAddCallback(mymenu.file.quit, XmNactivateCallback, f_cb, (XtPointer)3);

  /*   R e c o r d m e n u   */;
  PullDown=XmCreatePulldownMenu(menubar, "RecordPullDown", NULL,0);
  mymenu.recording.r_menu=XtVaCreateManagedWidget("Record",
			  xmCascadeButtonWidgetClass, menubar,
			  XmNlabelString, XmStringCreateLocalized("Recording"),
			  XmNmnemonic, 'R',
			  XmNsubMenuId, PullDown,
			  NULL);
  mymenu.recording.s_recording=XtVaCreateManagedWidget("Start Recording",
					      xmPushButtonGadgetClass, PullDown,
					      XmNmnemonic, 'S',
					      NULL);
  XtAddCallback(mymenu.recording.s_recording, XmNactivateCallback, r_cb, (XtPointer)0);
  mymenu.recording.e_recording=XtVaCreateManagedWidget("End Recording",
					      xmPushButtonGadgetClass, PullDown,
					      XmNmnemonic, 'E',
					      NULL);
  XtAddCallback(mymenu.recording.e_recording, XmNactivateCallback, r_cb, (XtPointer)1);

  /*   C a l i b r a t e m e n u   */;
  PullDown=XmCreatePulldownMenu(menubar, "CalibratePullDown", NULL,0);
  mymenu.calibrate.c_menu=XtVaCreateManagedWidget("Calibrate",
			  xmCascadeButtonWidgetClass, menubar,
			  XmNlabelString, XmStringCreateLocalized("Calibration"),
			  XmNmnemonic, 'C',
			  XmNsubMenuId, PullDown,
			  NULL);
  mymenu.calibrate.channel[0]=XtVaCreateManagedWidget("Channel 1",
					      xmPushButtonGadgetClass, PullDown,
					      XmNmnemonic, '1',
					      NULL);
  XtAddCallback(mymenu.calibrate.channel[0], XmNactivateCallback, c_cb, (XtPointer)0);
  mymenu.calibrate.channel[1]=XtVaCreateManagedWidget("Channel 2",
					      xmPushButtonGadgetClass, PullDown,
					      XmNmnemonic, '2',
					      NULL);
  XtAddCallback(mymenu.calibrate.channel[1], XmNactivateCallback, c_cb, (XtPointer)1);
  mymenu.calibrate.channel[2]=XtVaCreateManagedWidget("Channel 3",
					      xmPushButtonGadgetClass, PullDown,
					      XmNmnemonic, '3',
					      NULL);
  XtAddCallback(mymenu.calibrate.channel[2], XmNactivateCallback, c_cb, (XtPointer)2);
  mymenu.calibrate.channel[3]=XtVaCreateManagedWidget("Channel 4",
					      xmPushButtonGadgetClass, PullDown,
					      XmNmnemonic, '4',
					      NULL);
  XtAddCallback(mymenu.calibrate.channel[3], XmNactivateCallback, c_cb, (XtPointer)3);
  mymenu.calibrate.channel[4]=XtVaCreateManagedWidget("Channel 5",
					      xmPushButtonGadgetClass, PullDown,
					      XmNmnemonic, '5',
					      NULL);
  XtAddCallback(mymenu.calibrate.channel[4], XmNactivateCallback, c_cb, (XtPointer)4);
  mymenu.calibrate.channel[5]=XtVaCreateManagedWidget("Channel 6",
					      xmPushButtonGadgetClass, PullDown,
					      XmNmnemonic, '6',
					      NULL);
  XtAddCallback(mymenu.calibrate.channel[5], XmNactivateCallback, c_cb, (XtPointer)5);
  mymenu.calibrate.channel[6]=XtVaCreateManagedWidget("Channel 7",
					      xmPushButtonGadgetClass, PullDown,
					      XmNmnemonic, '7',
					      NULL);
  XtAddCallback(mymenu.calibrate.channel[6], XmNactivateCallback, c_cb, (XtPointer)6);
  mymenu.calibrate.channel[7]=XtVaCreateManagedWidget("Channel 8",
					      xmPushButtonGadgetClass, PullDown,
					      XmNmnemonic, '8',
					      NULL);
  XtAddCallback(mymenu.calibrate.channel[7], XmNactivateCallback, c_cb, (XtPointer)7);

  /*   O p t i o n s m e n u   */;
  PullDown=XmCreatePulldownMenu(menubar, "OptionsPullDown", NULL,0);
  mymenu.options.o_menu=XtVaCreateManagedWidget("Options",
			  xmCascadeButtonWidgetClass, menubar,
			  XmNlabelString, XmStringCreateLocalized("Options"),
			  XmNmnemonic, 'O',
			  XmNsubMenuId, PullDown,
			  NULL);
  mymenu.options.srate=XtVaCreateManagedWidget("Sampling Rate",
					      xmPushButtonGadgetClass, PullDown,
					      XmNmnemonic, 'S',
					      NULL);
  XtAddCallback(mymenu.options.srate, XmNactivateCallback, o_cb, (XtPointer)0);
  mymenu.options.tracel=XtVaCreateManagedWidget("Trace Length",
					      xmPushButtonGadgetClass, PullDown,
					      XmNmnemonic, 'T',
					      NULL);
  XtAddCallback(mymenu.options.tracel, XmNactivateCallback, o_cb, (XtPointer)1);
  mymenu.options.achannel=XtVaCreateManagedWidget("Channel Setup",
					      xmPushButtonGadgetClass, PullDown,
					      XmNmnemonic, 'C',
					      NULL);
  XtAddCallback(mymenu.options.achannel, XmNactivateCallback, o_cb, (XtPointer)2);
  XtVaCreateManagedWidget("separator", xmSeparatorGadgetClass, PullDown, NULL);
  mymenu.options.store=XtVaCreateManagedWidget("Monitor Mode",
					       xmToggleButtonGadgetClass, PullDown,
					       XmNmnemonic, 'M',
					       NULL);
  XtAddCallback(mymenu.options.store, XmNvalueChangedCallback, o_cb, (XtPointer)3);

  XtSetSensitive(mymenu.file.s_data,False);
  XtSetSensitive(mymenu.recording.s_recording,False);
  XtSetSensitive(mymenu.recording.e_recording,False);
  for (i=0; i<8; i++) XtSetSensitive(mymenu.calibrate.channel[i],False);
  XtManageChild(menubar);

  XmMainWindowSetAreas(main_w, menubar, NULL, NULL, NULL, drawing_a);
  XtAddCallback(drawing_a, XmNexposeCallback, da_cb, NULL);
  XtAddCallback(drawing_a, XmNresizeCallback, rs_cb, NULL);

  XtRealizeWidget(toplevel);
  XtAppMainLoop(app);
}
